@@ -1,8 +1,8 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
@@ -32,7 +32,7 @@
 #define MAX(x, y)	(((x) > (y))?(x):(y))
 #endif
 
-#define DEFAULT_NUM_SLAVES 3    /* default number of children to spawn */
+#define DEFAULT_NUM_CHILDREN 3  /* default number of children to spawn */
 #define DEFAULT_PORT 7470       /* default port to listen on */
 #define NOVALUE 99999   /* indicates a parameter is as of yet unspecified */
 #define MAX_ITERATIONS 10000    /* maximum 'depth' to look for mandelbrot value */
@@ -170,8 +170,8 @@ int main(int argc, char *argv[])
     int listener;
     int save_image = 0;
     int optval;
-    int num_children = DEFAULT_NUM_SLAVES;
-    int master = 1;
+    int num_children = DEFAULT_NUM_CHILDREN;
+    int parent = 1;
     MPI_Comm parent, *child_comm = NULL;
     MPI_Request *child_request = NULL;
     int error_code, error;
@@ -190,7 +190,7 @@ int main(int argc, char *argv[])
 
     if (parent == MPI_COMM_NULL) {
         if (numprocs > 1) {
-            printf("Error: only one process allowed for the master.\n");
+            printf("Error: only one process allowed for the parent.\n");
             PrintUsage();
             error_code = MPI_Abort(MPI_COMM_WORLD, -1);
             exit(error_code);
@@ -198,7 +198,7 @@ int main(int argc, char *argv[])
 
         printf("Welcome to the Mandelbrot/Julia set explorer.\n");
 
-        master = 1;
+        parent = 1;
 
         /* Get inputs-- region to view (must be within x/ymin to x/ymax, make sure
          * xmax>xmin and ymax>ymin) and resolution (number of pixels along an edge,
@@ -216,18 +216,18 @@ int main(int argc, char *argv[])
         if (julia == 1) /* we're doing a julia figure */
             check_julia_params(&julia_constant.real, &julia_constant.imaginary);
 
-        /* spawn slaves */
+        /* spawn children */
         child_comm = (MPI_Comm *) malloc(num_children * sizeof(MPI_Comm));
         child_request = (MPI_Request *) malloc(num_children * sizeof(MPI_Request));
         result = (header_t *) malloc(num_children * sizeof(header_t));
         if (child_comm == NULL || child_request == NULL || result == NULL) {
             printf
-                ("Error: unable to allocate an array of %d communicators, requests and work objects for the slaves.\n",
+                ("Error: unable to allocate an array of %d communicators, requests and work objects for the children.\n",
                  num_children);
             error_code = MPI_Abort(MPI_COMM_WORLD, -1);
             exit(error_code);
         }
-        printf("Spawning %d slaves.\n", num_children);
+        printf("Spawning %d children.\n", num_children);
         for (i = 0; i < num_children; i++) {
             error = MPI_Comm_spawn(argv[0], MPI_ARGV_NULL, 1,
                                    MPI_INFO_NULL, 0, MPI_COMM_WORLD, &child_comm[i], &error_code);
@@ -254,7 +254,7 @@ int main(int argc, char *argv[])
             MPI_Send(&alternate_equation, 1, MPI_INT, 0, 0, child_comm[i]);
         }
     } else {
-        master = 0;
+        parent = 0;
         MPI_Recv(&num_colors, 1, MPI_INT, 0, 0, parent, MPI_STATUS_IGNORE);
         MPI_Recv(&imax_iterations, 1, MPI_INT, 0, 0, parent, MPI_STATUS_IGNORE);
         MPI_Recv(&ipixels_across, 1, MPI_INT, 0, 0, parent, MPI_STATUS_IGNORE);
@@ -266,7 +266,7 @@ int main(int argc, char *argv[])
         MPI_Recv(&alternate_equation, 1, MPI_INT, 0, 0, parent, MPI_STATUS_IGNORE);
     }
 
-    if (master) {
+    if (parent) {
         colors = malloc((num_colors + 1) * sizeof(color_t));
         if (colors == NULL) {
             MPI_Abort(MPI_COMM_WORLD, -1);
@@ -283,7 +283,7 @@ int main(int argc, char *argv[])
         exit(-1);
     }
 
-    if (master) {
+    if (parent) {
         int istep, jstep;
         int i1[400], i2[400], j1[400], j2[400];
         int ii, jj;
@@ -445,7 +445,7 @@ int main(int argc, char *argv[])
             while (k < 400) {
                 MPI_Waitany(num_children, child_request, &index, &status);
                 memcpy(work, &result[index], 5 * sizeof(int));
-                /*printf("master receiving data in k<400 loop.\n");fflush(stdout); */
+                /*printf("parent receiving data in k<400 loop.\n");fflush(stdout); */
                 MPI_Recv(in_grid_array, (work[2] + 1 - work[1]) * (work[4] + 1 - work[3]), MPI_INT,
                          0, 201, child_comm[index], &status);
                 /* draw data */
@@ -466,7 +466,7 @@ int main(int argc, char *argv[])
             for (i = 0; i < num_children; i++) {
                 MPI_Wait(&child_request[i], &status);
                 memcpy(work, &result[i], 5 * sizeof(int));
-                /*printf("master receiving data in tail loop.\n");fflush(stdout); */
+                /*printf("parent receiving data in tail loop.\n");fflush(stdout); */
                 MPI_Recv(in_grid_array, (work[2] + 1 - work[1]) * (work[4] + 1 - work[3]), MPI_INT,
                          0, 201, child_comm[i], &status);
                 /* draw data */
@@ -491,17 +491,17 @@ int main(int argc, char *argv[])
         }
     } else {
         for (;;) {
-            /*printf("slave[%d] receiveing bounds.\n", pid);fflush(stdout); */
+            /*printf("child[%d] receiveing bounds.\n", pid);fflush(stdout); */
             MPI_Recv(&x_min, 1, MPI_DOUBLE, 0, 0, parent, MPI_STATUS_IGNORE);
             MPI_Recv(&x_max, 1, MPI_DOUBLE, 0, 0, parent, MPI_STATUS_IGNORE);
             MPI_Recv(&y_min, 1, MPI_DOUBLE, 0, 0, parent, MPI_STATUS_IGNORE);
             MPI_Recv(&y_max, 1, MPI_DOUBLE, 0, 0, parent, MPI_STATUS_IGNORE);
             MPI_Recv(&imax_iterations, 1, MPI_INT, 0, 0, parent, MPI_STATUS_IGNORE);
-            /*printf("slave[%d] received bounding box: (%f,%f)(%f,%f)\n", pid, x_min, y_min, x_max, y_max);fflush(stdout); */
+            /*printf("child[%d] received bounding box: (%f,%f)(%f,%f)\n", pid, x_min, y_min, x_max, y_max);fflush(stdout); */
 
             /* check for the end condition */
             if (x_min == x_max && y_min == y_max) {
-                /*printf("slave[%d] done.\n", pid);fflush(stdout); */
+                /*printf("child[%d] done.\n", pid);fflush(stdout); */
                 break;
             }
 
@@ -509,7 +509,7 @@ int main(int argc, char *argv[])
             y_resolution = (y_max - y_min) / ((double) ipixels_down);
 
             MPI_Recv(work, 5, MPI_INT, 0, 100, parent, &status);
-            /*printf("slave[%d] received work: %d, (%d,%d)(%d,%d)\n", pid, work[0], work[1], work[2], work[3], work[4]);fflush(stdout); */
+            /*printf("child[%d] received work: %d, (%d,%d)(%d,%d)\n", pid, work[0], work[1], work[2], work[3], work[4]);fflush(stdout); */
             while (work[0] != 0) {
                 imin = work[1];
                 imax = work[2];
@@ -552,20 +552,20 @@ int main(int argc, char *argv[])
                     }
                 }
                 /* send the result to the root */
-                /*printf("slave[%d] sending work %d back.\n", pid, work[0]);fflush(stdout); */
+                /*printf("child[%d] sending work %d back.\n", pid, work[0]);fflush(stdout); */
                 MPI_Send(work, 5, MPI_INT, 0, 200, parent);
-                /*printf("slave[%d] sending work %d data.\n", pid, work[0]);fflush(stdout); */
+                /*printf("child[%d] sending work %d data.\n", pid, work[0]);fflush(stdout); */
                 MPI_Send(in_grid_array, (work[2] + 1 - work[1]) * (work[4] + 1 - work[3]), MPI_INT,
                          0, 201, parent);
                 /* get the next piece of work */
-                /*printf("slave[%d] receiving new work.\n", pid);fflush(stdout); */
+                /*printf("child[%d] receiving new work.\n", pid);fflush(stdout); */
                 MPI_Recv(work, 5, MPI_INT, 0, 100, parent, &status);
-                /*printf("slave[%d] received work: %d, (%d,%d)(%d,%d)\n", pid, work[0], work[1], work[2], work[3], work[4]);fflush(stdout); */
+                /*printf("child[%d] received work: %d, (%d,%d)(%d,%d)\n", pid, work[0], work[1], work[2], work[3], work[4]);fflush(stdout); */
             }
         }
     }
 
-    if (master && save_image) {
+    if (parent && save_image) {
         imax_iterations = 0;
         for (i = 0; i < ipixels_across * ipixels_down; ++i) {
             /* look for "brightest" pixel value, for image use */
@@ -595,7 +595,7 @@ int main(int argc, char *argv[])
                   file_message, num_colors, colors);
     }
 
-    if (master) {
+    if (parent) {
         for (i = 0; i < num_children; i++) {
             MPI_Comm_disconnect(&child_comm[i]);
         }
@@ -612,7 +612,7 @@ void PrintUsage(void)
 {
     printf("usage: mpiexec -n 1 pmandel [options]\n");
     printf
-        ("options:\n -n # slaves\n -xmin # -xmax #\n -ymin # -ymax #\n -depth #\n -xscale # -yscale #\n -out filename\n -i\n");
+        ("options:\n -n # children\n -xmin # -xmax #\n -ymin # -ymax #\n -depth #\n -xscale # -yscale #\n -out filename\n -i\n");
     printf("All options are optional.\n");
     printf
         ("-i will allow you to input the min/max parameters from stdin and output the resulting image to a ppm file.");
@@ -832,7 +832,7 @@ void read_mand_args(int argc, char *argv[], int *o_max_iterations,
     *o_divergent_limit = INFINITE_LIMIT;        /* default total range is assumed
                                                  * if not explicitly overwritten */
 
-    *num_children = DEFAULT_NUM_SLAVES;
+    *num_children = DEFAULT_NUM_CHILDREN;
 
     /* We just cycle through all given parameters, matching what we can.
      * Note that we force casting, because we expect that a nonsensical
@@ -976,12 +976,12 @@ void check_mand_params(int *m_max_iterations,
     }
 
     if (*m_num_children < 1) {
-        printf("Error, invalid number of slaves (%d), setting to %d\n", *m_num_children,
-               DEFAULT_NUM_SLAVES);
-        *m_num_children = DEFAULT_NUM_SLAVES;
+        printf("Error, invalid number of children (%d), setting to %d\n", *m_num_children,
+               DEFAULT_NUM_CHILDREN);
+        *m_num_children = DEFAULT_NUM_CHILDREN;
     }
     if (*m_num_children > 400) {
-        printf("Error, number of slaves (%d) exceeds the maximum, setting to 400.\n",
+        printf("Error, number of children (%d) exceeds the maximum, setting to 400.\n",
                *m_num_children);
         *m_num_children = 400;
     }
