@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2009 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 /* This header file contains constants that might end up on the right hand side
@@ -34,7 +33,14 @@
 #define MPICH_THREAD_GRANULARITY__LOCKFREE 3
 /* _SINGLE is the "null" granularity, where all processes are single-threaded */
 #define MPICH_THREAD_GRANULARITY__SINGLE 4
-#define MPICH_THREAD_GRANULARITY__VNI 5
+#define MPICH_THREAD_GRANULARITY__VCI 5
+
+/* Define hashing method to map VCI */
+#define MPICH_VCI__ZERO 0       /* vci === 0 */
+#define MPICH_VCI__COMM 1       /* vci stored in communicator structure */
+#define MPICH_VCI__TAG 2        /* vcis parsed from tag */
+#define MPICH_VCI__IMPLICIT 3   /* vci from (comm, rank, tag), taking account of hints */
+#define MPICH_VCI__EXPLICIT 4   /* vci passed down explicitly via parameters (MPIX_xxx or Endpoint rank) */
 
 /* _NONE means no concurrency control, such as when using MPI_THREAD_SINGLE */
 #define MPICH_REFCOUNT__NONE 1
@@ -55,8 +61,9 @@
 
 /* Possible values for process state */
 #define MPICH_MPI_STATE__PRE_INIT 0
-#define MPICH_MPI_STATE__IN_INIT 1
-#define MPICH_MPI_STATE__POST_INIT 2
-#define MPICH_MPI_STATE__POST_FINALIZED 3
+#define MPICH_MPI_STATE__POST_INIT 1
+#define MPICH_MPI_STATE__POST_FINALIZED 2
+/* Internal init states. */
+#define MPICH_MPI_STATE__IN_INIT 3      /* can call MPIR_Err_return_comm(...) on error */
 
 #endif /* MPICHCONFCONST_H_INCLUDED */
