@@ -1,8 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
- *
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #ifndef MPIR_STATUS_H_INCLUDED
@@ -38,6 +36,13 @@
 
 #define MPIR_STATUS_GET_CANCEL_BIT(status_) ((status_).count_hi_and_cancelled & 1)
 
+/* Same as MPIR_STATUS_SET_CANCEL_BIT, but check MPI_STATUS_IGNORE as its family. */
+#define MPIR_Status_set_cancel_bit(status_, cancelled_)         \
+    {                                                           \
+        if ((status_) != MPI_STATUS_IGNORE)                     \
+            MPIR_STATUS_SET_CANCEL_BIT(*(status_), cancelled_)  \
+    }
+
 /* Do not set MPI_ERROR (only set if ERR_IN_STATUS is returned */
 #define MPIR_Status_set_empty(status_)                          \
     {                                                           \
