@@ -1,8 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -27,10 +25,6 @@ int MPI_Type_get_attr(MPI_Datatype datatype, int type_keyval, void *attribute_va
 #undef MPI_Type_get_attr
 #define MPI_Type_get_attr PMPI_Type_get_attr
 
-#undef FUNCNAME
-#define FUNCNAME MPII_Type_get_attr
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPII_Type_get_attr(MPI_Datatype datatype, int type_keyval, void *attribute_val,
                        int *flag, MPIR_Attr_type outAttrType)
 {
@@ -51,16 +45,6 @@ int MPII_Type_get_attr(MPI_Datatype datatype, int type_keyval, void *attribute_v
         {
             MPIR_ERRTEST_DATATYPE(datatype, "datatype", mpi_errno);
             MPIR_ERRTEST_KEYVAL(type_keyval, MPIR_DATATYPE, "datatype", mpi_errno);
-#ifdef NEEDS_POINTER_ALIGNMENT_ADJUST
-            /* A common user error is to pass the address of a 4-byte
-             * int when the address of a pointer (or an address-sized int)
-             * should have been used.  We can test for this specific
-             * case.  Note that this code assumes sizeof(intptr_t) is
-             * a power of 2. */
-            if ((intptr_t) attribute_val & (sizeof(intptr_t) - 1)) {
-                MPIR_ERR_SETANDSTMT(mpi_errno, MPI_ERR_ARG, goto fn_fail, "**attrnotptr");
-            }
-#endif
         }
         MPID_END_ERROR_CHECKS;
     }
@@ -135,21 +119,17 @@ int MPII_Type_get_attr(MPI_Datatype datatype, int type_keyval, void *attribute_v
   fn_fail:
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpir_type_get_attr", "**mpir_type_get_attr %D %d %p %p",
                                  datatype, type_keyval, attribute_val, flag);
     }
-    mpi_errno = MPIR_Err_return_comm(NULL, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(NULL, __func__, mpi_errno);
     goto fn_exit;
 #endif
     /* --END ERROR HANDLING-- */
 }
 #endif
 
-#undef FUNCNAME
-#define FUNCNAME MPI_Type_get_attr
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 
 /*@
    MPI_Type_get_attr - Retrieves attribute value by key
@@ -206,12 +186,12 @@ int MPI_Type_get_attr(MPI_Datatype datatype, int type_keyval, void *attribute_va
 #ifdef HAVE_ERROR_CHECKING
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpi_type_get_attr", "**mpi_type_get_attr %D %d %p %p", datatype,
                                  type_keyval, attribute_val, flag);
     }
 #endif
-    mpi_errno = MPIR_Err_return_comm(NULL, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(NULL, __func__, mpi_errno);
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
