@@ -1,8 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -18,10 +16,6 @@
 
    At the moment this function always scatters a buffer of nbytes starting at
    tmp_buf address. */
-#undef FUNCNAME
-#define FUNCNAME MPII_Scatter_for_bcast
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPII_Scatter_for_bcast(void *buffer ATTRIBUTE((unused)),
                            int count ATTRIBUTE((unused)),
                            MPI_Datatype datatype ATTRIBUTE((unused)),
