@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2010 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -14,7 +13,7 @@ cvars:
       category    : COLLECTIVE
       type        : int
       default     : 2
-      class       : device
+      class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : >-
@@ -22,42 +21,45 @@ cvars:
 
     - name        : MPIR_CVAR_IBARRIER_INTRA_ALGORITHM
       category    : COLLECTIVE
-      type        : string
+      type        : enum
       default     : auto
-      class       : device
+      class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : |-
         Variable to select ibarrier algorithm
-        auto               - Internal algorithm selection
-        recursive_doubling - Force recursive doubling algorithm
-        recexch            - Force generic transport based recursive exchange algorithm
+        auto - Internal algorithm selection (can be overridden with MPIR_CVAR_COLL_SELECTION_TUNING_JSON_FILE)
+        sched_auto - Internal algorithm selection for sched-based algorithms
+        sched_recursive_doubling - Force recursive doubling algorithm
+        gentran_recexch          - Force generic transport based recursive exchange algorithm
 
     - name        : MPIR_CVAR_IBARRIER_INTER_ALGORITHM
       category    : COLLECTIVE
-      type        : string
+      type        : enum
       default     : auto
-      class       : device
+      class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : |-
         Variable to select ibarrier algorithm
-        auto  - Internal algorithm selection
-        bcast - Force bcast algorithm
+        auto - Internal algorithm selection (can be overridden with MPIR_CVAR_COLL_SELECTION_TUNING_JSON_FILE)
+        sched_auto - Internal algorithm selection for sched-based algorithms
+        sched_bcast - Force bcast algorithm
 
     - name        : MPIR_CVAR_IBARRIER_DEVICE_COLLECTIVE
       category    : COLLECTIVE
       type        : boolean
       default     : true
-      class       : device
+      class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : >-
-        If set to true, MPI_Ibarrier will allow the device to override the
-        MPIR-level collective algorithms. The device still has the
-        option to call the MPIR-level algorithms manually.
-        If set to false, the device-level ibarrier function will not be
-        called.
+        This CVAR is only used when MPIR_CVAR_DEVICE_COLLECTIVES
+        is set to "percoll".  If set to true, MPI_Ibarrier will
+        allow the device to override the MPIR-level collective
+        algorithms.  The device might still call the MPIR-level
+        algorithms manually.  If set to false, the device-override
+        will be disabled.
 
 === END_MPI_T_CVAR_INFO_BLOCK ===
 */
@@ -83,97 +85,90 @@ int MPI_Ibarrier(MPI_Comm comm, MPI_Request * request)
 
 /* any non-MPI functions go here, especially non-static ones */
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Ibarrier_sched_intra_auto
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Ibarrier_sched_intra_auto(MPIR_Comm * comm_ptr, MPIR_Sched_t s)
+
+int MPIR_Ibarrier_allcomm_auto(MPIR_Comm * comm_ptr, MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
 
-    mpi_errno = MPIR_Ibarrier_sched_intra_recursive_doubling(comm_ptr, s);
+    MPIR_Csel_coll_sig_s coll_sig = {
+        .coll_type = MPIR_CSEL_COLL_TYPE__IBARRIER,
+        .comm_ptr = comm_ptr,
+    };
+
+    MPII_Csel_container_s *cnt = MPIR_Csel_search(comm_ptr->csel_comm, coll_sig);
+    MPIR_Assert(cnt);
+
+    switch (cnt->id) {
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Ibarrier_intra_sched_auto:
+            MPII_SCHED_WRAPPER_EMPTY(MPIR_Ibarrier_intra_sched_auto, comm_ptr, request);
+            break;
+
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Ibarrier_intra_sched_recursive_doubling:
+            MPII_SCHED_WRAPPER_EMPTY(MPIR_Ibarrier_intra_sched_recursive_doubling, comm_ptr,
+                                     request);
+            break;
+
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Ibarrier_intra_gentran_recexch:
+            mpi_errno =
+                MPIR_Ibarrier_intra_gentran_recexch(comm_ptr,
+                                                    cnt->u.ibarrier.intra_gentran_recexch.k,
+                                                    request);
+            break;
+
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Ibarrier_inter_sched_auto:
+            MPII_SCHED_WRAPPER_EMPTY(MPIR_Ibarrier_inter_sched_auto, comm_ptr, request);
+            break;
+
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Ibarrier_inter_sched_bcast:
+            MPII_SCHED_WRAPPER_EMPTY(MPIR_Ibarrier_inter_sched_bcast, comm_ptr, request);
+            break;
+
+        default:
+            MPIR_Assert(0);
+    }
+
+  fn_exit:
+    return mpi_errno;
+  fn_fail:
+    goto fn_exit;
+}
+
+int MPIR_Ibarrier_intra_sched_auto(MPIR_Comm * comm_ptr, MPIR_Sched_t s)
+{
+    int mpi_errno = MPI_SUCCESS;
+
+    mpi_errno = MPIR_Ibarrier_intra_sched_recursive_doubling(comm_ptr, s);
 
     return mpi_errno;
 }
 
 /* It will choose between several different algorithms based on the given
  * parameters. */
-#undef FUNCNAME
-#define FUNCNAME MPIR_Ibarrier_sched_inter_auto
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Ibarrier_sched_inter_auto(MPIR_Comm * comm_ptr, MPIR_Sched_t s)
+int MPIR_Ibarrier_inter_sched_auto(MPIR_Comm * comm_ptr, MPIR_Sched_t s)
 {
     int mpi_errno;
 
-    mpi_errno = MPIR_Ibarrier_sched_inter_bcast(comm_ptr, s);
+    mpi_errno = MPIR_Ibarrier_inter_sched_bcast(comm_ptr, s);
 
     return mpi_errno;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Ibarrier_sched_impl
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Ibarrier_sched_impl(MPIR_Comm * comm_ptr, MPIR_Sched_t s)
+int MPIR_Ibarrier_sched_auto(MPIR_Comm * comm_ptr, MPIR_Sched_t s)
 {
     int mpi_errno = MPI_SUCCESS;
 
     if (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) {
-        /* intracommunicator */
-        switch (MPIR_Ibarrier_intra_algo_choice) {
-            case MPIR_IBARRIER_INTRA_ALGO_RECURSIVE_DOUBLING:
-                mpi_errno = MPIR_Ibarrier_sched_intra_recursive_doubling(comm_ptr, s);
-                break;
-            case MPIR_IBARRIER_INTRA_ALGO_AUTO:
-                MPL_FALLTHROUGH;
-            default:
-                mpi_errno = MPIR_Ibarrier_sched_intra_auto(comm_ptr, s);
-                break;
-        }
+        mpi_errno = MPIR_Ibarrier_intra_sched_auto(comm_ptr, s);
     } else {
-        /* intercommunicator */
-        switch (MPIR_Ibarrier_inter_algo_choice) {
-            case MPIR_IBARRIER_INTER_ALGO_BCAST:
-                mpi_errno = MPIR_Ibarrier_sched_inter_bcast(comm_ptr, s);
-                break;
-            case MPIR_IBARRIER_INTER_ALGO_AUTO:
-                MPL_FALLTHROUGH;
-            default:
-                mpi_errno = MPIR_Ibarrier_sched_inter_auto(comm_ptr, s);
-                break;
-        }
+        mpi_errno = MPIR_Ibarrier_inter_sched_auto(comm_ptr, s);
     }
 
     return mpi_errno;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Ibarrier_sched
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Ibarrier_sched(MPIR_Comm * comm_ptr, MPIR_Sched_t s)
-{
-    int mpi_errno = MPI_SUCCESS;
-
-    if (MPIR_CVAR_IBARRIER_DEVICE_COLLECTIVE && MPIR_CVAR_DEVICE_COLLECTIVES) {
-        mpi_errno = MPID_Ibarrier_sched(comm_ptr, s);
-    } else {
-        mpi_errno = MPIR_Ibarrier_sched_impl(comm_ptr, s);
-    }
-
-    return mpi_errno;
-}
-
-#undef FUNCNAME
-#define FUNCNAME MPIR_Ibarrier_impl
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIR_Ibarrier_impl(MPIR_Comm * comm_ptr, MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
-    int tag = -1;
-    MPIR_Sched_t s = MPIR_SCHED_NULL;
 
     *request = NULL;
 
@@ -184,35 +179,50 @@ int MPIR_Ibarrier_impl(MPIR_Comm * comm_ptr, MPIR_Request ** request)
      * will require sufficient performance testing and replacement algorithms. */
     if (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) {
         /* intracommunicator */
-        switch (MPIR_Ibarrier_intra_algo_choice) {
-            case MPIR_IBARRIER_INTRA_ALGO_GENTRAN_RECEXCH:
-                mpi_errno = MPIR_Ibarrier_intra_recexch(comm_ptr, request);
+        switch (MPIR_CVAR_IBARRIER_INTRA_ALGORITHM) {
+            case MPIR_CVAR_IBARRIER_INTRA_ALGORITHM_gentran_recexch:
+                mpi_errno =
+                    MPIR_Ibarrier_intra_gentran_recexch(comm_ptr, MPIR_CVAR_IBARRIER_RECEXCH_KVAL,
+                                                        request);
 
-                if (mpi_errno)
-                    MPIR_ERR_POP(mpi_errno);
-                goto fn_exit;
                 break;
+
+            case MPIR_CVAR_IBARRIER_INTRA_ALGORITHM_sched_recursive_doubling:
+                MPII_SCHED_WRAPPER_EMPTY(MPIR_Ibarrier_intra_sched_recursive_doubling, comm_ptr,
+                                         request);
+                break;
+
+            case MPIR_CVAR_IBARRIER_INTRA_ALGORITHM_sched_auto:
+                MPII_SCHED_WRAPPER_EMPTY(MPIR_Ibarrier_intra_sched_auto, comm_ptr, request);
+                break;
+
+            case MPIR_CVAR_IBARRIER_INTRA_ALGORITHM_auto:
+                mpi_errno = MPIR_Ibarrier_allcomm_auto(comm_ptr, request);
+                break;
+
             default:
-                /* go down to the MPIR_Sched-based algorithms */
+                MPIR_Assert(0);
+        }
+    } else {
+        switch (MPIR_CVAR_IBARRIER_INTER_ALGORITHM) {
+            case MPIR_CVAR_IBARRIER_INTER_ALGORITHM_sched_bcast:
+                MPII_SCHED_WRAPPER_EMPTY(MPIR_Ibarrier_inter_sched_bcast, comm_ptr, request);
                 break;
+
+            case MPIR_CVAR_IBARRIER_INTER_ALGORITHM_sched_auto:
+                MPII_SCHED_WRAPPER_EMPTY(MPIR_Ibarrier_inter_sched_auto, comm_ptr, request);
+                break;
+
+            case MPIR_CVAR_IBARRIER_INTER_ALGORITHM_auto:
+                mpi_errno = MPIR_Ibarrier_allcomm_auto(comm_ptr, request);
+                break;
+
+            default:
+                MPIR_Assert(0);
         }
     }
-    if (comm_ptr->local_size != 1 || comm_ptr->comm_kind == MPIR_COMM_KIND__INTERCOMM) {
-        mpi_errno = MPIR_Sched_next_tag(comm_ptr, &tag);
-        if (mpi_errno)
-            MPIR_ERR_POP(mpi_errno);
-        mpi_errno = MPIR_Sched_create(&s);
-        if (mpi_errno)
-            MPIR_ERR_POP(mpi_errno);
 
-        mpi_errno = MPIR_Ibarrier_sched(comm_ptr, s);
-        if (mpi_errno)
-            MPIR_ERR_POP(mpi_errno);
-
-        mpi_errno = MPIR_Sched_start(&s, comm_ptr, tag, request);
-        if (mpi_errno)
-            MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
     return mpi_errno;
@@ -220,15 +230,13 @@ int MPIR_Ibarrier_impl(MPIR_Comm * comm_ptr, MPIR_Request ** request)
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Ibarrier
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIR_Ibarrier(MPIR_Comm * comm_ptr, MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
 
-    if (MPIR_CVAR_IBARRIER_DEVICE_COLLECTIVE && MPIR_CVAR_DEVICE_COLLECTIVES) {
+    if ((MPIR_CVAR_DEVICE_COLLECTIVES == MPIR_CVAR_DEVICE_COLLECTIVES_all) ||
+        ((MPIR_CVAR_DEVICE_COLLECTIVES == MPIR_CVAR_DEVICE_COLLECTIVES_percoll) &&
+         MPIR_CVAR_IBARRIER_DEVICE_COLLECTIVE)) {
         mpi_errno = MPID_Ibarrier(comm_ptr, request);
     } else {
         mpi_errno = MPIR_Ibarrier_impl(comm_ptr, request);
@@ -239,10 +247,6 @@ int MPIR_Ibarrier(MPIR_Comm * comm_ptr, MPIR_Request ** request)
 
 #endif /* MPICH_MPI_FROM_PMPI */
 
-#undef FUNCNAME
-#define FUNCNAME MPI_Ibarrier
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 /*@
 MPI_Ibarrier - Notifies the process that it has reached the barrier and returns
                immediately
@@ -313,8 +317,7 @@ int MPI_Ibarrier(MPI_Comm comm, MPI_Request * request)
     /* ... body of routine ...  */
 
     mpi_errno = MPIR_Ibarrier(comm_ptr, &request_ptr);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
     /* create a complete request, if needed */
     if (!request_ptr)
@@ -334,11 +337,11 @@ int MPI_Ibarrier(MPI_Comm comm, MPI_Request * request)
 #ifdef HAVE_ERROR_CHECKING
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpi_ibarrier", "**mpi_ibarrier %C %p", comm, request);
     }
 #endif
-    mpi_errno = MPIR_Err_return_comm(comm_ptr, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(comm_ptr, __func__, mpi_errno);
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
