@@ -1,16 +1,11 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2017 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
 #include "ibcast.h"
 
-#undef FUNCNAME
-#define FUNCNAME MPII_Ibcast_sched_test_length
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPII_Ibcast_sched_test_length(MPIR_Comm * comm, int tag, void *state)
 {
     int mpi_errno = MPI_SUCCESS;
@@ -20,7 +15,7 @@ int MPII_Ibcast_sched_test_length(MPIR_Comm * comm, int tag, void *state)
     MPIR_Get_count_impl(&ibcast_state->status, MPI_BYTE, &recv_size);
     if (ibcast_state->n_bytes != recv_size || ibcast_state->status.MPI_ERROR != MPI_SUCCESS) {
         mpi_errno = MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE,
-                                         FCNAME, __LINE__, MPI_ERR_OTHER,
+                                         __func__, __LINE__, MPI_ERR_OTHER,
                                          "**collective_size_mismatch",
                                          "**collective_size_mismatch %d %d", ibcast_state->n_bytes,
                                          recv_size);
@@ -30,10 +25,6 @@ int MPII_Ibcast_sched_test_length(MPIR_Comm * comm, int tag, void *state)
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPII_Ibcast_sched_test_curr_length
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPII_Ibcast_sched_test_curr_length(MPIR_Comm * comm, int tag, void *state)
 {
     int mpi_errno = MPI_SUCCESS;
@@ -41,7 +32,7 @@ int MPII_Ibcast_sched_test_curr_length(MPIR_Comm * comm, int tag, void *state)
 
     if (ibcast_state->n_bytes != ibcast_state->curr_bytes) {
         mpi_errno = MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE,
-                                         FCNAME, __LINE__, MPI_ERR_OTHER,
+                                         __func__, __LINE__, MPI_ERR_OTHER,
                                          "**collective_size_mismatch",
                                          "**collective_size_mismatch %d %d", ibcast_state->n_bytes,
                                          ibcast_state->curr_bytes);
@@ -51,10 +42,6 @@ int MPII_Ibcast_sched_test_curr_length(MPIR_Comm * comm, int tag, void *state)
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPII_Ibcast_sched_add_length
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPII_Ibcast_sched_add_length(MPIR_Comm * comm, int tag, void *state)
 {
     int mpi_errno = MPI_SUCCESS;
@@ -75,10 +62,6 @@ int MPII_Ibcast_sched_add_length(MPIR_Comm * comm, int tag, void *state)
 /* This is a binomial scatter operation, but it does *not* take
  * typical scatter arguments.  At the moment this function always
  * scatters a buffer of nbytes starting at tmp_buf address. */
-#undef FUNCNAME
-#define FUNCNAME MPII_Iscatter_for_bcast_sched
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPII_Iscatter_for_bcast_sched(void *tmp_buf, int root, MPIR_Comm * comm_ptr, int nbytes,
                                   MPIR_Sched_t s)
 {
@@ -122,8 +105,7 @@ int MPII_Iscatter_for_bcast_sched(void *tmp_buf, int root, MPIR_Comm * comm_ptr,
             if (recv_size > 0) {
                 mpi_errno = MPIR_Sched_recv(((char *) tmp_buf + relative_rank * scatter_size),
                                             recv_size, MPI_BYTE, src, comm_ptr, s);
-                if (mpi_errno)
-                    MPIR_ERR_POP(mpi_errno);
+                MPIR_ERR_CHECK(mpi_errno);
                 MPIR_SCHED_BARRIER(s);
             }
             break;
@@ -148,8 +130,7 @@ int MPII_Iscatter_for_bcast_sched(void *tmp_buf, int root, MPIR_Comm * comm_ptr,
                 mpi_errno =
                     MPIR_Sched_send(((char *) tmp_buf + scatter_size * (relative_rank + mask)),
                                     send_size, MPI_BYTE, dst, comm_ptr, s);
-                if (mpi_errno)
-                    MPIR_ERR_POP(mpi_errno);
+                MPIR_ERR_CHECK(mpi_errno);
 
                 curr_size -= send_size;
             }
