@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2010 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -12,41 +11,45 @@
 cvars:
     - name        : MPIR_CVAR_ISCATTERV_INTRA_ALGORITHM
       category    : COLLECTIVE
-      type        : string
+      type        : enum
       default     : auto
-      class       : device
+      class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : |-
         Variable to select iscatterv algorithm
-        auto   - Internal algorithm selection
-        linear - Force linear algorithm
+        auto - Internal algorithm selection (can be overridden with MPIR_CVAR_COLL_SELECTION_TUNING_JSON_FILE)
+        sched_auto - Internal algorithm selection for sched-based algorithms
+        sched_linear    - Force linear algorithm
+        gentran_linear  - Force generic transport based linear algorithm
 
     - name        : MPIR_CVAR_ISCATTERV_INTER_ALGORITHM
       category    : COLLECTIVE
-      type        : string
+      type        : enum
       default     : auto
-      class       : device
+      class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : |-
         Variable to select iscatterv algorithm
-        auto   - Internal algorithm selection
-        linear - Force linear algorithm
+        auto - Internal algorithm selection (can be overridden with MPIR_CVAR_COLL_SELECTION_TUNING_JSON_FILE)
+        sched_auto - Internal algorithm selection for sched-based algorithms
+        sched_linear - Force linear algorithm
 
     - name        : MPIR_CVAR_ISCATTERV_DEVICE_COLLECTIVE
       category    : COLLECTIVE
       type        : boolean
       default     : true
-      class       : device
+      class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : >-
-        If set to true, MPI_Iscatterv will allow the device to override the
-        MPIR-level collective algorithms. The device still has the
-        option to call the MPIR-level algorithms manually.
-        If set to false, the device-level iscatterv function will not be
-        called.
+        This CVAR is only used when MPIR_CVAR_DEVICE_COLLECTIVES
+        is set to "percoll".  If set to true, MPI_Iscatterv will
+        allow the device to override the MPIR-level collective
+        algorithms.  The device might still call the MPIR-level
+        algorithms manually.  If set to false, the device-override
+        will be disabled.
 
 === END_MPI_T_CVAR_INFO_BLOCK ===
 */
@@ -74,11 +77,65 @@ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[
 
 /* any non-MPI functions go here, especially non-static ones */
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Iscatterv_sched_intra_auto
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Iscatterv_sched_intra_auto(const void *sendbuf, const int sendcounts[], const int displs[],
+
+int MPIR_Iscatterv_allcomm_auto(const void *sendbuf, const int *sendcounts, const int *displs,
+                                MPI_Datatype sendtype, void *recvbuf, int recvcount,
+                                MPI_Datatype recvtype, int root, MPIR_Comm * comm_ptr,
+                                MPIR_Request ** request)
+{
+    int mpi_errno = MPI_SUCCESS;
+
+    MPIR_Csel_coll_sig_s coll_sig = {
+        .coll_type = MPIR_CSEL_COLL_TYPE__ISCATTERV,
+        .comm_ptr = comm_ptr,
+
+        .u.iscatterv.sendbuf = sendbuf,
+        .u.iscatterv.sendcounts = sendcounts,
+        .u.iscatterv.displs = displs,
+        .u.iscatterv.sendtype = sendtype,
+        .u.iscatterv.recvcount = recvcount,
+        .u.iscatterv.recvbuf = recvbuf,
+        .u.iscatterv.recvtype = recvtype,
+        .u.iscatterv.root = root,
+    };
+
+    MPII_Csel_container_s *cnt = MPIR_Csel_search(comm_ptr->csel_comm, coll_sig);
+    MPIR_Assert(cnt);
+
+    switch (cnt->id) {
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Iscatterv_allcomm_gentran_linear:
+            mpi_errno =
+                MPIR_Iscatterv_allcomm_gentran_linear(sendbuf, sendcounts, displs, sendtype,
+                                                      recvbuf, recvcount, recvtype, root, comm_ptr,
+                                                      request);
+            break;
+
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Iscatterv_intra_sched_auto:
+            MPII_SCHED_WRAPPER(MPIR_Iscatterv_intra_sched_auto, comm_ptr, request, sendbuf,
+                               sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root);
+            break;
+
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Iscatterv_inter_sched_auto:
+            MPII_SCHED_WRAPPER(MPIR_Iscatterv_inter_sched_auto, comm_ptr, request, sendbuf,
+                               sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root);
+            break;
+
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Iscatterv_allcomm_sched_linear:
+            MPII_SCHED_WRAPPER(MPIR_Iscatterv_allcomm_sched_linear, comm_ptr, request, sendbuf,
+                               sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root);
+            break;
+
+        default:
+            MPIR_Assert(0);
+    }
+
+  fn_exit:
+    return mpi_errno;
+  fn_fail:
+    goto fn_exit;
+}
+
+int MPIR_Iscatterv_intra_sched_auto(const void *sendbuf, const int sendcounts[], const int displs[],
                                     MPI_Datatype sendtype, void *recvbuf, int recvcount,
                                     MPI_Datatype recvtype, int root, MPIR_Comm * comm_ptr,
                                     MPIR_Sched_t s)
@@ -86,10 +143,9 @@ int MPIR_Iscatterv_sched_intra_auto(const void *sendbuf, const int sendcounts[],
     int mpi_errno = MPI_SUCCESS;
 
     mpi_errno =
-        MPIR_Iscatterv_sched_allcomm_linear(sendbuf, sendcounts, displs, sendtype, recvbuf,
+        MPIR_Iscatterv_allcomm_sched_linear(sendbuf, sendcounts, displs, sendtype, recvbuf,
                                             recvcount, recvtype, root, comm_ptr, s);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
     return mpi_errno;
@@ -98,11 +154,7 @@ int MPIR_Iscatterv_sched_intra_auto(const void *sendbuf, const int sendcounts[],
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Iscatterv_sched_inter_auto
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Iscatterv_sched_inter_auto(const void *sendbuf, const int sendcounts[], const int displs[],
+int MPIR_Iscatterv_inter_sched_auto(const void *sendbuf, const int sendcounts[], const int displs[],
                                     MPI_Datatype sendtype, void *recvbuf, int recvcount,
                                     MPI_Datatype recvtype, int root, MPIR_Comm * comm_ptr,
                                     MPIR_Sched_t s)
@@ -110,10 +162,9 @@ int MPIR_Iscatterv_sched_inter_auto(const void *sendbuf, const int sendcounts[],
     int mpi_errno = MPI_SUCCESS;
 
     mpi_errno =
-        MPIR_Iscatterv_sched_allcomm_linear(sendbuf, sendcounts, displs, sendtype, recvbuf,
+        MPIR_Iscatterv_allcomm_sched_linear(sendbuf, sendcounts, displs, sendtype, recvbuf,
                                             recvcount, recvtype, root, comm_ptr, s);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
     return mpi_errno;
@@ -122,51 +173,22 @@ int MPIR_Iscatterv_sched_inter_auto(const void *sendbuf, const int sendcounts[],
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Iscatterv_sched_impl
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Iscatterv_sched_impl(const void *sendbuf, const int sendcounts[], const int displs[],
+int MPIR_Iscatterv_sched_auto(const void *sendbuf, const int sendcounts[], const int displs[],
                               MPI_Datatype sendtype, void *recvbuf, int recvcount,
                               MPI_Datatype recvtype, int root, MPIR_Comm * comm_ptr, MPIR_Sched_t s)
 {
     int mpi_errno = MPI_SUCCESS;
 
     if (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) {
-        switch (MPIR_Iscatterv_intra_algo_choice) {
-            case MPIR_ISCATTERV_INTRA_ALGO_LINEAR:
-                mpi_errno =
-                    MPIR_Iscatterv_sched_allcomm_linear(sendbuf, sendcounts, displs, sendtype,
-                                                        recvbuf, recvcount, recvtype, root,
-                                                        comm_ptr, s);
-                break;
-            case MPIR_ISCATTERV_INTRA_ALGO_AUTO:
-                MPL_FALLTHROUGH;
-            default:
-                mpi_errno =
-                    MPIR_Iscatterv_sched_intra_auto(sendbuf, sendcounts, displs, sendtype, recvbuf,
-                                                    recvcount, recvtype, root, comm_ptr, s);
-                break;
-        }
+        mpi_errno =
+            MPIR_Iscatterv_intra_sched_auto(sendbuf, sendcounts, displs, sendtype, recvbuf,
+                                            recvcount, recvtype, root, comm_ptr, s);
     } else {
-        switch (MPIR_Iscatterv_inter_algo_choice) {
-            case MPIR_ISCATTERV_INTER_ALGO_LINEAR:
-                mpi_errno =
-                    MPIR_Iscatterv_sched_allcomm_linear(sendbuf, sendcounts, displs, sendtype,
-                                                        recvbuf, recvcount, recvtype, root,
-                                                        comm_ptr, s);
-                break;
-            case MPIR_ISCATTERV_INTER_ALGO_AUTO:
-                MPL_FALLTHROUGH;
-            default:
-                mpi_errno =
-                    MPIR_Iscatterv_sched_inter_auto(sendbuf, sendcounts, displs, sendtype, recvbuf,
-                                                    recvcount, recvtype, root, comm_ptr, s);
-                break;
-        }
+        mpi_errno =
+            MPIR_Iscatterv_inter_sched_auto(sendbuf, sendcounts, displs, sendtype, recvbuf,
+                                            recvcount, recvtype, root, comm_ptr, s);
     }
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
     return mpi_errno;
@@ -174,58 +196,75 @@ int MPIR_Iscatterv_sched_impl(const void *sendbuf, const int sendcounts[], const
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Iscatterv_sched
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Iscatterv_sched(const void *sendbuf, const int sendcounts[], const int displs[],
-                         MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype,
-                         int root, MPIR_Comm * comm_ptr, MPIR_Sched_t s)
-{
-    int mpi_errno = MPI_SUCCESS;
-
-    if (MPIR_CVAR_BARRIER_DEVICE_COLLECTIVE && MPIR_CVAR_DEVICE_COLLECTIVES) {
-        mpi_errno = MPID_Iscatterv_sched(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount,
-                                         recvtype, root, comm_ptr, s);
-    } else {
-        mpi_errno = MPIR_Iscatterv_sched_impl(sendbuf, sendcounts, displs, sendtype, recvbuf,
-                                              recvcount, recvtype, root, comm_ptr, s);
-    }
-
-    return mpi_errno;
-}
-
-#undef FUNCNAME
-#define FUNCNAME MPIR_Iscatterv_impl
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIR_Iscatterv_impl(const void *sendbuf, const int sendcounts[], const int displs[],
                         MPI_Datatype sendtype, void *recvbuf, int recvcount,
                         MPI_Datatype recvtype, int root, MPIR_Comm * comm_ptr,
                         MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
-    int tag = -1;
-    MPIR_Sched_t s = MPIR_SCHED_NULL;
 
     *request = NULL;
+    /* If the user picks one of the transport-enabled algorithms, branch there
+     * before going down to the MPIR_Sched-based algorithms. */
+    /* TODO - Eventually the intention is to replace all of the
+     * MPIR_Sched-based algorithms with transport-enabled algorithms, but that
+     * will require sufficient performance testing and replacement algorithms. */
+    if (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) {
+        switch (MPIR_CVAR_ISCATTERV_INTRA_ALGORITHM) {
+            case MPIR_CVAR_ISCATTERV_INTRA_ALGORITHM_gentran_linear:
+                mpi_errno =
+                    MPIR_Iscatterv_allcomm_gentran_linear(sendbuf, sendcounts, displs, sendtype,
+                                                          recvbuf, recvcount, recvtype, root,
+                                                          comm_ptr, request);
+                break;
 
-    mpi_errno = MPIR_Sched_next_tag(comm_ptr, &tag);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
-    mpi_errno = MPIR_Sched_create(&s);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+            case MPIR_CVAR_ISCATTERV_INTRA_ALGORITHM_sched_linear:
+                MPII_SCHED_WRAPPER(MPIR_Iscatterv_allcomm_sched_linear, comm_ptr, request, sendbuf,
+                                   sendcounts, displs, sendtype, recvbuf, recvcount, recvtype,
+                                   root);
+                break;
 
-    mpi_errno =
-        MPIR_Iscatterv_sched(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype,
-                             root, comm_ptr, s);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+            case MPIR_CVAR_ISCATTERV_INTRA_ALGORITHM_sched_auto:
+                MPII_SCHED_WRAPPER(MPIR_Iscatterv_intra_sched_auto, comm_ptr, request, sendbuf,
+                                   sendcounts, displs, sendtype, recvbuf, recvcount, recvtype,
+                                   root);
+                break;
 
-    mpi_errno = MPIR_Sched_start(&s, comm_ptr, tag, request);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+            case MPIR_CVAR_ISCATTERV_INTRA_ALGORITHM_auto:
+                mpi_errno =
+                    MPIR_Iscatterv_allcomm_auto(sendbuf, sendcounts, displs, sendtype, recvbuf,
+                                                recvcount, recvtype, root, comm_ptr, request);
+                break;
+
+            default:
+                MPIR_Assert(0);
+        }
+    } else {
+        switch (MPIR_CVAR_ISCATTERV_INTER_ALGORITHM) {
+            case MPIR_CVAR_ISCATTERV_INTER_ALGORITHM_sched_linear:
+                MPII_SCHED_WRAPPER(MPIR_Iscatterv_allcomm_sched_linear, comm_ptr, request, sendbuf,
+                                   sendcounts, displs, sendtype, recvbuf, recvcount, recvtype,
+                                   root);
+                break;
+
+            case MPIR_CVAR_ISCATTERV_INTER_ALGORITHM_sched_auto:
+                MPII_SCHED_WRAPPER(MPIR_Iscatterv_inter_sched_auto, comm_ptr, request, sendbuf,
+                                   sendcounts, displs, sendtype, recvbuf, recvcount, recvtype,
+                                   root);
+                break;
+
+            case MPIR_CVAR_ISCATTERV_INTER_ALGORITHM_auto:
+                mpi_errno =
+                    MPIR_Iscatterv_allcomm_auto(sendbuf, sendcounts, displs, sendtype, recvbuf,
+                                                recvcount, recvtype, root, comm_ptr, request);
+                break;
+
+            default:
+                MPIR_Assert(0);
+        }
+    }
+
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
     return mpi_errno;
@@ -233,19 +272,18 @@ int MPIR_Iscatterv_impl(const void *sendbuf, const int sendcounts[], const int d
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Iscatterv
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIR_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[],
                    MPI_Datatype sendtype, void *recvbuf, int recvcount,
                    MPI_Datatype recvtype, int root, MPIR_Comm * comm_ptr, MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
 
-    if (MPIR_CVAR_BARRIER_DEVICE_COLLECTIVE && MPIR_CVAR_DEVICE_COLLECTIVES) {
-        mpi_errno = MPID_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount,
-                                   recvtype, root, comm_ptr, request);
+    if ((MPIR_CVAR_DEVICE_COLLECTIVES == MPIR_CVAR_DEVICE_COLLECTIVES_all) ||
+        ((MPIR_CVAR_DEVICE_COLLECTIVES == MPIR_CVAR_DEVICE_COLLECTIVES_percoll) &&
+         MPIR_CVAR_BARRIER_DEVICE_COLLECTIVE)) {
+        mpi_errno =
+            MPID_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype,
+                           root, comm_ptr, request);
     } else {
         mpi_errno = MPIR_Iscatterv_impl(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount,
                                         recvtype, root, comm_ptr, request);
@@ -256,10 +294,6 @@ int MPIR_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs
 
 #endif /* MPICH_MPI_FROM_PMPI */
 
-#undef FUNCNAME
-#define FUNCNAME MPI_Iscatterv
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 /*@
 MPI_Iscatterv - Scatters a buffer in parts to all processes in a communicator
                 in a nonblocking way
@@ -337,7 +371,7 @@ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[
                         MPIR_ERRTEST_COUNT(sendcounts[i], mpi_errno);
                         MPIR_ERRTEST_DATATYPE(sendtype, "sendtype", mpi_errno);
                     }
-                    if (HANDLE_GET_KIND(sendtype) != HANDLE_KIND_BUILTIN) {
+                    if (!HANDLE_IS_BUILTIN(sendtype)) {
                         MPIR_Datatype_get_ptr(sendtype, sendtype_ptr);
                         MPIR_Datatype_valid_ptr(sendtype_ptr, mpi_errno);
                         if (mpi_errno != MPI_SUCCESS)
@@ -374,7 +408,7 @@ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[
                 if (recvbuf != MPI_IN_PLACE) {
                     MPIR_ERRTEST_COUNT(recvcount, mpi_errno);
                     MPIR_ERRTEST_DATATYPE(recvtype, "recvtype", mpi_errno);
-                    if (HANDLE_GET_KIND(recvtype) != HANDLE_KIND_BUILTIN) {
+                    if (!HANDLE_IS_BUILTIN(recvtype)) {
                         MPIR_Datatype_get_ptr(recvtype, recvtype_ptr);
                         MPIR_Datatype_valid_ptr(recvtype_ptr, mpi_errno);
                         if (mpi_errno != MPI_SUCCESS)
@@ -395,7 +429,7 @@ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[
                         MPIR_ERRTEST_COUNT(sendcounts[i], mpi_errno);
                         MPIR_ERRTEST_DATATYPE(sendtype, "sendtype", mpi_errno);
                     }
-                    if (HANDLE_GET_KIND(sendtype) != HANDLE_KIND_BUILTIN) {
+                    if (!HANDLE_IS_BUILTIN(sendtype)) {
                         MPIR_Datatype_get_ptr(sendtype, sendtype_ptr);
                         MPIR_Datatype_valid_ptr(sendtype_ptr, mpi_errno);
                         if (mpi_errno != MPI_SUCCESS)
@@ -414,7 +448,7 @@ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[
                 } else if (root != MPI_PROC_NULL) {
                     MPIR_ERRTEST_COUNT(recvcount, mpi_errno);
                     MPIR_ERRTEST_DATATYPE(recvtype, "recvtype", mpi_errno);
-                    if (HANDLE_GET_KIND(recvtype) != HANDLE_KIND_BUILTIN) {
+                    if (!HANDLE_IS_BUILTIN(recvtype)) {
                         MPIR_Datatype_get_ptr(recvtype, recvtype_ptr);
                         MPIR_Datatype_valid_ptr(recvtype_ptr, mpi_errno);
                         if (mpi_errno != MPI_SUCCESS)
@@ -436,8 +470,7 @@ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[
 
     mpi_errno = MPIR_Iscatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype,
                                root, comm_ptr, &request_ptr);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
     /* create a complete request, if needed */
     if (!request_ptr)
@@ -457,13 +490,13 @@ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[
 #ifdef HAVE_ERROR_CHECKING
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpi_iscatterv", "**mpi_iscatterv %p %p %p %D %p %d %D %d %C %p",
                                  sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount,
                                  recvtype, root, comm, request);
     }
 #endif
-    mpi_errno = MPIR_Err_return_comm(comm_ptr, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(comm_ptr, __func__, mpi_errno);
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
