@@ -1,14 +1,12 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2009 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 
 /* This implementation of MPI_Reduce_scatter_block was obtained by taking
    the implementation of MPI_Reduce_scatter from reduce_scatter.c and replacing
-   recvcnts[i] with recvcount everywhere. */
+   recvcounts[i] with recvcount everywhere. */
 
 #include "mpiimpl.h"
 
@@ -18,14 +16,14 @@
 cvars:
     - name        : MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTRA_ALGORITHM
       category    : COLLECTIVE
-      type        : string
+      type        : enum
       default     : auto
-      class       : device
+      class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : |-
         Variable to select reduce_scatter_block algorithm
-        auto               - Internal algorithm selection
+        auto - Internal algorithm selection (can be overridden with MPIR_CVAR_COLL_SELECTION_TUNING_JSON_FILE)
         noncommutative     - Force noncommutative algorithm
         recursive_doubling - Force recursive doubling algorithm
         pairwise           - Force pairwise algorithm
@@ -34,14 +32,14 @@ cvars:
 
     - name        : MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTER_ALGORITHM
       category    : COLLECTIVE
-      type        : string
+      type        : enum
       default     : auto
-      class       : device
+      class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : |-
         Variable to select reduce_scatter_block algorithm
-        auto                        - Internal algorithm selection
+        auto - Internal algorithm selection (can be overridden with MPIR_CVAR_COLL_SELECTION_TUNING_JSON_FILE)
         nb                          - Force nonblocking algorithm
         remote_reduce_local_scatter - Force remote-reduce-local-scatter algorithm
 
@@ -49,15 +47,16 @@ cvars:
       category    : COLLECTIVE
       type        : boolean
       default     : true
-      class       : device
+      class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : >-
-        If set to true, MPI_Reduce_scatter_block will allow the device to override the
-        MPIR-level collective algorithms. The device still has the
-        option to call the MPIR-level algorithms manually.
-        If set to false, the device-level reduce_scatter_block function will not be
-        called.
+        This CVAR is only used when MPIR_CVAR_DEVICE_COLLECTIVES
+        is set to "percoll".  If set to true, MPI_Reduce_scatter_block will
+        allow the device to override the MPIR-level collective
+        algorithms.  The device might still call the MPIR-level
+        algorithms manually.  If set to false, the device-override
+        will be disabled.
 
 === END_MPI_T_CVAR_INFO_BLOCK ===
 */
@@ -92,195 +91,68 @@ int MPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount,
    End Algorithm: MPI_Reduce_scatter
 */
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Reduce_scatter_block_intra_auto
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Reduce_scatter_block_intra_auto(const void *sendbuf,
-                                         void *recvbuf,
-                                         int recvcount,
-                                         MPI_Datatype datatype,
-                                         MPI_Op op, MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
+
+int MPIR_Reduce_scatter_block_allcomm_auto(const void *sendbuf, void *recvbuf, int recvcount,
+                                           MPI_Datatype datatype, MPI_Op op, MPIR_Comm * comm_ptr,
+                                           MPIR_Errflag_t * errflag)
 {
-    int comm_size;
-    MPI_Aint true_extent, true_lb;
     int mpi_errno = MPI_SUCCESS;
-    int mpi_errno_ret = MPI_SUCCESS;
-    int type_size, total_count, nbytes;
-    int is_commutative;
 
-    comm_size = comm_ptr->local_size;
+    MPIR_Csel_coll_sig_s coll_sig = {
+        .coll_type = MPIR_CSEL_COLL_TYPE__REDUCE_SCATTER_BLOCK,
+        .comm_ptr = comm_ptr,
 
-    /* set op_errno to 0. stored in perthread structure */
-    {
-        MPIR_Per_thread_t *per_thread = NULL;
-        int err = 0;
+        .u.reduce_scatter_block.sendbuf = sendbuf,
+        .u.reduce_scatter_block.recvbuf = recvbuf,
+        .u.reduce_scatter_block.recvcount = recvcount,
+        .u.reduce_scatter_block.datatype = datatype,
+        .u.reduce_scatter_block.op = op,
+    };
 
-        MPID_THREADPRIV_KEY_GET_ADDR(MPIR_ThreadInfo.isThreaded, MPIR_Per_thread_key,
-                                     MPIR_Per_thread, per_thread, &err);
-        MPIR_Assert(err == 0);
-        per_thread->op_errno = 0;
-    }
+    MPII_Csel_container_s *cnt = MPIR_Csel_search(comm_ptr->csel_comm, coll_sig);
+    MPIR_Assert(cnt);
 
-    if (recvcount == 0) {
-        goto fn_exit;
-    }
-
-    MPIR_Type_get_true_extent_impl(datatype, &true_lb, &true_extent);
-
-    is_commutative = MPIR_Op_is_commutative(op);
-
-    total_count = comm_size * recvcount;
-
-    MPIR_Datatype_get_size_macro(datatype, type_size);
-    nbytes = total_count * type_size;
-
-    if ((is_commutative) && (nbytes < MPIR_CVAR_REDUCE_SCATTER_COMMUTATIVE_LONG_MSG_SIZE)) {
-        /* commutative and short. use recursive halving algorithm */
-        mpi_errno =
-            MPIR_Reduce_scatter_block_intra_recursive_halving(sendbuf, recvbuf, recvcount, datatype,
-                                                              op, comm_ptr, errflag);
-    } else if (is_commutative && (nbytes >= MPIR_CVAR_REDUCE_SCATTER_COMMUTATIVE_LONG_MSG_SIZE)) {
-        /* commutative and long message, or noncommutative and long message.
-         * use (p-1) pairwise exchanges */
-        mpi_errno =
-            MPIR_Reduce_scatter_block_intra_pairwise(sendbuf, recvbuf, recvcount, datatype, op,
-                                                     comm_ptr, errflag);
-    } else if (!(comm_size & (comm_size - 1))) {        /* power of two check */
-        /* noncommutative, pof2 size */
-        mpi_errno =
-            MPIR_Reduce_scatter_block_intra_noncommutative(sendbuf, recvbuf, recvcount, datatype,
-                                                           op, comm_ptr, errflag);
-    } else {
-        /* noncommutative and non-pof2, use recursive doubling. */
-        mpi_errno =
-            MPIR_Reduce_scatter_block_intra_recursive_doubling(sendbuf, recvbuf, recvcount,
+    switch (cnt->id) {
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Reduce_scatter_block_intra_noncommutative:
+            mpi_errno =
+                MPIR_Reduce_scatter_block_intra_noncommutative(sendbuf, recvbuf, recvcount,
                                                                datatype, op, comm_ptr, errflag);
-    }
+            break;
 
-    if (mpi_errno) {
-        /* for communication errors, just record the error but continue */
-        *errflag =
-            MPIX_ERR_PROC_FAILED ==
-            MPIR_ERR_GET_CLASS(mpi_errno) ? MPIR_ERR_PROC_FAILED : MPIR_ERR_OTHER;
-        MPIR_ERR_SET(mpi_errno, *errflag, "**fail");
-        MPIR_ERR_ADD(mpi_errno_ret, mpi_errno);
-    }
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Reduce_scatter_block_intra_pairwise:
+            mpi_errno =
+                MPIR_Reduce_scatter_block_intra_pairwise(sendbuf, recvbuf, recvcount, datatype, op,
+                                                         comm_ptr, errflag);
+            break;
 
-  fn_exit:
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Reduce_scatter_block_intra_recursive_doubling:
+            mpi_errno =
+                MPIR_Reduce_scatter_block_intra_recursive_doubling(sendbuf, recvbuf, recvcount,
+                                                                   datatype, op, comm_ptr, errflag);
+            break;
 
-    {
-        MPIR_Per_thread_t *per_thread = NULL;
-        int err = 0;
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Reduce_scatter_block_intra_recursive_halving:
+            mpi_errno =
+                MPIR_Reduce_scatter_block_intra_recursive_halving(sendbuf, recvbuf, recvcount,
+                                                                  datatype, op, comm_ptr, errflag);
+            break;
 
-        MPID_THREADPRIV_KEY_GET_ADDR(MPIR_ThreadInfo.isThreaded, MPIR_Per_thread_key,
-                                     MPIR_Per_thread, per_thread, &err);
-        MPIR_Assert(err == 0);
-        if (per_thread->op_errno)
-            mpi_errno = per_thread->op_errno;
-    }
-
-    /* --BEGIN ERROR HANDLING-- */
-    if (mpi_errno_ret)
-        mpi_errno = mpi_errno_ret;
-    else if (*errflag != MPIR_ERR_NONE)
-        MPIR_ERR_SET(mpi_errno, *errflag, "**coll_fail");
-    /* --END ERROR HANDLING-- */
-    return mpi_errno;
-}
-
-#undef FUNCNAME
-#define FUNCNAME MPIR_Reduce_scatter_block_inter_auto
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Reduce_scatter_block_inter_auto(const void *sendbuf,
-                                         void *recvbuf,
-                                         int recvcount,
-                                         MPI_Datatype datatype,
-                                         MPI_Op op, MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
-{
-    int mpi_errno = MPI_SUCCESS;
-
-    mpi_errno = MPIR_Reduce_scatter_block_inter_remote_reduce_local_scatter(sendbuf, recvbuf,
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Reduce_scatter_block_inter_remote_reduce_local_scatter:
+            mpi_errno =
+                MPIR_Reduce_scatter_block_inter_remote_reduce_local_scatter(sendbuf, recvbuf,
                                                                             recvcount, datatype, op,
                                                                             comm_ptr, errflag);
+            break;
 
-    return mpi_errno;
-}
+        case MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Reduce_scatter_block_allcomm_nb:
+            mpi_errno =
+                MPIR_Reduce_scatter_block_allcomm_nb(sendbuf, recvbuf, recvcount, datatype, op,
+                                                     comm_ptr, errflag);
+            break;
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Reduce_scatter_block_impl
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIR_Reduce_scatter_block_impl(const void *sendbuf, void *recvbuf,
-                                   int recvcount, MPI_Datatype datatype,
-                                   MPI_Op op, MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
-{
-    int mpi_errno = MPI_SUCCESS;
-
-    if (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) {
-        switch (MPIR_Reduce_scatter_block_intra_algo_choice) {
-                /* intracommunicator */
-            case MPIR_REDUCE_SCATTER_BLOCK_INTRA_ALGO_NONCOMMUTATIVE:
-                mpi_errno = MPIR_Reduce_scatter_block_intra_noncommutative(sendbuf, recvbuf,
-                                                                           recvcount, datatype, op,
-                                                                           comm_ptr, errflag);
-                break;
-            case MPIR_REDUCE_SCATTER_BLOCK_INTRA_ALGO_PAIRWISE:
-                mpi_errno = MPIR_Reduce_scatter_block_intra_pairwise(sendbuf, recvbuf,
-                                                                     recvcount, datatype, op,
-                                                                     comm_ptr, errflag);
-                break;
-            case MPIR_REDUCE_SCATTER_BLOCK_INTRA_ALGO_RECURSIVE_HALVING:
-                mpi_errno = MPIR_Reduce_scatter_block_intra_recursive_halving(sendbuf, recvbuf,
-                                                                              recvcount, datatype,
-                                                                              op, comm_ptr,
-                                                                              errflag);
-                break;
-            case MPIR_REDUCE_SCATTER_BLOCK_INTRA_ALGO_RECURSIVE_DOUBLING:
-                mpi_errno = MPIR_Reduce_scatter_block_intra_recursive_doubling(sendbuf, recvbuf,
-                                                                               recvcount, datatype,
-                                                                               op, comm_ptr,
-                                                                               errflag);
-                break;
-            case MPIR_REDUCE_SCATTER_BLOCK_INTRA_ALGO_NB:
-                mpi_errno = MPIR_Reduce_scatter_block_allcomm_nb(sendbuf, recvbuf,
-                                                                 recvcount, datatype, op, comm_ptr,
-                                                                 errflag);
-                break;
-            case MPIR_REDUCE_SCATTER_BLOCK_INTRA_ALGO_AUTO:
-                MPL_FALLTHROUGH;
-            default:
-                mpi_errno = MPIR_Reduce_scatter_block_intra_auto(sendbuf, recvbuf,
-                                                                 recvcount, datatype, op, comm_ptr,
-                                                                 errflag);
-                break;
-        }
-    } else {
-        /* intercommunicator */
-        switch (MPIR_Reduce_scatter_block_inter_algo_choice) {
-            case MPIR_REDUCE_SCATTER_BLOCK_INTER_ALGO_REMOTE_REDUCE_LOCAL_SCATTER:
-                mpi_errno =
-                    MPIR_Reduce_scatter_block_inter_remote_reduce_local_scatter(sendbuf, recvbuf,
-                                                                                recvcount, datatype,
-                                                                                op, comm_ptr,
-                                                                                errflag);
-                break;
-            case MPIR_REDUCE_SCATTER_BLOCK_INTER_ALGO_NB:
-                mpi_errno = MPIR_Reduce_scatter_block_allcomm_nb(sendbuf, recvbuf,
-                                                                 recvcount, datatype, op, comm_ptr,
-                                                                 errflag);
-                break;
-            case MPIR_REDUCE_SCATTER_BLOCK_INTER_ALGO_AUTO:
-                MPL_FALLTHROUGH;
-            default:
-                mpi_errno = MPIR_Reduce_scatter_block_inter_auto(sendbuf, recvbuf, recvcount,
-                                                                 datatype, op, comm_ptr, errflag);
-                break;
-        }
+        default:
+            MPIR_Assert(0);
     }
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
 
   fn_exit:
     return mpi_errno;
@@ -288,33 +160,120 @@ int MPIR_Reduce_scatter_block_impl(const void *sendbuf, void *recvbuf,
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Reduce_scatter_block
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
+int MPIR_Reduce_scatter_block_impl(const void *sendbuf, void *recvbuf,
+                                   int recvcount, MPI_Datatype datatype,
+                                   MPI_Op op, MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
+{
+    int mpi_errno = MPI_SUCCESS;
+
+    if (comm_ptr->comm_kind == MPIR_COMM_KIND__INTRACOMM) {
+        switch (MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTRA_ALGORITHM) {
+                /* intracommunicator */
+            case MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTRA_ALGORITHM_noncommutative:
+                mpi_errno = MPIR_Reduce_scatter_block_intra_noncommutative(sendbuf, recvbuf,
+                                                                           recvcount, datatype, op,
+                                                                           comm_ptr, errflag);
+                break;
+            case MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTRA_ALGORITHM_pairwise:
+                mpi_errno = MPIR_Reduce_scatter_block_intra_pairwise(sendbuf, recvbuf,
+                                                                     recvcount, datatype, op,
+                                                                     comm_ptr, errflag);
+                break;
+            case MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTRA_ALGORITHM_recursive_halving:
+                mpi_errno = MPIR_Reduce_scatter_block_intra_recursive_halving(sendbuf, recvbuf,
+                                                                              recvcount, datatype,
+                                                                              op, comm_ptr,
+                                                                              errflag);
+                break;
+            case MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTRA_ALGORITHM_recursive_doubling:
+                mpi_errno = MPIR_Reduce_scatter_block_intra_recursive_doubling(sendbuf, recvbuf,
+                                                                               recvcount, datatype,
+                                                                               op, comm_ptr,
+                                                                               errflag);
+                break;
+            case MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTRA_ALGORITHM_nb:
+                mpi_errno = MPIR_Reduce_scatter_block_allcomm_nb(sendbuf, recvbuf,
+                                                                 recvcount, datatype, op, comm_ptr,
+                                                                 errflag);
+                break;
+            case MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTRA_ALGORITHM_auto:
+                mpi_errno = MPIR_Reduce_scatter_block_allcomm_auto(sendbuf, recvbuf,
+                                                                   recvcount, datatype, op,
+                                                                   comm_ptr, errflag);
+                break;
+            default:
+                MPIR_Assert(0);
+        }
+    } else {
+        /* intercommunicator */
+        switch (MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTER_ALGORITHM) {
+            case MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTER_ALGORITHM_remote_reduce_local_scatter:
+                mpi_errno =
+                    MPIR_Reduce_scatter_block_inter_remote_reduce_local_scatter(sendbuf, recvbuf,
+                                                                                recvcount, datatype,
+                                                                                op, comm_ptr,
+                                                                                errflag);
+                break;
+            case MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTER_ALGORITHM_nb:
+                mpi_errno = MPIR_Reduce_scatter_block_allcomm_nb(sendbuf, recvbuf,
+                                                                 recvcount, datatype, op, comm_ptr,
+                                                                 errflag);
+                break;
+            case MPIR_CVAR_REDUCE_SCATTER_BLOCK_INTER_ALGORITHM_auto:
+                mpi_errno = MPIR_Reduce_scatter_block_allcomm_auto(sendbuf, recvbuf, recvcount,
+                                                                   datatype, op, comm_ptr, errflag);
+                break;
+            default:
+                MPIR_Assert(0);
+        }
+    }
+    MPIR_ERR_CHECK(mpi_errno);
+
+  fn_exit:
+    return mpi_errno;
+  fn_fail:
+    goto fn_exit;
+}
+
 int MPIR_Reduce_scatter_block(const void *sendbuf, void *recvbuf,
                               int recvcount, MPI_Datatype datatype,
                               MPI_Op op, MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
 {
     int mpi_errno = MPI_SUCCESS;
+    void *in_recvbuf = recvbuf;
+    void *host_sendbuf;
+    void *host_recvbuf;
 
-    if (MPIR_CVAR_REDUCE_SCATTER_BLOCK_DEVICE_COLLECTIVE && MPIR_CVAR_DEVICE_COLLECTIVES) {
-        mpi_errno = MPID_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm_ptr,
-                                              errflag);
+    MPIR_Coll_host_buffer_alloc(sendbuf, recvbuf, recvcount, datatype, &host_sendbuf,
+                                &host_recvbuf);
+    if (host_sendbuf)
+        sendbuf = host_sendbuf;
+    if (host_recvbuf)
+        recvbuf = host_recvbuf;
+
+    if ((MPIR_CVAR_DEVICE_COLLECTIVES == MPIR_CVAR_DEVICE_COLLECTIVES_all) ||
+        ((MPIR_CVAR_DEVICE_COLLECTIVES == MPIR_CVAR_DEVICE_COLLECTIVES_percoll) &&
+         MPIR_CVAR_REDUCE_SCATTER_BLOCK_DEVICE_COLLECTIVE)) {
+        mpi_errno =
+            MPID_Reduce_scatter_block(sendbuf, recvbuf, recvcount, datatype, op, comm_ptr, errflag);
     } else {
         mpi_errno = MPIR_Reduce_scatter_block_impl(sendbuf, recvbuf, recvcount, datatype, op,
                                                    comm_ptr, errflag);
     }
 
+    /* Copy out data from host recv buffer to GPU buffer */
+    if (host_recvbuf) {
+        recvbuf = in_recvbuf;
+        MPIR_Localcopy(host_recvbuf, recvcount, datatype, recvbuf, recvcount, datatype);
+    }
+
+    MPIR_Coll_host_buffer_free(host_sendbuf, host_recvbuf);
+
     return mpi_errno;
 }
 
 #endif
 
-#undef FUNCNAME
-#define FUNCNAME MPI_Reduce_scatter_block
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 /*@
 
 MPI_Reduce_scatter_block - Combines values and scatters the results
@@ -386,7 +345,7 @@ int MPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf,
             MPIR_ERRTEST_COUNT(recvcount, mpi_errno);
 
             MPIR_ERRTEST_DATATYPE(datatype, "datatype", mpi_errno);
-            if (HANDLE_GET_KIND(datatype) != HANDLE_KIND_BUILTIN) {
+            if (!HANDLE_IS_BUILTIN(datatype)) {
                 MPIR_Datatype_get_ptr(datatype, datatype_ptr);
                 MPIR_Datatype_valid_ptr(datatype_ptr, mpi_errno);
                 if (mpi_errno != MPI_SUCCESS)
@@ -407,11 +366,10 @@ int MPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf,
 
             MPIR_ERRTEST_OP(op, mpi_errno);
 
-            if (HANDLE_GET_KIND(op) != HANDLE_KIND_BUILTIN) {
+            if (!HANDLE_IS_BUILTIN(op)) {
                 MPIR_Op_get_ptr(op, op_ptr);
                 MPIR_Op_valid_ptr(op_ptr, mpi_errno);
-            }
-            if (HANDLE_GET_KIND(op) == HANDLE_KIND_BUILTIN) {
+            } else {
                 mpi_errno = (*MPIR_OP_HDL_TO_DTYPE_FN(op)) (datatype);
             }
             if (mpi_errno != MPI_SUCCESS)
@@ -440,13 +398,13 @@ int MPI_Reduce_scatter_block(const void *sendbuf, void *recvbuf,
 #ifdef HAVE_ERROR_CHECKING
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpi_reduce_scatter_block",
                                  "**mpi_reduce_scatter_block %p %p %d %D %O %C", sendbuf, recvbuf,
                                  recvcount, datatype, op, comm);
     }
 #endif
-    mpi_errno = MPIR_Err_return_comm(comm_ptr, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(comm_ptr, __func__, mpi_errno);
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
