@@ -1,11 +1,6 @@
 /*
- *  (C) 2006 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
- *
- *  Portions of this code were written by Intel Corporation.
- *  Copyright (C) 2011-2018 Intel Corporation.  Intel provides this material
- *  to Argonne National Laboratory subject to Software Grant and Corporate
- *  Contributor License Agreement dated February 8, 2012.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #ifndef TSP_GENTRAN_H_INCLUDED
@@ -27,6 +22,8 @@
 #undef MPIR_TSP_sched_selective_sink
 #undef MPIR_TSP_sched_sink
 #undef MPIR_TSP_sched_fence
+#undef MPIR_TSP_sched_new_type
+#undef MPIR_TSP_sched_generic
 #undef MPIR_TSP_sched_malloc
 #undef MPIR_TSP_sched_start
 #undef MPIR_TSP_sched_free
@@ -43,21 +40,36 @@
 #define MPIR_TSP_sched_isend               MPII_Genutil_sched_isend
 #define MPIR_TSP_sched_irecv               MPII_Genutil_sched_irecv
 #define MPIR_TSP_sched_imcast              MPII_Genutil_sched_imcast
+#define MPIR_TSP_sched_issend              MPII_Genutil_sched_issend
 #define MPIR_TSP_sched_reduce_local        MPII_Genutil_sched_reduce_local
 #define MPIR_TSP_sched_localcopy           MPII_Genutil_sched_localcopy
 #define MPIR_TSP_sched_selective_sink      MPII_Genutil_sched_selective_sink
 #define MPIR_TSP_sched_sink                MPII_Genutil_sched_sink
 #define MPIR_TSP_sched_fence               MPII_Genutil_sched_fence
+#define MPIR_TSP_sched_new_type            MPII_Genutil_sched_new_type
+#define MPIR_TSP_sched_generic             MPII_Genutil_sched_generic
 #define MPIR_TSP_sched_malloc              MPII_Genutil_sched_malloc
 #define MPIR_TSP_sched_start               MPII_Genutil_sched_start
+#define MPIR_TSP_sched_free                MPII_Genutil_sched_free
 
-extern MPII_Coll_queue_t coll_queue;
+extern MPII_Coll_queue_t MPII_coll_queue;
 extern int MPII_Genutil_progress_hook_id;
 
 /* Transport function to initialize a new schedule */
 int MPII_Genutil_sched_create(MPII_Genutil_sched_t * sched);
 
-/* Transport function to schedule a isend vertex */
+/* Transport function to free a schedule */
+void MPII_Genutil_sched_free(MPII_Genutil_sched_t * sched);
+
+int MPII_Genutil_sched_new_type(MPII_Genutil_sched_t * sched, MPII_Genutil_sched_issue_fn issue_fn,
+                                MPII_Genutil_sched_complete_fn complete_fn,
+                                MPII_Genutil_sched_free_fn free_fn);
+
+int MPII_Genutil_sched_generic(int type_id, void *data,
+                               MPII_Genutil_sched_t * sched, int n_in_vtcs, int *in_vtcs,
+                               int *vtx_id);
+
+/* Transport function to schedule an isend vertex */
 int MPII_Genutil_sched_isend(const void *buf,
                              int count,
                              MPI_Datatype dt,
@@ -66,7 +78,16 @@ int MPII_Genutil_sched_isend(const void *buf,
                              MPIR_Comm * comm_ptr,
                              MPII_Genutil_sched_t * sched, int n_in_vtcs, int *in_vtcs);
 
-/* Transport function to schedule a irecv vertex */
+/* Transport function to schedule a issend vertex */
+int MPII_Genutil_sched_issend(const void *buf,
+                              int count,
+                              MPI_Datatype dt,
+                              int dest,
+                              int tag,
+                              MPIR_Comm * comm_ptr,
+                              MPII_Genutil_sched_t * sched, int n_in_vtcs, int *in_vtcs);
+
+/* Transport function to schedule an irecv vertex */
 int MPII_Genutil_sched_irecv(void *buf,
                              int count,
                              MPI_Datatype dt,
@@ -75,7 +96,7 @@ int MPII_Genutil_sched_irecv(void *buf,
                              MPIR_Comm * comm_ptr,
                              MPII_Genutil_sched_t * sched, int n_in_vtcs, int *in_vtcs);
 
-/* Transport function to schedule a imcast vertex */
+/* Transport function to schedule an imcast vertex */
 int MPII_Genutil_sched_imcast(const void *buf,
                               int count,
                               MPI_Datatype dt,
