@@ -1,8 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -28,39 +26,18 @@ int MPI_Type_indexed(int count, const int *array_of_blocklengths,
 #undef MPI_Type_indexed
 #define MPI_Type_indexed PMPI_Type_indexed
 
-/*@
-  MPIR_Type_indexed - create an indexed datatype
-
-Input Parameters:
-+ count - number of blocks in type
-. blocklength_array - number of elements in each block
-. displacement_array - offsets of blocks from start of type (see next
-  parameter for units)
-. dispinbytes - if nonzero, then displacements are in bytes (the
-  displacement_array is an array of ints), otherwise they in terms of
-  extent of oldtype (the displacement_array is an array of MPI_Aints)
-- oldtype - type (using handle) of datatype on which new type is based
-
-Output Parameters:
-. newtype - handle of new indexed datatype
-
-  Return Value:
-  0 on success, -1 on failure.
-@*/
-
 int MPIR_Type_indexed(int count,
                       const int *blocklength_array,
                       const void *displacement_array,
                       int dispinbytes, MPI_Datatype oldtype, MPI_Datatype * newtype)
 {
     int mpi_errno = MPI_SUCCESS;
-    int is_builtin, old_is_contig;
+    int old_is_contig;
     int i;
     MPI_Aint contig_count;
-    MPI_Aint el_sz, el_ct, old_ct, old_sz;
+    MPI_Aint el_ct, old_ct, old_sz;
     MPI_Aint old_lb, old_ub, old_extent, old_true_lb, old_true_ub;
     MPI_Aint min_lb = 0, max_ub = 0, eff_disp;
-    MPI_Datatype el_type;
 
     MPIR_Datatype *new_dtp;
 
@@ -69,7 +46,7 @@ int MPIR_Type_indexed(int count,
 
     /* sanity check that blocklens are all non-negative */
     for (i = 0; i < count; ++i) {
-        DLOOP_Assert(blocklength_array[i] >= 0);
+        MPIR_Assert(blocklength_array[i] >= 0);
     }
 
     /* allocate new datatype object and handle */
@@ -86,27 +63,21 @@ int MPIR_Type_indexed(int count,
 
     /* handle is filled in by MPIR_Handle_obj_alloc() */
     MPIR_Object_set_ref(new_dtp, 1);
-    new_dtp->is_permanent = 0;
     new_dtp->is_committed = 0;
     new_dtp->attributes = NULL;
-    new_dtp->cache_id = 0;
     new_dtp->name[0] = 0;
     new_dtp->contents = NULL;
+    new_dtp->flattened = NULL;
 
-    new_dtp->dataloop = NULL;
-    new_dtp->dataloop_size = -1;
-    new_dtp->dataloop_depth = -1;
+    new_dtp->typerep.handle = NULL;
 
-    is_builtin = (HANDLE_GET_KIND(oldtype) == HANDLE_KIND_BUILTIN);
-
-    if (is_builtin) {
+    if (HANDLE_IS_BUILTIN(oldtype)) {
         /* builtins are handled differently than user-defined types because
-         * they have no associated dataloop or datatype structure.
+         * they have no associated typerep or datatype structure.
          */
-        el_sz = MPIR_Datatype_get_basic_size(oldtype);
+        MPI_Aint el_sz = MPIR_Datatype_get_basic_size(oldtype);
         old_sz = el_sz;
         el_ct = 1;
-        el_type = oldtype;
 
         old_lb = 0;
         old_true_lb = 0;
@@ -115,14 +86,9 @@ int MPIR_Type_indexed(int count,
         old_extent = (MPI_Aint) el_sz;
         old_is_contig = 1;
 
-        new_dtp->has_sticky_ub = 0;
-        new_dtp->has_sticky_lb = 0;
-
         MPIR_Assign_trunc(new_dtp->alignsize, el_sz, MPI_Aint);
         new_dtp->builtin_element_size = el_sz;
-        new_dtp->basic_type = el_type;
-
-        new_dtp->max_contig_blocks = count;
+        new_dtp->basic_type = oldtype;
     } else {
         /* user-defined base type (oldtype) */
         MPIR_Datatype *old_dtp;
@@ -132,10 +98,8 @@ int MPIR_Type_indexed(int count,
         /* Ensure that "builtin_element_size" fits into an int datatype. */
         MPIR_Ensure_Aint_fits_in_int(old_dtp->builtin_element_size);
 
-        el_sz = old_dtp->builtin_element_size;
         old_sz = old_dtp->size;
         el_ct = old_dtp->n_builtin_elements;
-        el_type = old_dtp->basic_type;
 
         old_lb = old_dtp->lb;
         old_true_lb = old_dtp->true_lb;
@@ -144,17 +108,9 @@ int MPIR_Type_indexed(int count,
         old_extent = old_dtp->extent;
         MPIR_Datatype_is_contig(oldtype, &old_is_contig);
 
-        new_dtp->has_sticky_lb = old_dtp->has_sticky_lb;
-        new_dtp->has_sticky_ub = old_dtp->has_sticky_ub;
-
         new_dtp->alignsize = old_dtp->alignsize;
-        new_dtp->builtin_element_size = (MPI_Aint) el_sz;
-        new_dtp->basic_type = el_type;
-
-        new_dtp->max_contig_blocks = 0;
-        for (i = 0; i < count; i++)
-            new_dtp->max_contig_blocks
-                += old_dtp->max_contig_blocks * ((MPI_Aint) blocklength_array[i]);
+        new_dtp->builtin_element_size = old_dtp->builtin_element_size;
+        new_dtp->basic_type = old_dtp->basic_type;
     }
 
     /* find the first nonzero blocklength element */
@@ -218,24 +174,36 @@ int MPIR_Type_indexed(int count,
         MPIR_Assert(blklens != NULL);
         for (i = 0; i < count; i++)
             blklens[i] = blocklength_array[i];
-        contig_count = MPIR_Type_indexed_count_contig(count,
-                                                      blklens,
-                                                      displacement_array, dispinbytes, old_extent);
-        new_dtp->max_contig_blocks = contig_count;
+        contig_count = MPII_Datatype_indexed_count_contig(count,
+                                                          blklens,
+                                                          displacement_array, dispinbytes,
+                                                          old_extent);
         if ((contig_count == 1) && ((MPI_Aint) new_dtp->size == new_dtp->extent)) {
             new_dtp->is_contig = 1;
         }
         MPL_free(blklens);
     }
 
+    if (dispinbytes) {
+        mpi_errno =
+            MPIR_Typerep_create_hindexed(count, blocklength_array, displacement_array, oldtype,
+                                         new_dtp);
+        MPIR_ERR_CHECK(mpi_errno);
+    } else {
+        mpi_errno =
+            MPIR_Typerep_create_indexed(count, blocklength_array, displacement_array, oldtype,
+                                        new_dtp);
+        MPIR_ERR_CHECK(mpi_errno);
+    }
+
     *newtype = new_dtp->handle;
+
+  fn_exit:
     return mpi_errno;
+  fn_fail:
+    goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Type_indexed_impl
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIR_Type_indexed_impl(int count, const int *array_of_blocklengths,
                            const int *array_of_displacements,
                            MPI_Datatype oldtype, MPI_Datatype * newtype)
@@ -248,8 +216,7 @@ int MPIR_Type_indexed_impl(int count, const int *array_of_blocklengths,
 
     mpi_errno = MPIR_Type_indexed(count, array_of_blocklengths, array_of_displacements, 0,      /* displacements not in bytes */
                                   oldtype, &new_handle);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
     /* copy all integer values into a temporary buffer; this
      * includes the count, the blocklengths, and the displacements.
@@ -270,8 +237,7 @@ int MPIR_Type_indexed_impl(int count, const int *array_of_blocklengths,
                                            0,   /* aints  */
                                            1,   /* types */
                                            ints, NULL, &oldtype);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
     MPIR_OBJ_PUBLISH_HANDLE(*newtype, new_handle);
 
@@ -285,10 +251,6 @@ int MPIR_Type_indexed_impl(int count, const int *array_of_blocklengths,
 
 #endif
 
-#undef FUNCNAME
-#define FUNCNAME MPI_Type_indexed
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 /*@
     MPI_Type_indexed - Creates an indexed datatype
 
@@ -362,7 +324,7 @@ int MPI_Type_indexed(int count,
             }
             MPIR_ERRTEST_DATATYPE(oldtype, "datatype", mpi_errno);
 
-            if (HANDLE_GET_KIND(oldtype) != HANDLE_KIND_BUILTIN) {
+            if (!HANDLE_IS_BUILTIN(oldtype)) {
                 MPIR_Datatype_get_ptr(oldtype, datatype_ptr);
                 MPIR_Datatype_valid_ptr(datatype_ptr, mpi_errno);
             }
@@ -397,12 +359,12 @@ int MPI_Type_indexed(int count,
 #ifdef HAVE_ERROR_CHECKING
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpi_type_indexed", "**mpi_type_indexed %d %p %p %D %p", count,
                                  array_of_blocklengths, array_of_displacements, oldtype, newtype);
     }
 #endif
-    mpi_errno = MPIR_Err_return_comm(NULL, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(NULL, __func__, mpi_errno);
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
