@@ -1,8 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -28,10 +26,6 @@ int MPI_Comm_call_errhandler(MPI_Comm comm, int errorcode)
 
 #endif
 
-#undef FUNCNAME
-#define FUNCNAME MPI_Comm_call_errhandler
-#undef FCNAME
-#define FCNAME "MPI_Comm_call_errhandler"
 
 /*@
    MPI_Comm_call_errhandler - Call the error handler installed on a
@@ -155,8 +149,9 @@ int MPI_Comm_call_errhandler(MPI_Comm comm, int errorcode)
     /* ... end of body of routine ... */
 
   fn_exit:
-    if (in_cs)
+    if (in_cs) {
         MPID_THREAD_CS_EXIT(POBJ, MPIR_THREAD_POBJ_COMM_MUTEX(comm_ptr));
+    }
 
     MPIR_FUNC_TERSE_EXIT(MPID_STATE_MPI_COMM_CALL_ERRHANDLER);
     return mpi_errno;
@@ -166,11 +161,11 @@ int MPI_Comm_call_errhandler(MPI_Comm comm, int errorcode)
   fn_fail:
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpi_comm_call_errhandler", "**mpi_comm_call_errhandler %C %d",
                                  comm, errorcode);
     }
-    mpi_errno = MPIR_Err_return_comm(comm_ptr, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(comm_ptr, __func__, mpi_errno);
     goto fn_exit;
 #endif
     /* --END ERROR HANDLING-- */
