@@ -1,8 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -27,10 +25,6 @@ int MPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group * newgr
 #undef MPI_Group_intersection
 #define MPI_Group_intersection PMPI_Group_intersection
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Group_intersection_impl
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIR_Group_intersection_impl(MPIR_Group * group_ptr1, MPIR_Group * group_ptr2,
                                  MPIR_Group ** new_group_ptr)
 {
@@ -45,9 +39,8 @@ int MPIR_Group_intersection_impl(MPIR_Group * group_ptr1, MPIR_Group * group_ptr
     /* Insure that the lpid lists are setup */
     MPIR_Group_setup_lpid_pairs(group_ptr1, group_ptr2);
 
-    for (i = 0; i < size1; i++) {
-        group_ptr1->lrank_to_lpid[i].flag = 0;
-    }
+    int *flags = MPL_calloc(size1, sizeof(int), MPL_MEM_OTHER);
+
     g1_idx = group_ptr1->idx_of_first_lpid;
     g2_idx = group_ptr2->idx_of_first_lpid;
 
@@ -61,7 +54,7 @@ int MPIR_Group_intersection_impl(MPIR_Group * group_ptr1, MPIR_Group * group_ptr
             g2_idx = group_ptr2->lrank_to_lpid[g2_idx].next_lpid;
         } else {
             /* Equal */
-            group_ptr1->lrank_to_lpid[g1_idx].flag = 1;
+            flags[g1_idx] = 1;
             g1_idx = group_ptr1->lrank_to_lpid[g1_idx].next_lpid;
             g2_idx = group_ptr2->lrank_to_lpid[g2_idx].next_lpid;
             nnew++;
@@ -74,14 +67,13 @@ int MPIR_Group_intersection_impl(MPIR_Group * group_ptr1, MPIR_Group * group_ptr
     }
 
     mpi_errno = MPIR_Group_create(nnew, new_group_ptr);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
     (*new_group_ptr)->rank = MPI_UNDEFINED;
     (*new_group_ptr)->is_local_dense_monotonic = TRUE;
     k = 0;
     for (i = 0; i < size1; i++) {
-        if (group_ptr1->lrank_to_lpid[i].flag) {
+        if (flags[i]) {
             int lpid = group_ptr1->lrank_to_lpid[i].lpid;
             (*new_group_ptr)->lrank_to_lpid[k].lpid = lpid;
             if (i == group_ptr1->rank)
@@ -95,6 +87,8 @@ int MPIR_Group_intersection_impl(MPIR_Group * group_ptr1, MPIR_Group * group_ptr
         }
     }
 
+    MPL_free(flags);
+
   fn_exit:
     MPIR_FUNC_TERSE_EXIT(MPID_STATE_MPIR_GROUP_INTERSECTION_IMPL);
     return mpi_errno;
@@ -105,10 +99,6 @@ int MPIR_Group_intersection_impl(MPIR_Group * group_ptr1, MPIR_Group * group_ptr
 
 #endif
 
-#undef FUNCNAME
-#define FUNCNAME MPI_Group_intersection
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 
 /*@
 
@@ -203,12 +193,12 @@ int MPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group * newgr
 #ifdef HAVE_ERROR_CHECKING
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpi_group_intersection", "**mpi_group_intersection %G %G %p",
                                  group1, group2, newgroup);
     }
 #endif
-    mpi_errno = MPIR_Err_return_comm(NULL, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(NULL, __func__, mpi_errno);
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
