@@ -1,8 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -28,10 +26,6 @@ int MPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_G
 #undef MPI_Group_translate_ranks
 #define MPI_Group_translate_ranks PMPI_Group_translate_ranks
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Group_translate_ranks_impl
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIR_Group_translate_ranks_impl(MPIR_Group * gp1, int n, const int ranks1[],
                                     MPIR_Group * gp2, int ranks2[])
 {
@@ -107,10 +101,6 @@ int MPIR_Group_translate_ranks_impl(MPIR_Group * gp1, int n, const int ranks1[],
 
 #endif
 
-#undef FUNCNAME
-#define FUNCNAME MPI_Group_translate_ranks
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 /*@
  MPI_Group_translate_ranks - Translates the ranks of processes in one group to
                              those in another group
@@ -185,7 +175,7 @@ int MPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[],
                 for (i = 0; i < n; i++) {
                     if ((ranks1[i] < 0 && ranks1[i] != MPI_PROC_NULL) || ranks1[i] >= size1) {
                         mpi_errno = MPIR_Err_create_code(MPI_SUCCESS,
-                                                         MPIR_ERR_RECOVERABLE, FCNAME, __LINE__,
+                                                         MPIR_ERR_RECOVERABLE, __func__, __LINE__,
                                                          MPI_ERR_RANK,
                                                          "**rank", "**rank %d %d",
                                                          ranks1[i], size1);
@@ -202,8 +192,7 @@ int MPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[],
     /* ... body of routine ...  */
 
     mpi_errno = MPIR_Group_translate_ranks_impl(group_ptr1, n, ranks1, group_ptr2, ranks2);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
     /* ... end of body of routine ... */
 
@@ -216,12 +205,12 @@ int MPI_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[],
   fn_fail:
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpi_group_translate_ranks",
                                  "**mpi_group_translate_ranks %G %d %p %G %p", group1, n, ranks1,
                                  group2, ranks2);
     }
-    mpi_errno = MPIR_Err_return_comm(NULL, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(NULL, __func__, mpi_errno);
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
