@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -13,10 +12,10 @@
 #endif
 
 /* Preallocated window objects */
-MPIR_Win MPIR_Win_direct[MPIR_WIN_PREALLOC] = { {0}
-};
+MPIR_Win MPIR_Win_direct[MPIR_WIN_PREALLOC];
 
 MPIR_Object_alloc_t MPIR_Win_mem = { 0, 0, 0, 0, MPIR_WIN,
     sizeof(MPIR_Win), MPIR_Win_direct,
-    MPIR_WIN_PREALLOC
+    MPIR_WIN_PREALLOC,
+    NULL
 };
