@@ -1,8 +1,8 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
+
 #include "mpi.h"
 #include <stdio.h>
 #include <string.h>
@@ -556,6 +556,11 @@ int test_file(char *filename, int mynod, int nprocs, char *cb_hosts, const char
 
     MPI_CHECK(MPI_File_close(&fh));
 
+    /* delete the file */
+    if (!mynod)
+        MPI_File_delete(filename, info);
+    MPI_Barrier(MPI_COMM_WORLD);
+
     MPI_Type_free(&newtype);
     free(buf);
     if (info != MPI_INFO_NULL)
