@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -30,10 +29,6 @@ int MPI_Comm_spawn_multiple(int count, char *array_of_commands[], char **array_o
 /* Any internal routines can go here.  Make them static if possible */
 #endif
 
-#undef FUNCNAME
-#define FUNCNAME MPI_Comm_spawn_multiple
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 /*@
    MPI_Comm_spawn_multiple - short description
 
@@ -160,14 +155,14 @@ int MPI_Comm_spawn_multiple(int count, char *array_of_commands[],
 #ifdef HAVE_ERROR_CHECKING
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpi_comm_spawn_multiple",
                                  "**mpi_comm_spawn_multiple %d %p %p %p %p %d %C %p %p", count,
                                  array_of_commands, array_of_argv, array_of_maxprocs, array_of_info,
                                  root, comm, intercomm, array_of_errcodes);
     }
 #endif
-    mpi_errno = MPIR_Err_return_comm(comm_ptr, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(comm_ptr, __func__, mpi_errno);
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
