@@ -1,8 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpiimpl.h"
@@ -41,10 +39,6 @@ int MPIR_Graph_map(const MPIR_Comm * comm_ptr, int nnodes,
     return MPI_SUCCESS;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIR_Graph_map_impl
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIR_Graph_map_impl(const MPIR_Comm * comm_ptr, int nnodes,
                         const int indx[], const int edges[], int *newrank)
 {
@@ -54,14 +48,12 @@ int MPIR_Graph_map_impl(const MPIR_Comm * comm_ptr, int nnodes,
         /* --BEGIN USEREXTENSION-- */
         mpi_errno = comm_ptr->topo_fns->graphMap(comm_ptr, nnodes,
                                                  (const int *) indx, (const int *) edges, newrank);
-        if (mpi_errno)
-            MPIR_ERR_POP(mpi_errno);
+        MPIR_ERR_CHECK(mpi_errno);
         /* --END USEREXTENSION-- */
     } else {
         mpi_errno = MPIR_Graph_map(comm_ptr, nnodes,
                                    (const int *) indx, (const int *) edges, newrank);
-        if (mpi_errno)
-            MPIR_ERR_POP(mpi_errno);
+        MPIR_ERR_CHECK(mpi_errno);
     }
 
   fn_exit:
@@ -72,10 +64,6 @@ int MPIR_Graph_map_impl(const MPIR_Comm * comm_ptr, int nnodes,
 
 #endif
 
-#undef FUNCNAME
-#define FUNCNAME MPI_Graph_map
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 /*@
 MPI_Graph_map - Maps process to graph topology information
 
@@ -148,8 +136,7 @@ int MPI_Graph_map(MPI_Comm comm, int nnodes, const int indx[], const int edges[]
 
     mpi_errno = MPIR_Graph_map_impl(comm_ptr, nnodes, (const int *) indx,
                                     (const int *) edges, newrank);
-    if (mpi_errno)
-        MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
     /* ... end of body of routine ... */
 
@@ -162,12 +149,12 @@ int MPI_Graph_map(MPI_Comm comm, int nnodes, const int indx[], const int edges[]
 #ifdef HAVE_ERROR_CHECKING
     {
         mpi_errno =
-            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+            MPIR_Err_create_code(mpi_errno, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
                                  "**mpi_graph_map", "**mpi_graph_map %C %d %p %p %p", comm, nnodes,
                                  indx, edges, newrank);
     }
 #endif
-    mpi_errno = MPIR_Err_return_comm(comm_ptr, FCNAME, mpi_errno);
+    mpi_errno = MPIR_Err_return_comm(comm_ptr, __func__, mpi_errno);
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
