@@ -1,15 +1,10 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpid_nem_impl.h"
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3_Finalize
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_CH3_Finalize(void)
 {
     int mpi_errno = MPI_SUCCESS;
@@ -18,7 +13,7 @@ int MPIDI_CH3_Finalize(void)
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_CH3_FINALIZE);
 
     mpi_errno = MPIDI_CH3I_Progress_finalize();
-    if (mpi_errno) MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
     
     mpi_errno = MPID_nem_finalize();
     if (mpi_errno) MPIR_ERR_POP (mpi_errno);
