@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2006 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpid_nem_impl.h"
@@ -149,16 +148,13 @@ static int ckpt_cb(void *arg)
     return 0;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_nem_ckpt_init
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_nem_ckpt_init(void)
 {
     int mpi_errno = MPI_SUCCESS;
     cr_callback_id_t cb_id;
     cr_client_id_t client_id;
     int ret;
+    char strerrbuf[MPIR_STRERROR_BUF_SIZE];
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_NEM_CKPT_INIT);
 
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_NEM_CKPT_INIT);
@@ -170,15 +166,18 @@ int MPIDI_nem_ckpt_init(void)
     MPIR_ERR_CHKANDJUMP(client_id < 0 && errno == ENOSYS, mpi_errno, MPI_ERR_OTHER, "**blcr_mod");
 
     cb_id = cr_register_callback(ckpt_cb, NULL, CR_THREAD_CONTEXT);
-    MPIR_ERR_CHKANDJUMP1(cb_id == -1, mpi_errno, MPI_ERR_OTHER, "**intern", "**intern %s", MPIR_Strerror(errno));
+    MPIR_ERR_CHKANDJUMP1(cb_id == -1, mpi_errno, MPI_ERR_OTHER, "**intern", "**intern %s",
+                         MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
     
     checkpointing = FALSE;
     current_wave = 0;
 
     ret = sem_init(&ckpt_sem, 0, 0);
-    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_init", "**sem_init %s", MPIR_Strerror(errno));
+    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_init", "**sem_init %s",
+                         MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
     ret = sem_init(&cont_sem, 0, 0);
-    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_init", "**sem_init %s", MPIR_Strerror(errno));
+    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_init", "**sem_init %s",
+                         MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
 
  fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_NEM_CKPT_INIT);
@@ -187,22 +186,21 @@ int MPIDI_nem_ckpt_init(void)
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_nem_ckpt_finalize
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_nem_ckpt_finalize(void)
 {
     int mpi_errno = MPI_SUCCESS;
     int ret;
+    char strerrbuf[MPIR_STRERROR_BUF_SIZE];
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_NEM_CKPT_FINALIZE);
 
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_NEM_CKPT_FINALIZE);
 
     ret = sem_destroy(&ckpt_sem);
-    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_destroy", "**sem_destroy %s", MPIR_Strerror(errno));
+    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_destroy", "**sem_destroy %s",
+                         MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
     ret = sem_destroy(&cont_sem);
-    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_destroy", "**sem_destroy %s", MPIR_Strerror(errno));
+    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_destroy", "**sem_destroy %s",
+                         MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
 
  fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_NEM_CKPT_FINALIZE);
@@ -211,10 +209,6 @@ int MPIDI_nem_ckpt_finalize(void)
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME reinit_pmi
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static int reinit_pmi(void)
 {
     int ret;
@@ -269,25 +263,21 @@ static int reinit_pmi(void)
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME restore_env
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static int restore_env(pid_t parent_pid, int rank)
 {
     FILE *f;
     char env_filename[MAX_STR_LEN];
     char var_val[MAX_STR_LEN];
     int ret;
-    
+    char strerrbuf[MPIR_STRERROR_BUF_SIZE];
 
     MPL_snprintf(env_filename, MAX_STR_LEN, "/tmp/hydra-env-file-%d:%d", parent_pid, rank); 
 
     f = fopen(env_filename, "r");
-    CHECK_ERR(!f, MPIR_Strerror (errno));
+    CHECK_ERR(!f, MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
 
     ret = unlink(env_filename);
-    CHECK_ERR(ret, MPIR_Strerror (errno));
+    CHECK_ERR(ret, MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
 
     while (fgets(var_val, MAX_STR_LEN, f)) {
         size_t len = strlen(var_val);
@@ -295,11 +285,11 @@ static int restore_env(pid_t parent_pid, int rank)
         if (var_val[len-1] == '\n')
             var_val[len-1] = '\0';
         ret = MPL_putenv(MPL_strdup(var_val));
-        CHECK_ERR(ret != 0, MPIR_Strerror (errno));
+        CHECK_ERR(ret != 0, MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
     }
 
     ret = fclose(f);
-    CHECK_ERR(ret, MPIR_Strerror (errno));
+    CHECK_ERR(ret, MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
 
     return 0;
 }
@@ -313,10 +303,6 @@ typedef struct sock_ident {
     
 #define STDINOUTERR_PORT_NAME "CKPOINT_STDINOUTERR_PORT"
 
-#undef FUNCNAME
-#define FUNCNAME open_io_socket
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static int open_io_socket(socktype_t socktype, int rank, int dupfd)
 {
     int fd;
@@ -376,10 +362,6 @@ fn_exit:
     return 0;
 }
 
-#undef FUNCNAME
-#define FUNCNAME restore_stdinouterr
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static int restore_stdinouterr(int rank)
 {
     int ret;
@@ -401,10 +383,6 @@ static int restore_stdinouterr(int rank)
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_nem_ckpt_start
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_nem_ckpt_start(void)
 {
     int mpi_errno = MPI_SUCCESS;
@@ -450,7 +428,7 @@ int MPIDI_nem_ckpt_start(void)
 
         if (!vc_ch->is_local) {
             mpi_errno = vc_ch->ckpt_pause_send_vc(vc);
-            if (mpi_errno) MPIR_ERR_POP(mpi_errno);
+            MPIR_ERR_CHECK(mpi_errno);
         }
     }
     
@@ -463,15 +441,12 @@ fn_fail:
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_nem_ckpt_finish
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_nem_ckpt_finish(void)
 {
     int mpi_errno = MPI_SUCCESS;
     int i;
     int ret;
+    char strerrbuf[MPIR_STRERROR_BUF_SIZE];
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_NEM_CKPT_FINISH);
 
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_NEM_CKPT_FINISH);
@@ -480,21 +455,23 @@ int MPIDI_nem_ckpt_finish(void)
        channels between local procs), we don't have to flush those
        channels, just make sure no one is sending or receiving during
        the checkpoint */
-    mpi_errno = MPIDU_shm_barrier(MPID_nem_mem_region.barrier, MPID_nem_mem_region.num_local);
-    if (mpi_errno) MPIR_ERR_POP(mpi_errno);
+    mpi_errno = MPIDU_Init_shm_barrier();
+    MPIR_ERR_CHECK(mpi_errno);
 
     do {
         ret = sem_post(&ckpt_sem);
     } while (ret == -1 && errno == EINTR);
-    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_post", "**sem_post %s", MPIR_Strerror(errno));
+    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_post", "**sem_post %s",
+                         MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
 
     do {
         ret = sem_wait(&cont_sem);
     } while (ret == -1 && errno == EINTR);
-    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_wait", "**sem_wait %s", MPIR_Strerror(errno));
+    MPIR_ERR_CHKANDJUMP1(ret, mpi_errno, MPI_ERR_OTHER, "**sem_wait", "**sem_wait %s",
+                         MPIR_Strerror(errno, strerrbuf, MPIR_STRERROR_BUF_SIZE));
 
-    mpi_errno = MPIDU_shm_barrier(MPID_nem_mem_region.barrier, MPID_nem_mem_region.num_local);
-    if (mpi_errno) MPIR_ERR_POP(mpi_errno);
+    mpi_errno = MPIDU_Init_shm_barrier();
+    MPIR_ERR_CHECK(mpi_errno);
 
     if (ckpt_result == CKPT_CONTINUE) {
         for (i = 0; i < MPIDI_Process.my_pg->size; ++i) {
@@ -508,7 +485,7 @@ int MPIDI_nem_ckpt_finish(void)
             vc_ch = &vc->ch;
             if (!vc_ch->is_local) {
                 mpi_errno = vc_ch->ckpt_continue_vc(vc);
-                if (mpi_errno) MPIR_ERR_POP(mpi_errno);
+                MPIR_ERR_CHECK(mpi_errno);
             }
         }
     }
@@ -524,10 +501,6 @@ fn_fail:
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME pkt_ckpt_marker_handler
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static int pkt_ckpt_marker_handler(MPIDI_VC_t *vc, MPIDI_CH3_Pkt_t *pkt, void *data ATTRIBUTE((unused)),
                                    intptr_t *buflen, MPIR_Request **req)
 {
@@ -539,7 +512,7 @@ static int pkt_ckpt_marker_handler(MPIDI_VC_t *vc, MPIDI_CH3_Pkt_t *pkt, void *d
 
     if (!checkpointing) {
         mpi_errno = MPIDI_nem_ckpt_start();
-        if (mpi_errno) MPIR_ERR_POP(mpi_errno);
+        MPIR_ERR_CHECK(mpi_errno);
     }
 
     MPIR_Assert(current_wave == ckpt_pkt->wave);
@@ -568,10 +541,6 @@ fn_fail:
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPID_nem_ckpt_pkthandler_init
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_nem_ckpt_pkthandler_init(MPIDI_CH3_PktHandler_Fcn *pktArray[], int arraySize)
 {
     int mpi_errno = MPI_SUCCESS;
