@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #ifndef MPID_RMA_SHM_H_INCLUDED
@@ -12,7 +11,8 @@
 
 static inline int do_accumulate_op(void *source_buf, int source_count, MPI_Datatype source_dtp,
                                    void *target_buf, int target_count, MPI_Datatype target_dtp,
-                                   MPI_Aint stream_offset, MPI_Op acc_op);
+                                   MPI_Aint stream_offset, MPI_Op acc_op,
+                                   MPIDI_RMA_Acc_srcbuf_kind_t srckind);
 
 #define ASSIGN_COPY(src, dest, count, type)     \
     {                                           \
@@ -223,11 +223,6 @@ static inline int shm_copy(const void *src, int scount, MPI_Datatype stype,
         case MPI_2INTEGER:
 #endif
 
-#if 0   /* Random types not present in the standard */
-        case MPI_2COMPLEX:
-        case MPI_2DOUBLE_COMPLEX:
-#endif
-
         default:
             /* Just to make sure the switch statement is not empty */
             ;
@@ -235,9 +230,7 @@ static inline int shm_copy(const void *src, int scount, MPI_Datatype stype,
     }
 
     mpi_errno = MPIR_Localcopy(src, scount, stype, dest, dcount, dtype);
-    if (mpi_errno) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
     return mpi_errno;
@@ -247,10 +240,6 @@ static inline int shm_copy(const void *src, int scount, MPI_Datatype stype,
     /* --END ERROR HANDLING-- */
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3I_Shm_put_op
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static inline int MPIDI_CH3I_Shm_put_op(const void *origin_addr, int origin_count, MPI_Datatype
                                         origin_datatype, int target_rank, MPI_Aint target_disp,
                                         int target_count, MPI_Datatype target_datatype,
@@ -276,9 +265,7 @@ static inline int MPIDI_CH3I_Shm_put_op(const void *origin_addr, int origin_coun
 
     mpi_errno = shm_copy(origin_addr, origin_count, origin_datatype,
                          (char *) base + disp_unit * target_disp, target_count, target_datatype);
-    if (mpi_errno) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
     MPIR_FUNC_VERBOSE_RMA_EXIT(MPID_STATE_MPIDI_CH3I_SHM_PUT_OP);
@@ -290,10 +277,6 @@ static inline int MPIDI_CH3I_Shm_put_op(const void *origin_addr, int origin_coun
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3I_Shm_acc_op
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static inline int MPIDI_CH3I_Shm_acc_op(const void *origin_addr, int origin_count, MPI_Datatype
                                         origin_datatype, int target_rank, MPI_Aint target_disp,
                                         int target_count, MPI_Datatype target_datatype, MPI_Op op,
@@ -331,13 +314,13 @@ static inline int MPIDI_CH3I_Shm_acc_op(const void *origin_addr, int origin_coun
         }
         mpi_errno = do_accumulate_op((void *) origin_addr, origin_count, origin_datatype,
                                      (void *) ((char *) base + disp_unit * target_disp),
-                                     target_count, target_datatype, 0, op);
+                                     target_count, target_datatype, 0, op,
+                                     MPIDI_RMA_ACC_SRCBUF_DEFAULT);
         if (shm_op) {
             MPIDI_CH3I_SHM_MUTEX_UNLOCK(win_ptr);
         }
 
-        if (mpi_errno != MPI_SUCCESS)
-            MPIR_ERR_POP(mpi_errno);
+        MPIR_ERR_CHECK(mpi_errno);
 
         goto fn_exit;
     }
@@ -360,10 +343,7 @@ static inline int MPIDI_CH3I_Shm_acc_op(const void *origin_addr, int origin_coun
 
     rest_len = total_len;
     for (i = 0; i < stream_unit_count; i++) {
-        MPIR_Segment *seg = NULL;
         void *packed_buf = NULL;
-        MPI_Aint first, last;
-        int is_predef_contig;
         MPI_Aint stream_offset, stream_size, stream_count;
 
         stream_offset = i * stream_elem_count * predefined_dtp_size;
@@ -371,29 +351,12 @@ static inline int MPIDI_CH3I_Shm_acc_op(const void *origin_addr, int origin_coun
         stream_count = stream_size / predefined_dtp_size;
         rest_len -= stream_size;
 
-        first = stream_offset;
-        last = stream_offset + stream_size;
-
         packed_buf = MPL_malloc(stream_size, MPL_MEM_BUFFER);
 
-        seg = MPIR_Segment_alloc();
-        MPIR_ERR_CHKANDJUMP1(seg == NULL, mpi_errno, MPI_ERR_OTHER, "**nomem", "**nomem %s",
-                             "MPIR_Segment");
-        MPIR_Segment_init(origin_addr, origin_count, origin_datatype, seg);
-        MPIR_Segment_pack(seg, first, &last, packed_buf);
-        MPIR_Segment_free(seg);
-
-        MPIR_Datatype_is_contig(basic_type, &is_predef_contig);
-
-        if (!is_predef_contig) {
-            void *tmpbuf = MPL_malloc(stream_count * predefined_dtp_extent, MPL_MEM_BUFFER);
-            mpi_errno = MPIR_Localcopy(tmpbuf, stream_count, basic_type,
-                                       packed_buf, stream_size, MPI_BYTE);
-            if (mpi_errno != MPI_SUCCESS)
-                MPIR_ERR_POP(mpi_errno);
-            MPL_free(packed_buf);
-            packed_buf = tmpbuf;
-        }
+        MPI_Aint actual_pack_bytes;
+        MPIR_Typerep_pack(origin_addr, origin_count, origin_datatype,
+                       stream_offset, packed_buf, stream_size, &actual_pack_bytes);
+        MPIR_Assert(actual_pack_bytes == stream_size);
 
         if (shm_op) {
             MPIDI_CH3I_SHM_MUTEX_LOCK(win_ptr);
@@ -402,14 +365,14 @@ static inline int MPIDI_CH3I_Shm_acc_op(const void *origin_addr, int origin_coun
         MPIR_Assert(stream_count == (int) stream_count);
         mpi_errno = do_accumulate_op((void *) packed_buf, (int) stream_count, basic_type,
                                      (void *) ((char *) base + disp_unit * target_disp),
-                                     target_count, target_datatype, stream_offset, op);
+                                     target_count, target_datatype, stream_offset, op,
+                                     MPIDI_RMA_ACC_SRCBUF_PACKED);
 
         if (shm_op) {
             MPIDI_CH3I_SHM_MUTEX_UNLOCK(win_ptr);
         }
 
-        if (mpi_errno != MPI_SUCCESS)
-            MPIR_ERR_POP(mpi_errno);
+        MPIR_ERR_CHECK(mpi_errno);
 
         MPL_free(packed_buf);
     }
@@ -424,10 +387,6 @@ static inline int MPIDI_CH3I_Shm_acc_op(const void *origin_addr, int origin_coun
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3I_Shm_get_acc_op
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static inline int MPIDI_CH3I_Shm_get_acc_op(const void *origin_addr, int origin_count, MPI_Datatype
                                             origin_datatype, void *result_addr, int result_count,
                                             MPI_Datatype result_datatype, int target_rank, MPI_Aint
@@ -470,21 +429,19 @@ static inline int MPIDI_CH3I_Shm_get_acc_op(const void *origin_addr, int origin_
     /* Perform the local get first, then the accumulate */
     mpi_errno = shm_copy((char *) base + disp_unit * target_disp, target_count, target_datatype,
                          result_addr, result_count, result_datatype);
-    if (mpi_errno) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
 
     if (is_empty_origin == TRUE || MPIR_DATATYPE_IS_PREDEFINED(origin_datatype)) {
 
         mpi_errno = do_accumulate_op((void *) origin_addr, origin_count, origin_datatype,
                                      (void *) ((char *) base + disp_unit * target_disp),
-                                     target_count, target_datatype, 0, op);
+                                     target_count, target_datatype, 0, op,
+                                     MPIDI_RMA_ACC_SRCBUF_DEFAULT);
         if (shm_locked) {
             MPIDI_CH3I_SHM_MUTEX_UNLOCK(win_ptr);
         }
 
-        if (mpi_errno != MPI_SUCCESS)
-            MPIR_ERR_POP(mpi_errno);
+        MPIR_ERR_CHECK(mpi_errno);
 
         goto fn_exit;
     }
@@ -507,10 +464,7 @@ static inline int MPIDI_CH3I_Shm_get_acc_op(const void *origin_addr, int origin_
 
     rest_len = total_len;
     for (i = 0; i < stream_unit_count; i++) {
-        MPIR_Segment *seg = NULL;
         void *packed_buf = NULL;
-        MPI_Aint first, last;
-        int is_predef_contig;
         MPI_Aint stream_offset, stream_size, stream_count;
 
         stream_offset = i * stream_elem_count * predefined_dtp_size;
@@ -518,37 +472,20 @@ static inline int MPIDI_CH3I_Shm_get_acc_op(const void *origin_addr, int origin_
         stream_count = stream_size / predefined_dtp_size;
         rest_len -= stream_size;
 
-        first = stream_offset;
-        last = stream_offset + stream_size;
-
         packed_buf = MPL_malloc(stream_size, MPL_MEM_BUFFER);
 
-        seg = MPIR_Segment_alloc();
-        MPIR_ERR_CHKANDJUMP1(seg == NULL, mpi_errno, MPI_ERR_OTHER, "**nomem", "**nomem %s",
-                             "MPIR_Segment");
-        MPIR_Segment_init(origin_addr, origin_count, origin_datatype, seg);
-        MPIR_Segment_pack(seg, first, &last, packed_buf);
-        MPIR_Segment_free(seg);
-
-        MPIR_Datatype_is_contig(basic_type, &is_predef_contig);
-
-        if (!is_predef_contig) {
-            void *tmpbuf = MPL_malloc(stream_count * predefined_dtp_extent, MPL_MEM_BUFFER);
-            mpi_errno = MPIR_Localcopy(tmpbuf, stream_count, basic_type,
-                                       packed_buf, stream_size, MPI_BYTE);
-            if (mpi_errno != MPI_SUCCESS)
-                MPIR_ERR_POP(mpi_errno);
-            MPL_free(packed_buf);
-            packed_buf = tmpbuf;
-        }
+        MPI_Aint actual_pack_bytes;
+        MPIR_Typerep_pack(origin_addr, origin_count, origin_datatype,
+                       stream_offset, packed_buf, stream_size, &actual_pack_bytes);
+        MPIR_Assert(actual_pack_bytes == stream_size);
 
         MPIR_Assert(stream_count == (int) stream_count);
         mpi_errno = do_accumulate_op((void *) packed_buf, (int) stream_count, basic_type,
                                      (void *) ((char *) base + disp_unit * target_disp),
-                                     target_count, target_datatype, stream_offset, op);
+                                     target_count, target_datatype, stream_offset, op,
+                                     MPIDI_RMA_ACC_SRCBUF_PACKED);
 
-        if (mpi_errno != MPI_SUCCESS)
-            MPIR_ERR_POP(mpi_errno);
+        MPIR_ERR_CHECK(mpi_errno);
 
         MPL_free(packed_buf);
     }
@@ -571,10 +508,6 @@ static inline int MPIDI_CH3I_Shm_get_acc_op(const void *origin_addr, int origin_
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3I_Shm_get_op
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static inline int MPIDI_CH3I_Shm_get_op(void *origin_addr, int origin_count,
                                         MPI_Datatype origin_datatype, int target_rank,
                                         MPI_Aint target_disp, int target_count,
@@ -600,9 +533,7 @@ static inline int MPIDI_CH3I_Shm_get_op(void *origin_addr, int origin_count,
 
     mpi_errno = shm_copy((char *) base + disp_unit * target_disp, target_count, target_datatype,
                          origin_addr, origin_count, origin_datatype);
-    if (mpi_errno) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
     MPIR_FUNC_VERBOSE_RMA_EXIT(MPID_STATE_MPIDI_CH3I_SHM_GET_OP);
@@ -614,10 +545,6 @@ static inline int MPIDI_CH3I_Shm_get_op(void *origin_addr, int origin_count,
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3I_Shm_cas_op
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static inline int MPIDI_CH3I_Shm_cas_op(const void *origin_addr, const void *compare_addr,
                                         void *result_addr, MPI_Datatype datatype, int target_rank,
                                         MPI_Aint target_disp, MPIR_Win * win_ptr)
@@ -672,10 +599,6 @@ static inline int MPIDI_CH3I_Shm_cas_op(const void *origin_addr, const void *com
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3I_Shm_fop_op
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static inline int MPIDI_CH3I_Shm_fop_op(const void *origin_addr, void *result_addr,
                                         MPI_Datatype datatype, int target_rank,
                                         MPI_Aint target_disp, MPI_Op op, MPIR_Win * win_ptr)
@@ -690,6 +613,9 @@ static inline int MPIDI_CH3I_Shm_fop_op(const void *origin_addr, void *result_ad
 
     MPIR_FUNC_VERBOSE_RMA_ENTER(MPID_STATE_MPIDI_CH3I_SHM_FOP_OP);
 
+    if ((*MPIR_OP_HDL_TO_DTYPE_FN(op)) (datatype) != MPI_SUCCESS)
+        goto fn_exit;
+
     if (win_ptr->shm_allocated == TRUE) {
         int local_target_rank = win_ptr->comm_ptr->intranode_table[target_rank];
         MPIR_Assert(local_target_rank >= 0);
