@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #ifndef MPIDPKT_H_INCLUDED
@@ -132,7 +131,7 @@ typedef enum {
     MPIDI_CH3_PKT_INVALID = -1  /* forces a signed enum to quash warnings */
 } MPIDI_CH3_Pkt_type_t;
 
-/* These flags can be "OR'ed" together */
+/* These pkt_flags can be "OR'ed" together */
 typedef enum {
     MPIDI_CH3_PKT_FLAG_NONE = 0,
     MPIDI_CH3_PKT_FLAG_RMA_LOCK_SHARED = 1,
@@ -325,7 +324,7 @@ MPIDI_CH3_PKT_DEFS
 
 #define MPIDI_CH3_PKT_RMA_GET_FLAGS(pkt_, flags_, err_)                 \
     {                                                                   \
-        /* This macro returns flags in RMA operation packets (PUT, GET, \
+        /* This macro returns pkt_flags in RMA operation packets (PUT, GET, \
            ACC, GACC, FOP, CAS), RMA operation response packets         \
            (GET_RESP, GET_ACCUM_RESP, FOP_RESP, CAS_RESP), RMA control  \
            packets (UNLOCK) and RMA control response packets (LOCK_ACK, \
@@ -334,52 +333,52 @@ MPIDI_CH3_PKT_DEFS
         switch((pkt_).type) {                                           \
         case (MPIDI_CH3_PKT_PUT):                                       \
         case (MPIDI_CH3_PKT_PUT_IMMED):                                 \
-            flags_ = (pkt_).put.flags;                                  \
+            flags_ = (pkt_).put.pkt_flags;                                  \
             break;                                                      \
         case (MPIDI_CH3_PKT_GET):                                       \
-            flags_ = (pkt_).get.flags;                                  \
+            flags_ = (pkt_).get.pkt_flags;                                  \
             break;                                                      \
         case (MPIDI_CH3_PKT_ACCUMULATE):                                \
         case (MPIDI_CH3_PKT_ACCUMULATE_IMMED):                          \
-            flags_ = (pkt_).accum.flags;                                \
+            flags_ = (pkt_).accum.pkt_flags;                                \
             break;                                                      \
         case (MPIDI_CH3_PKT_GET_ACCUM):                                 \
         case (MPIDI_CH3_PKT_GET_ACCUM_IMMED):                           \
-            flags_ = (pkt_).get_accum.flags;                            \
+            flags_ = (pkt_).get_accum.pkt_flags;                            \
             break;                                                      \
         case (MPIDI_CH3_PKT_CAS_IMMED):                                 \
-            flags_ = (pkt_).cas.flags;                                  \
+            flags_ = (pkt_).cas.pkt_flags;                                  \
             break;                                                      \
         case (MPIDI_CH3_PKT_FOP):                                       \
         case (MPIDI_CH3_PKT_FOP_IMMED):                                 \
-            flags_ = (pkt_).fop.flags;                                  \
+            flags_ = (pkt_).fop.pkt_flags;                                  \
             break;                                                      \
         case (MPIDI_CH3_PKT_GET_RESP):                                  \
         case (MPIDI_CH3_PKT_GET_RESP_IMMED):                            \
-            flags_ = (pkt_).get_resp.flags;                             \
+            flags_ = (pkt_).get_resp.pkt_flags;                             \
             break;                                                      \
         case (MPIDI_CH3_PKT_GET_ACCUM_RESP):                            \
         case (MPIDI_CH3_PKT_GET_ACCUM_RESP_IMMED):                      \
-            flags_ = (pkt_).get_accum_resp.flags;                       \
+            flags_ = (pkt_).get_accum_resp.pkt_flags;                       \
             break;                                                      \
         case (MPIDI_CH3_PKT_FOP_RESP):                                  \
         case (MPIDI_CH3_PKT_FOP_RESP_IMMED):                            \
-            flags_ = (pkt_).fop_resp.flags;                             \
+            flags_ = (pkt_).fop_resp.pkt_flags;                             \
             break;                                                      \
         case (MPIDI_CH3_PKT_CAS_RESP_IMMED):                            \
-            flags_ = (pkt_).cas_resp.flags;                             \
+            flags_ = (pkt_).cas_resp.pkt_flags;                             \
             break;                                                      \
         case (MPIDI_CH3_PKT_LOCK):                                      \
-            flags_ = (pkt_).lock.flags;                                 \
+            flags_ = (pkt_).lock.pkt_flags;                                 \
             break;                                                      \
         case (MPIDI_CH3_PKT_UNLOCK):                                    \
-            flags_ = (pkt_).unlock.flags;                               \
+            flags_ = (pkt_).unlock.pkt_flags;                               \
             break;                                                      \
         case (MPIDI_CH3_PKT_LOCK_ACK):                                  \
-            flags_ = (pkt_).lock_ack.flags;                             \
+            flags_ = (pkt_).lock_ack.pkt_flags;                             \
             break;                                                      \
         case (MPIDI_CH3_PKT_LOCK_OP_ACK):                               \
-            flags_ = (pkt_).lock_op_ack.flags;                          \
+            flags_ = (pkt_).lock_op_ack.pkt_flags;                          \
             break;                                                      \
         default:                                                        \
             MPIR_ERR_SETANDJUMP1(err_, MPI_ERR_OTHER, "**invalidpkt", "**invalidpkt %d", (pkt_).type); \
@@ -411,7 +410,7 @@ MPIDI_CH3_PKT_DEFS
 
 #define MPIDI_CH3_PKT_RMA_ERASE_FLAGS(pkt_, err_)                       \
     {                                                                   \
-        /* This macro erases flags in RMA operation packets (PUT, GET,  \
+        /* This macro erases pkt_flags in RMA operation packets (PUT, GET,  \
            ACC, GACC, FOP, CAS), RMA operation response packets         \
            (GET_RESP, GET_ACCUM_RESP, FOP_RESP, CAS_RESP), RMA control  \
            packets (UNLOCK) and RMA control response packets (LOCK_ACK, \
@@ -420,52 +419,52 @@ MPIDI_CH3_PKT_DEFS
         switch((pkt_).type) {                                           \
         case (MPIDI_CH3_PKT_PUT):                                       \
         case (MPIDI_CH3_PKT_PUT_IMMED):                                 \
-            (pkt_).put.flags = MPIDI_CH3_PKT_FLAG_NONE;                 \
+            (pkt_).put.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;                 \
             break;                                                      \
         case (MPIDI_CH3_PKT_GET):                                       \
-            (pkt_).get.flags = MPIDI_CH3_PKT_FLAG_NONE;                 \
+            (pkt_).get.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;                 \
             break;                                                      \
         case (MPIDI_CH3_PKT_ACCUMULATE):                                \
         case (MPIDI_CH3_PKT_ACCUMULATE_IMMED):                          \
-            (pkt_).accum.flags = MPIDI_CH3_PKT_FLAG_NONE;               \
+            (pkt_).accum.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;               \
             break;                                                      \
         case (MPIDI_CH3_PKT_GET_ACCUM):                                 \
         case (MPIDI_CH3_PKT_GET_ACCUM_IMMED):                           \
-            (pkt_).get_accum.flags = MPIDI_CH3_PKT_FLAG_NONE;           \
+            (pkt_).get_accum.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;           \
             break;                                                      \
         case (MPIDI_CH3_PKT_CAS_IMMED):                                 \
-            (pkt_).cas.flags = MPIDI_CH3_PKT_FLAG_NONE;                 \
+            (pkt_).cas.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;                 \
             break;                                                      \
         case (MPIDI_CH3_PKT_FOP):                                       \
         case (MPIDI_CH3_PKT_FOP_IMMED):                                 \
-            (pkt_).fop.flags = MPIDI_CH3_PKT_FLAG_NONE;                 \
+            (pkt_).fop.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;                 \
             break;                                                      \
         case (MPIDI_CH3_PKT_GET_RESP):                                  \
         case (MPIDI_CH3_PKT_GET_RESP_IMMED):                            \
-            (pkt_).get_resp.flags = MPIDI_CH3_PKT_FLAG_NONE;            \
+            (pkt_).get_resp.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;            \
             break;                                                      \
         case (MPIDI_CH3_PKT_GET_ACCUM_RESP):                            \
         case (MPIDI_CH3_PKT_GET_ACCUM_RESP_IMMED):                      \
-            (pkt_).get_accum_resp.flags = MPIDI_CH3_PKT_FLAG_NONE;      \
+            (pkt_).get_accum_resp.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;      \
             break;                                                      \
         case (MPIDI_CH3_PKT_FOP_RESP):                                  \
         case (MPIDI_CH3_PKT_FOP_RESP_IMMED):                            \
-            (pkt_).fop_resp.flags = MPIDI_CH3_PKT_FLAG_NONE;            \
+            (pkt_).fop_resp.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;            \
             break;                                                      \
         case (MPIDI_CH3_PKT_CAS_RESP_IMMED):                            \
-            (pkt_).cas_resp.flags = MPIDI_CH3_PKT_FLAG_NONE;            \
+            (pkt_).cas_resp.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;            \
             break;                                                      \
         case (MPIDI_CH3_PKT_LOCK):                                      \
-            (pkt_).lock.flags = MPIDI_CH3_PKT_FLAG_NONE;                \
+            (pkt_).lock.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;                \
             break;                                                      \
         case (MPIDI_CH3_PKT_UNLOCK):                                    \
-            (pkt_).unlock.flags = MPIDI_CH3_PKT_FLAG_NONE;              \
+            (pkt_).unlock.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;              \
             break;                                                      \
         case (MPIDI_CH3_PKT_LOCK_ACK):                                  \
-            (pkt_).lock_ack.flags = MPIDI_CH3_PKT_FLAG_NONE;            \
+            (pkt_).lock_ack.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;            \
             break;                                                      \
         case (MPIDI_CH3_PKT_LOCK_OP_ACK):                               \
-            (pkt_).lock_op_ack.flags = MPIDI_CH3_PKT_FLAG_NONE;         \
+            (pkt_).lock_op_ack.pkt_flags = MPIDI_CH3_PKT_FLAG_NONE;         \
             break;                                                      \
         default:                                                        \
             MPIR_ERR_SETANDJUMP1(err_, MPI_ERR_OTHER, "**invalidpkt", "**invalidpkt %d", (pkt_).type); \
@@ -558,29 +557,6 @@ MPIDI_CH3_PKT_DEFS
         }                                                               \
     }
 
-#define MPIDI_CH3_PKT_RMA_SET_DATALOOP_SIZE(pkt_, dataloop_size_, err_) \
-    {                                                                   \
-        /* This macro sets dataloop_size in RMA operation packets       \
-           (PUT, GET, ACC, GACC) */                                     \
-        err_ = MPI_SUCCESS;                                             \
-        switch((pkt_).type) {                                           \
-        case (MPIDI_CH3_PKT_PUT):                                       \
-            (pkt_).put.info.dataloop_size = (dataloop_size_);           \
-            break;                                                      \
-        case (MPIDI_CH3_PKT_GET):                                       \
-            (pkt_).get.info.dataloop_size = (dataloop_size_);           \
-            break;                                                      \
-        case (MPIDI_CH3_PKT_ACCUMULATE):                                \
-            (pkt_).accum.info.dataloop_size = (dataloop_size_);         \
-            break;                                                      \
-        case (MPIDI_CH3_PKT_GET_ACCUM):                                 \
-            (pkt_).get_accum.info.dataloop_size = (dataloop_size_);     \
-            break;                                                      \
-        default:                                                        \
-            MPIR_ERR_SETANDJUMP1(err_, MPI_ERR_OTHER, "**invalidpkt", "**invalidpkt %d", (pkt_).type); \
-        }                                                               \
-    }
-
 #define MPIDI_CH3_PKT_RMA_GET_REQUEST_HANDLE(pkt_, request_hdl_, err_)  \
     {                                                                   \
         err_ = MPI_SUCCESS;                                             \
@@ -650,26 +626,26 @@ MPIDI_CH3_PKT_DEFS
 
 typedef struct MPIDI_CH3_Pkt_put {
     MPIDI_CH3_Pkt_type_t type;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     void *addr;
     int count;
     MPI_Datatype datatype;
     MPI_Win target_win_handle;
     MPI_Win source_win_handle;
     union {
-        int dataloop_size;
+        int flattened_type_size;
         MPIDI_CH3_RMA_Immed_u data;
     } info;
 } MPIDI_CH3_Pkt_put_t;
 
 typedef struct MPIDI_CH3_Pkt_get {
     MPIDI_CH3_Pkt_type_t type;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     void *addr;
     int count;
     MPI_Datatype datatype;
     struct {
-        int dataloop_size;      /* for derived datatypes */
+        int flattened_type_size;
     } info;
     MPI_Request request_handle;
     MPI_Win target_win_handle;
@@ -680,7 +656,7 @@ typedef struct MPIDI_CH3_Pkt_get_resp {
     MPI_Request request_handle;
     /* followings are used to decrement ack_counter at origin */
     int target_rank;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     /* Followings are to piggyback IMMED data */
     struct {
         /* note that we use struct here in order
@@ -692,7 +668,7 @@ typedef struct MPIDI_CH3_Pkt_get_resp {
 
 typedef struct MPIDI_CH3_Pkt_accum {
     MPIDI_CH3_Pkt_type_t type;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     void *addr;
     int count;
     MPI_Datatype datatype;
@@ -700,14 +676,14 @@ typedef struct MPIDI_CH3_Pkt_accum {
     MPI_Win target_win_handle;
     MPI_Win source_win_handle;
     union {
-        int dataloop_size;
+        int flattened_type_size;
         MPIDI_CH3_RMA_Immed_u data;
     } info;
 } MPIDI_CH3_Pkt_accum_t;
 
 typedef struct MPIDI_CH3_Pkt_get_accum {
     MPIDI_CH3_Pkt_type_t type;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     MPI_Request request_handle; /* For get_accumulate response */
     void *addr;
     int count;
@@ -715,7 +691,7 @@ typedef struct MPIDI_CH3_Pkt_get_accum {
     MPI_Op op;
     MPI_Win target_win_handle;
     union {
-        int dataloop_size;
+        int flattened_type_size;
         MPIDI_CH3_RMA_Immed_u data;
     } info;
 } MPIDI_CH3_Pkt_get_accum_t;
@@ -725,7 +701,7 @@ typedef struct MPIDI_CH3_Pkt_get_accum_resp {
     MPI_Request request_handle;
     /* followings are used to decrement ack_counter at origin */
     int target_rank;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     /* Followings are to piggyback IMMED data */
     struct {
         /* note that we use struct here in order
@@ -737,7 +713,7 @@ typedef struct MPIDI_CH3_Pkt_get_accum_resp {
 
 typedef struct MPIDI_CH3_Pkt_cas {
     MPIDI_CH3_Pkt_type_t type;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     MPI_Datatype datatype;
     void *addr;
     MPI_Request request_handle;
@@ -757,12 +733,12 @@ typedef struct MPIDI_CH3_Pkt_cas_resp {
     } info;
     /* followings are used to decrement ack_counter at orign */
     int target_rank;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
 } MPIDI_CH3_Pkt_cas_resp_t;
 
 typedef struct MPIDI_CH3_Pkt_fop {
     MPIDI_CH3_Pkt_type_t type;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     MPI_Datatype datatype;
     void *addr;
     MPI_Op op;
@@ -787,12 +763,12 @@ typedef struct MPIDI_CH3_Pkt_fop_resp {
     } info;
     /* followings are used to decrement ack_counter at orign */
     int target_rank;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
 } MPIDI_CH3_Pkt_fop_resp_t;
 
 typedef struct MPIDI_CH3_Pkt_lock {
     MPIDI_CH3_Pkt_type_t type;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     MPI_Win target_win_handle;
     /* Note that either source_win_handle
      * or request_handle will be used. Here
@@ -808,7 +784,7 @@ typedef struct MPIDI_CH3_Pkt_unlock {
     MPIDI_CH3_Pkt_type_t type;
     MPI_Win target_win_handle;
     MPI_Win source_win_handle;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
 } MPIDI_CH3_Pkt_unlock_t;
 
 typedef struct MPIDI_CH3_Pkt_flush {
@@ -819,7 +795,7 @@ typedef struct MPIDI_CH3_Pkt_flush {
 
 typedef struct MPIDI_CH3_Pkt_lock_ack {
     MPIDI_CH3_Pkt_type_t type;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     /* note that either source_win_handle
      * or request_handle is used. */
     MPI_Win source_win_handle;
@@ -829,7 +805,7 @@ typedef struct MPIDI_CH3_Pkt_lock_ack {
 
 typedef struct MPIDI_CH3_Pkt_lock_op_ack {
     MPIDI_CH3_Pkt_type_t type;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
     /* note that either source_win_handle
      * or request_handle is used. */
     MPI_Win source_win_handle;
@@ -844,14 +820,14 @@ typedef struct MPIDI_CH3_Pkt_ack {
     MPIDI_CH3_Pkt_type_t type;
     MPI_Win source_win_handle;
     int target_rank;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
 } MPIDI_CH3_Pkt_ack_t;
 
 typedef struct MPIDI_CH3_Pkt_decr_at_counter {
     MPIDI_CH3_Pkt_type_t type;
     MPI_Win target_win_handle;
     MPI_Win source_win_handle;
-    MPIDI_CH3_Pkt_flags_t flags;
+    int pkt_flags;
 } MPIDI_CH3_Pkt_decr_at_counter_t;
 
 typedef struct MPIDI_CH3_Pkt_close {
@@ -918,59 +894,10 @@ typedef union MPIDI_CH3_Pkt {
 
 /* Extended header packet types */
 
-/* to send derived datatype across in RMA ops */
-typedef struct MPIDI_RMA_dtype_info {   /* for derived datatypes */
-    int is_contig;
-    MPI_Aint max_contig_blocks;
-    MPI_Aint size;
-    MPI_Aint extent;
-    MPI_Aint dataloop_size;     /* not needed because this info is sent in
-                                 * packet header. remove it after lock/unlock
-                                 * is implemented in the device */
-    void *dataloop;             /* pointer needed to update pointers
-                                 * within dataloop on remote side */
-    int dataloop_depth;
-    int basic_type;
-    MPI_Aint ub, lb, true_ub, true_lb;
-    int has_sticky_ub, has_sticky_lb;
-} MPIDI_RMA_dtype_info;
-
 typedef struct MPIDI_CH3_Ext_pkt_stream {
     MPI_Aint stream_offset;
 } MPIDI_CH3_Ext_pkt_stream_t;
 
-typedef struct MPIDI_CH3_Ext_pkt_derived {
-    MPIDI_RMA_dtype_info dtype_info;
-    /* Follow with variable-length dataloop.
-     * On origin we allocate a large buffer including
-     * this header and the dataloop; on target we use
-     * separate buffer to receive dataloop in order
-     * to avoid extra copy.*/
-} MPIDI_CH3_Ext_pkt_derived_t;
-
-typedef struct MPIDI_CH3_Ext_pkt_stream_derived {
-    MPI_Aint stream_offset;
-    MPIDI_RMA_dtype_info dtype_info;
-    /* follow with variable-length dataloop. */
-} MPIDI_CH3_Ext_pkt_stream_derived_t;
-
-/* Note that since ACC and GET_ACC contain the same extended attributes,
- * we use generic routines for them in some places (see below).
- * If we add OP-specific attribute in future, we should handle them separately.
- *  1. origin issuing function
- *  2. target packet handler
- *  3. target data receive complete handler. */
-typedef MPIDI_CH3_Ext_pkt_stream_t MPIDI_CH3_Ext_pkt_accum_stream_t;
-typedef MPIDI_CH3_Ext_pkt_derived_t MPIDI_CH3_Ext_pkt_accum_derived_t;
-typedef MPIDI_CH3_Ext_pkt_stream_derived_t MPIDI_CH3_Ext_pkt_accum_stream_derived_t;
-
-typedef MPIDI_CH3_Ext_pkt_stream_t MPIDI_CH3_Ext_pkt_get_accum_stream_t;
-typedef MPIDI_CH3_Ext_pkt_derived_t MPIDI_CH3_Ext_pkt_get_accum_derived_t;
-typedef MPIDI_CH3_Ext_pkt_stream_derived_t MPIDI_CH3_Ext_pkt_get_accum_stream_derived_t;
-
-typedef MPIDI_CH3_Ext_pkt_derived_t MPIDI_CH3_Ext_pkt_put_derived_t;
-typedef MPIDI_CH3_Ext_pkt_derived_t MPIDI_CH3_Ext_pkt_get_derived_t;
-
 #if defined(MPID_USE_SEQUENCE_NUMBERS)
 typedef struct MPIDI_CH3_Pkt_send_container {
     MPIDI_CH3_Pkt_send_t pkt;
