@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpidimpl.h"
@@ -20,14 +19,10 @@
 
 /* Max depth of recursive calls of MPID_Request_complete */
 #define REQUEST_CB_DEPTH 2
-#define MPIDI_LOAD_RECV_IOV_ORIG_SEGMENT_FIRST_UNSET (-1)
+#define MPIDI_LOAD_RECV_IOV_ORIG_MSG_OFFSET_UNSET (-1)
 
 /* See the comments above about request creation.  Some routines will
    use macros in mpidimpl.h *instead* of this routine */
-#undef FUNCNAME
-#define FUNCNAME MPID_Request_create_hook
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 void MPID_Request_create_hook(MPIR_Request *req)
 {
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPID_REQUEST_INIT);
@@ -35,8 +30,8 @@ void MPID_Request_create_hook(MPIR_Request *req)
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPID_REQUEST_INIT);
     
     req->dev.datatype_ptr	   = NULL;
-    req->dev.segment_ptr	   = NULL;
-    /* Masks and flags for channel device state in an MPIR_Request */
+    req->dev.msg_offset         = 0;
+    /* Masks and pkt_flags for channel device state in an MPIR_Request */
     req->dev.state		   = 0;
     req->dev.cancel_pending	   = FALSE;
     /* FIXME: RMA ops shouldn't need to be set except when creating a
@@ -44,9 +39,9 @@ void MPID_Request_create_hook(MPIR_Request *req)
     req->dev.target_win_handle = MPI_WIN_NULL;
     req->dev.source_win_handle = MPI_WIN_NULL;
     req->dev.target_lock_queue_entry = NULL;
-    req->dev.dataloop	   = NULL;
+    req->dev.flattened_type = NULL;
     req->dev.iov_offset        = 0;
-    req->dev.flags             = MPIDI_CH3_PKT_FLAG_NONE;
+    req->dev.pkt_flags             = MPIDI_CH3_PKT_FLAG_NONE;
     req->dev.resp_request_handle = MPI_REQUEST_NULL;
     req->dev.user_buf          = NULL;
     req->dev.OnDataAvail       = NULL;
@@ -58,7 +53,7 @@ void MPID_Request_create_hook(MPIR_Request *req)
     req->dev.ext_hdr_sz        = 0;
     req->dev.rma_target_ptr    = NULL;
     req->dev.request_handle    = MPI_REQUEST_NULL;
-    req->dev.orig_segment_first = MPIDI_LOAD_RECV_IOV_ORIG_SEGMENT_FIRST_UNSET;
+    req->dev.orig_msg_offset = MPIDI_LOAD_RECV_IOV_ORIG_MSG_OFFSET_UNSET;
 
     req->dev.request_completed_cb  = NULL;
 #ifdef MPIDI_CH3_REQUEST_INIT
@@ -83,42 +78,45 @@ void MPID_Request_create_hook(MPIR_Request *req)
  *
  * Expects sreq->dev.OnFinal to be initialized (even if it's NULL).
  */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3U_Request_load_send_iov
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_CH3U_Request_load_send_iov(MPIR_Request * const sreq,
-				     MPL_IOV * const iov, int * const iov_n)
+				     struct iovec * const iov, int * const iov_n)
 {
     MPI_Aint last;
     int mpi_errno = MPI_SUCCESS;
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_CH3U_REQUEST_LOAD_SEND_IOV);
 
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_CH3U_REQUEST_LOAD_SEND_IOV);
-    MPIR_Assert(sreq->dev.segment_ptr != NULL);
-    last = sreq->dev.segment_size;
+
+    last = sreq->dev.msgsize;
     MPL_DBG_MSG_FMT(MPIDI_CH3_DBG_CHANNEL,VERBOSE,(MPL_DBG_FDEST,
      "pre-pv: first=%" PRIdPTR ", last=%" PRIdPTR ", iov_n=%d",
-		      sreq->dev.segment_first, last, *iov_n));
-    MPIR_Assert(sreq->dev.segment_first < last);
+		      sreq->dev.msg_offset, last, *iov_n));
+    MPIR_Assert(sreq->dev.msg_offset < last);
     MPIR_Assert(last > 0);
     MPIR_Assert(*iov_n > 0 && *iov_n <= MPL_IOV_LIMIT);
-    MPIR_Segment_pack_vector(sreq->dev.segment_ptr, sreq->dev.segment_first, 
-			     &last, iov, iov_n);
+
+    int max_iov_len = *iov_n;
+    MPI_Aint actual_iov_bytes, actual_iov_len;
+    MPIR_Typerep_to_iov(sreq->dev.user_buf, sreq->dev.user_count, sreq->dev.datatype,
+                     sreq->dev.msg_offset, iov, (MPI_Aint) max_iov_len,
+                     sreq->dev.msgsize - sreq->dev.msg_offset, &actual_iov_len, &actual_iov_bytes);
+    *iov_n = (int) actual_iov_len;
+    last = sreq->dev.msg_offset + actual_iov_bytes;
+
     MPL_DBG_MSG_FMT(MPIDI_CH3_DBG_CHANNEL,VERBOSE,(MPL_DBG_FDEST,
     "post-pv: first=%" PRIdPTR ", last=%" PRIdPTR ", iov_n=%d",
-		      sreq->dev.segment_first, last, *iov_n));
+		      sreq->dev.msg_offset, last, *iov_n));
     MPIR_Assert(*iov_n > 0 && *iov_n <= MPL_IOV_LIMIT);
     
-    if (last == sreq->dev.segment_size)
+    if (last == sreq->dev.msgsize)
     {
 	MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,"remaining data loaded into IOV");
 	sreq->dev.OnDataAvail = sreq->dev.OnFinal;
     }
-    else if ((last - sreq->dev.segment_first) / *iov_n >= MPIDI_IOV_DENSITY_MIN)
+    else if ((last - sreq->dev.msg_offset) / *iov_n >= MPIDI_IOV_DENSITY_MIN)
     {
 	MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,"more data loaded into IOV");
-	sreq->dev.segment_first = last;
+	sreq->dev.msg_offset = last;
 	sreq->dev.OnDataAvail = MPIDI_CH3_ReqHandler_SendReloadIOV;
     }
     else
@@ -128,7 +126,7 @@ int MPIDI_CH3U_Request_load_send_iov(MPIR_Request * const sreq,
 	
 	MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,"low density.  using SRBuf.");
 	    
-	data_sz = sreq->dev.segment_size - sreq->dev.segment_first;
+	data_sz = sreq->dev.msgsize - sreq->dev.msg_offset;
 	if (!MPIDI_Request_get_srbuf_flag(sreq))
 	{
 	    MPIDI_CH3U_SRBuf_alloc(sreq, data_sz);
@@ -137,7 +135,7 @@ int MPIDI_CH3U_Request_load_send_iov(MPIR_Request * const sreq,
 	    {
 		MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,TYPICAL,"SRBuf allocation failure");
 		mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL, 
-                                FCNAME, __LINE__, MPI_ERR_OTHER, "**nomem", 
+                                __func__, __LINE__, MPI_ERR_OTHER, "**nomem", 
 						 "**nomem %d", data_sz);
 		sreq->status.MPI_ERROR = mpi_errno;
 		goto fn_exit;
@@ -148,26 +146,28 @@ int MPIDI_CH3U_Request_load_send_iov(MPIR_Request * const sreq,
 	iov_data_copied = 0;
 	for (i = 0; i < *iov_n; i++) {
 	    MPIR_Memcpy((char*) sreq->dev.tmpbuf + iov_data_copied,
-		   iov[i].MPL_IOV_BUF, iov[i].MPL_IOV_LEN);
-	    iov_data_copied += iov[i].MPL_IOV_LEN;
+		   iov[i].iov_base, iov[i].iov_len);
+	    iov_data_copied += iov[i].iov_len;
 	}
-	sreq->dev.segment_first = last;
+	sreq->dev.msg_offset = last;
 
-	last = (data_sz <= sreq->dev.tmpbuf_sz - iov_data_copied) ? 
-	    sreq->dev.segment_size :
-	    sreq->dev.segment_first + sreq->dev.tmpbuf_sz - iov_data_copied;
-	MPL_DBG_MSG_FMT(MPIDI_CH3_DBG_CHANNEL,VERBOSE,(MPL_DBG_FDEST,
-               "pre-pack: first=%" PRIdPTR ", last=%" PRIdPTR,
-			  sreq->dev.segment_first, last));
-	MPIR_Segment_pack(sreq->dev.segment_ptr, sreq->dev.segment_first, 
-			  &last, (char*) sreq->dev.tmpbuf + iov_data_copied);
-	MPL_DBG_MSG_FMT(MPIDI_CH3_DBG_CHANNEL,VERBOSE,(MPL_DBG_FDEST,
-              "post-pack: first=%" PRIdPTR ", last=%" PRIdPTR,
-			   sreq->dev.segment_first, last));
-	iov[0].MPL_IOV_BUF = (MPL_IOV_BUF_CAST)sreq->dev.tmpbuf;
-	iov[0].MPL_IOV_LEN = last - sreq->dev.segment_first + iov_data_copied;
+        MPI_Aint max_pack_bytes;
+        MPI_Aint actual_pack_bytes;
+
+        if (data_sz > sreq->dev.tmpbuf_sz - iov_data_copied)
+            max_pack_bytes = sreq->dev.tmpbuf_sz - iov_data_copied;
+        else
+            max_pack_bytes = sreq->dev.msgsize - sreq->dev.msg_offset;
+
+        MPIR_Typerep_pack(sreq->dev.user_buf, sreq->dev.user_count, sreq->dev.datatype,
+                       sreq->dev.msg_offset, (char*) sreq->dev.tmpbuf + iov_data_copied,
+                       max_pack_bytes, &actual_pack_bytes);
+        last = sreq->dev.msg_offset + actual_pack_bytes;
+
+	iov[0].iov_base = (void *)sreq->dev.tmpbuf;
+	iov[0].iov_len = actual_pack_bytes + iov_data_copied;
 	*iov_n = 1;
-	if (last == sreq->dev.segment_size)
+	if (last == sreq->dev.msgsize)
 	{
 	    MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,"remaining data packed into SRBuf");
 	    sreq->dev.OnDataAvail = sreq->dev.OnFinal;
@@ -175,14 +175,17 @@ int MPIDI_CH3U_Request_load_send_iov(MPIR_Request * const sreq,
 	else 
 	{
 	    MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,"more data packed into SRBuf");
-	    sreq->dev.segment_first = last;
+	    sreq->dev.msg_offset = last;
 	    sreq->dev.OnDataAvail = MPIDI_CH3_ReqHandler_SendReloadIOV;
 	}
     }
-    
+
   fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_CH3U_REQUEST_LOAD_SEND_IOV);
     return mpi_errno;
+
+  fn_fail:
+    goto fn_exit;
 }
 
 
@@ -194,10 +197,6 @@ int MPIDI_CH3U_Request_load_send_iov(MPIR_Request * const sreq,
  * structure).  If the density of IOV is not sufficient, allocate a 
  * send/receive buffer and point the IOV at the buffer.
  */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3U_Request_load_recv_iov
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 {
     MPI_Aint last;
@@ -206,11 +205,11 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_CH3U_REQUEST_LOAD_RECV_IOV);
 
-    if (rreq->dev.orig_segment_first == MPIDI_LOAD_RECV_IOV_ORIG_SEGMENT_FIRST_UNSET) {
-        rreq->dev.orig_segment_first = rreq->dev.segment_first;
+    if (rreq->dev.orig_msg_offset == MPIDI_LOAD_RECV_IOV_ORIG_MSG_OFFSET_UNSET) {
+        rreq->dev.orig_msg_offset = rreq->dev.msg_offset;
     }
 
-    if (rreq->dev.segment_first < rreq->dev.segment_size)
+    if (rreq->dev.msg_offset < rreq->dev.msgsize)
     {
 	/* still reading data that needs to go into the user buffer */
 	
@@ -228,7 +227,7 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 	       could force the use of the SRBuf only 
 	       when (rreq->dev.tmpbuf_off > 0)... */
 	    
-	    data_sz = rreq->dev.segment_size - rreq->dev.segment_first - 
+	    data_sz = rreq->dev.msgsize - rreq->dev.msg_offset - 
 		rreq->dev.tmpbuf_off;
 	    MPIR_Assert(data_sz > 0);
 	    tmpbuf_sz = rreq->dev.tmpbuf_sz - rreq->dev.tmpbuf_off;
@@ -236,21 +235,21 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 	    {
 		data_sz = tmpbuf_sz;
 	    }
-	    rreq->dev.iov[0].MPL_IOV_BUF = 
-		(MPL_IOV_BUF_CAST)((char *) rreq->dev.tmpbuf + 
+	    rreq->dev.iov[0].iov_base =
+		(void *)((char *) rreq->dev.tmpbuf +
 				    rreq->dev.tmpbuf_off);
-	    rreq->dev.iov[0].MPL_IOV_LEN = data_sz;
+	    rreq->dev.iov[0].iov_len = data_sz;
             rreq->dev.iov_offset = 0;
 	    rreq->dev.iov_count = 1;
-	    MPIR_Assert(rreq->dev.segment_first - rreq->dev.orig_segment_first + data_sz +
+	    MPIR_Assert(rreq->dev.msg_offset - rreq->dev.orig_msg_offset + data_sz +
 			rreq->dev.tmpbuf_off <= rreq->dev.recv_data_sz);
-	    if (rreq->dev.segment_first - rreq->dev.orig_segment_first + data_sz + rreq->dev.tmpbuf_off ==
+	    if (rreq->dev.msg_offset - rreq->dev.orig_msg_offset + data_sz + rreq->dev.tmpbuf_off ==
 		rreq->dev.recv_data_sz)
 	    {
 		MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,
 		  "updating rreq to read the remaining data into the SRBuf");
 		rreq->dev.OnDataAvail = MPIDI_CH3_ReqHandler_UnpackSRBufComplete;
-                rreq->dev.orig_segment_first = MPIDI_LOAD_RECV_IOV_ORIG_SEGMENT_FIRST_UNSET;
+                rreq->dev.orig_msg_offset = MPIDI_LOAD_RECV_IOV_ORIG_MSG_OFFSET_UNSET;
 	    }
 	    else
 	    {
@@ -261,20 +260,26 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 	    goto fn_exit;
 	}
 	
-	last = rreq->dev.segment_size;
+	last = rreq->dev.msgsize;
 	rreq->dev.iov_count = MPL_IOV_LIMIT;
 	rreq->dev.iov_offset = 0;
 	MPL_DBG_MSG_FMT(MPIDI_CH3_DBG_CHANNEL,VERBOSE,(MPL_DBG_FDEST,
    "pre-upv: first=%" PRIdPTR ", last=%" PRIdPTR ", iov_n=%d",
-			  rreq->dev.segment_first, last, rreq->dev.iov_count));
-	MPIR_Assert(rreq->dev.segment_first < last);
+			  rreq->dev.msg_offset, last, rreq->dev.iov_count));
+	MPIR_Assert(rreq->dev.msg_offset < last);
 	MPIR_Assert(last > 0);
-	MPIR_Segment_unpack_vector(rreq->dev.segment_ptr, 
-				   rreq->dev.segment_first,
-				   &last, &rreq->dev.iov[0], &rreq->dev.iov_count);
+
+        MPI_Aint actual_iov_bytes, actual_iov_len;
+        MPIR_Typerep_to_iov(rreq->dev.user_buf, rreq->dev.user_count, rreq->dev.datatype,
+                         rreq->dev.msg_offset, &rreq->dev.iov[0], MPL_IOV_LIMIT,
+                         rreq->dev.msgsize - rreq->dev.msg_offset,
+                         &actual_iov_len, &actual_iov_bytes);
+        rreq->dev.iov_count = (int) actual_iov_len;
+        last = rreq->dev.msg_offset + actual_iov_bytes;
+
 	MPL_DBG_MSG_FMT(MPIDI_CH3_DBG_CHANNEL,VERBOSE,(MPL_DBG_FDEST,
    "post-upv: first=%" PRIdPTR ", last=%" PRIdPTR ", iov_n=%d, iov_offset=%lld",
-			  rreq->dev.segment_first, last, rreq->dev.iov_count, (long long)rreq->dev.iov_offset));
+			  rreq->dev.msg_offset, last, rreq->dev.iov_count, (long long)rreq->dev.iov_offset));
 	MPIR_Assert(rreq->dev.iov_count >= 0 && rreq->dev.iov_count <=
 		    MPL_IOV_LIMIT);
 
@@ -286,10 +291,10 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 	       the segment info so that the remaining data is received and 
 	       thrown away. */
 	    rreq->status.MPI_ERROR = MPIR_Err_create_code(MPI_SUCCESS, 
-		       MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_TYPE,
+		       MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_TYPE,
 		       "**dtypemismatch", 0);
-            MPIR_STATUS_SET_COUNT(rreq->status, rreq->dev.segment_first);
-	    rreq->dev.segment_size = rreq->dev.segment_first;
+            MPIR_STATUS_SET_COUNT(rreq->status, rreq->dev.msg_offset);
+	    rreq->dev.msgsize = rreq->dev.msg_offset;
 	    mpi_errno = MPIDI_CH3U_Request_load_recv_iov(rreq);
 	    goto fn_exit;
 	}
@@ -299,22 +304,22 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
         }
 	/* --END ERROR HANDLING-- */
 
-	if (last == rreq->dev.recv_data_sz + rreq->dev.orig_segment_first)
+	if (last == rreq->dev.recv_data_sz + rreq->dev.orig_msg_offset)
 	{
 	    MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,
      "updating rreq to read the remaining data directly into the user buffer");
 	    /* Eventually, use OnFinal for this instead */
 	    rreq->dev.OnDataAvail = rreq->dev.OnFinal;
-            rreq->dev.orig_segment_first = MPIDI_LOAD_RECV_IOV_ORIG_SEGMENT_FIRST_UNSET;
+            rreq->dev.orig_msg_offset = MPIDI_LOAD_RECV_IOV_ORIG_MSG_OFFSET_UNSET;
 	}
 	else if (MPIDI_Request_get_type(rreq) == MPIDI_REQUEST_TYPE_ACCUM_RECV ||
                  MPIDI_Request_get_type(rreq) == MPIDI_REQUEST_TYPE_GET_ACCUM_RECV ||
-                 (last == rreq->dev.segment_size ||
-                  (last - rreq->dev.segment_first) / rreq->dev.iov_count >= MPIDI_IOV_DENSITY_MIN))
+                 (last == rreq->dev.msgsize ||
+                  (last - rreq->dev.msg_offset) / rreq->dev.iov_count >= MPIDI_IOV_DENSITY_MIN))
 	{
 	    MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,
 	     "updating rreq to read more data directly into the user buffer");
-	    rreq->dev.segment_first = last;
+	    rreq->dev.msg_offset = last;
 	    rreq->dev.OnDataAvail = MPIDI_CH3_ReqHandler_ReloadIOV;
 	}
 	else
@@ -325,7 +330,7 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 	    MPIR_Assert(MPIDI_Request_get_srbuf_flag(rreq) == FALSE);
 	    
 	    MPIDI_CH3U_SRBuf_alloc(rreq, 
-			    rreq->dev.segment_size - rreq->dev.segment_first);
+			    rreq->dev.msgsize - rreq->dev.msg_offset);
 	    rreq->dev.tmpbuf_off = 0;
 	    /* --BEGIN ERROR HANDLING-- */
 	    if (rreq->dev.tmpbuf_sz == 0)
@@ -335,9 +340,9 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 		   a fatal error? */
 		MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,"SRBuf allocation failure");
 		mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL, 
-			      FCNAME, __LINE__, MPI_ERR_OTHER, "**nomem", 
+			      __func__, __LINE__, MPI_ERR_OTHER, "**nomem", 
 			 "**nomem %d", 
-			 rreq->dev.segment_size - rreq->dev.segment_first);
+			 rreq->dev.msgsize - rreq->dev.msg_offset);
 		rreq->status.MPI_ERROR = mpi_errno;
 		goto fn_exit;
 	    }
@@ -353,7 +358,7 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 	   buffer */
 	intptr_t data_sz;
 
-	data_sz = rreq->dev.recv_data_sz - rreq->dev.segment_first;
+	data_sz = rreq->dev.recv_data_sz - rreq->dev.msg_offset;
 	if (!MPIDI_Request_get_srbuf_flag(rreq))
 	{
 	    MPIDI_CH3U_SRBuf_alloc(rreq, data_sz);
@@ -362,7 +367,7 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 	    {
 		MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,TYPICAL,"SRBuf allocation failure");
 		mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL, 
-			       FCNAME, __LINE__, MPI_ERR_OTHER, "**nomem", 0);
+			       __func__, __LINE__, MPI_ERR_OTHER, "**nomem", 0);
 		rreq->status.MPI_ERROR = mpi_errno;
 		goto fn_exit;
 	    }
@@ -373,22 +378,22 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
 	{
 	    MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,
 	    "updating rreq to read overflow data into the SRBuf and complete");
-	    rreq->dev.iov[0].MPL_IOV_LEN = data_sz;
+	    rreq->dev.iov[0].iov_len = data_sz;
 	    MPIR_Assert(MPIDI_Request_get_type(rreq) == MPIDI_REQUEST_TYPE_RECV);
 	    /* Eventually, use OnFinal for this instead */
 	    rreq->dev.OnDataAvail = rreq->dev.OnFinal;
-            rreq->dev.orig_segment_first = MPIDI_LOAD_RECV_IOV_ORIG_SEGMENT_FIRST_UNSET;
+            rreq->dev.orig_msg_offset = MPIDI_LOAD_RECV_IOV_ORIG_MSG_OFFSET_UNSET;
 	}
 	else
 	{
 	    MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,VERBOSE,
 	  "updating rreq to read overflow data into the SRBuf and reload IOV");
-	    rreq->dev.iov[0].MPL_IOV_LEN = rreq->dev.tmpbuf_sz;
-	    rreq->dev.segment_first += rreq->dev.tmpbuf_sz;
+	    rreq->dev.iov[0].iov_len = rreq->dev.tmpbuf_sz;
+	    rreq->dev.msg_offset += rreq->dev.tmpbuf_sz;
 	    rreq->dev.OnDataAvail = MPIDI_CH3_ReqHandler_ReloadIOV;
 	}
 	
-	rreq->dev.iov[0].MPL_IOV_BUF = (MPL_IOV_BUF_CAST)rreq->dev.tmpbuf;
+	rreq->dev.iov[0].iov_base = (void *)rreq->dev.tmpbuf;
 	rreq->dev.iov_count = 1;
     }
     
@@ -402,10 +407,6 @@ int MPIDI_CH3U_Request_load_recv_iov(MPIR_Request * const rreq)
  *
  * Unpack data from a send/receive buffer into the user buffer.
  */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3U_Request_unpack_srbuf
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_CH3U_Request_unpack_srbuf(MPIR_Request * rreq)
 {
     MPI_Aint last;
@@ -415,43 +416,47 @@ int MPIDI_CH3U_Request_unpack_srbuf(MPIR_Request * rreq)
     
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_CH3U_REQUEST_UNPACK_SRBUF);
 
-    tmpbuf_last = (int)(rreq->dev.segment_first + rreq->dev.tmpbuf_sz);
-    if (rreq->dev.segment_size < tmpbuf_last)
+    tmpbuf_last = (int)(rreq->dev.msg_offset + rreq->dev.tmpbuf_sz);
+    if (rreq->dev.msgsize < tmpbuf_last)
     {
-	tmpbuf_last = (int)rreq->dev.segment_size;
+	tmpbuf_last = (int)rreq->dev.msgsize;
     }
-    last = tmpbuf_last;
-    MPIR_Segment_unpack(rreq->dev.segment_ptr, rreq->dev.segment_first, 
-			&last, rreq->dev.tmpbuf);
-    if (last == 0 || last == rreq->dev.segment_first)
+
+    MPI_Aint actual_unpack_bytes;
+    MPIR_Typerep_unpack(rreq->dev.tmpbuf, tmpbuf_last - rreq->dev.msg_offset,
+                     rreq->dev.user_buf, rreq->dev.user_count, rreq->dev.datatype,
+                     rreq->dev.msg_offset, &actual_unpack_bytes);
+    last = rreq->dev.msg_offset + actual_unpack_bytes;
+
+    if (last == 0 || last == rreq->dev.msg_offset)
     {
 	/* --BEGIN ERROR HANDLING-- */
 	/* If no data can be unpacked, then we have a datatype processing 
 	   problem.  Adjust the segment info so that the remaining
 	   data is received and thrown away. */
-	MPIR_STATUS_SET_COUNT(rreq->status, rreq->dev.segment_first);
-	rreq->dev.segment_size = rreq->dev.segment_first;
-	rreq->dev.segment_first += tmpbuf_last;
+	MPIR_STATUS_SET_COUNT(rreq->status, rreq->dev.msg_offset);
+	rreq->dev.msgsize = rreq->dev.msg_offset;
+	rreq->dev.msg_offset += tmpbuf_last;
 	rreq->status.MPI_ERROR = MPIR_Err_create_code(MPI_SUCCESS, 
-		       MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_TYPE,
+		       MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_TYPE,
 		       "**dtypemismatch", 0);
 	/* --END ERROR HANDLING-- */
     }
-    else if (tmpbuf_last == rreq->dev.segment_size)
+    else if (tmpbuf_last == rreq->dev.msgsize)
     {
 	/* --BEGIN ERROR HANDLING-- */
 	if (last != tmpbuf_last)
 	{
 	    /* received data was not entirely consumed by unpack() because too
 	       few bytes remained to fill the next basic datatype.
-	       Note: the segment_first field is set to segment_last so that if
+	       Note: the msg_offset field is set to segment_last so that if
 	       this is a truncated message, extra data will be read
 	       off the pipe. */
 	    MPIR_STATUS_SET_COUNT(rreq->status, last);
-	    rreq->dev.segment_size = last;
-	    rreq->dev.segment_first = tmpbuf_last;
+	    rreq->dev.msgsize = last;
+	    rreq->dev.msg_offset = tmpbuf_last;
 	    rreq->status.MPI_ERROR = MPIR_Err_create_code(MPI_SUCCESS, 
-		  MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_TYPE,
+		  MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_TYPE,
 							  "**dtypemismatch", 0);
 	}
 	/* --END ERROR HANDLING-- */
@@ -465,9 +470,9 @@ int MPIDI_CH3U_Request_unpack_srbuf(MPIR_Request * rreq)
 	       Note: memmove() is used since the data regions could
                overlap. */
 	    memmove(rreq->dev.tmpbuf, (char *) rreq->dev.tmpbuf + 
-		    (last - rreq->dev.segment_first), rreq->dev.tmpbuf_off);
+		    (last - rreq->dev.msg_offset), rreq->dev.tmpbuf_off);
 	}
-	rreq->dev.segment_first = last;
+	rreq->dev.msg_offset = last;
     }
 
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_CH3U_REQUEST_UNPACK_SRBUF);
@@ -479,10 +484,6 @@ int MPIDI_CH3U_Request_unpack_srbuf(MPIR_Request * rreq)
  *
  * Copy/unpack data from an "unexpected eager buffer" into the user buffer.
  */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_CH3U_Request_unpack_uebuf
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_CH3U_Request_unpack_uebuf(MPIR_Request * rreq)
 {
     int dt_contig;
@@ -513,7 +514,7 @@ int MPIDI_CH3U_Request_unpack_uebuf(MPIR_Request * rreq)
 	unpack_sz = userbuf_sz;
 	MPIR_STATUS_SET_COUNT(rreq->status, userbuf_sz);
 	rreq->status.MPI_ERROR = MPIR_Err_create_code(MPI_SUCCESS, 
-		 MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_TRUNCATE,
+		 MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_TRUNCATE,
 		 "**truncate", "**truncate %d %d", 
                  rreq->dev.recv_data_sz, userbuf_sz);
 	/* --END ERROR HANDLING-- */
@@ -523,10 +524,9 @@ int MPIDI_CH3U_Request_unpack_uebuf(MPIR_Request * rreq)
     {
 	if (dt_contig)
 	{
-	    /* TODO - check that amount of data is consistent with datatype.  
-	       In other words, if we were to use Segment_unpack()
-	       would last = unpack?  If not we should return an error 
-	       (unless configured with --enable-fast) */
+	    /* TODO - check that amount of data is consistent with
+	       datatype.  If not we should return an error (unless
+	       configured with --enable-fast) */
 	    MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MEMCPY);
 	    MPIR_Memcpy((char *)rreq->dev.user_buf + dt_true_lb, rreq->dev.tmpbuf,
 		   unpack_sz);
@@ -534,22 +534,20 @@ int MPIDI_CH3U_Request_unpack_uebuf(MPIR_Request * rreq)
 	}
 	else
 	{
-	    MPIR_Segment seg;
-	    MPI_Aint last;
+	    MPI_Aint actual_unpack_bytes;
+	    MPIR_Typerep_unpack(rreq->dev.tmpbuf, unpack_sz,
+			     rreq->dev.user_buf, rreq->dev.user_count,
+			     rreq->dev.datatype, 0, &actual_unpack_bytes);
 
-	    MPIR_Segment_init(rreq->dev.user_buf, rreq->dev.user_count, 
-			      rreq->dev.datatype, &seg);
-	    last = unpack_sz;
-	    MPIR_Segment_unpack(&seg, 0, &last, rreq->dev.tmpbuf);
-	    if (last != unpack_sz)
+	    if (actual_unpack_bytes != unpack_sz)
 	    {
 		/* --BEGIN ERROR HANDLING-- */
 		/* received data was not entirely consumed by unpack() 
 		   because too few bytes remained to fill the next basic
 		   datatype */
-		MPIR_STATUS_SET_COUNT(rreq->status, last);
+		MPIR_STATUS_SET_COUNT(rreq->status, actual_unpack_bytes);
 		rreq->status.MPI_ERROR = MPIR_Err_create_code(MPI_SUCCESS, 
-                         MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_TYPE,
+                         MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_TYPE,
 			 "**dtypemismatch", 0);
 		/* --END ERROR HANDLING-- */
 	    }
@@ -564,31 +562,17 @@ int MPID_Request_complete(MPIR_Request *req)
 {
     int incomplete, notify_counter;
     int mpi_errno = MPI_SUCCESS;
-    static int called_cnt = 0;
-
-    MPIR_Assert(called_cnt <= REQUEST_CB_DEPTH);
-    called_cnt++;
 
     MPIDI_CH3U_Request_decrement_cc(req, &incomplete);
     if (!incomplete) {
-        /* trigger request_completed callback function */
-        if (req->dev.request_completed_cb != NULL) {
-            mpi_errno = req->dev.request_completed_cb(req);
-            if (mpi_errno != MPI_SUCCESS) {
-                MPIR_ERR_POP(mpi_errno);
-            }
-        }
-
         /* decrement completion_notification counter */
         if (req->completion_notification)
             MPIR_cc_decr(req->completion_notification, &notify_counter);
 
 	MPIR_Request_free(req);
-	MPIDI_CH3_Progress_signal_completion();
     }
 
  fn_exit:
-    called_cnt--;
     return mpi_errno;
  fn_fail:
     goto fn_exit;
@@ -596,7 +580,22 @@ int MPID_Request_complete(MPIR_Request *req)
 
 void MPID_Request_free_hook(MPIR_Request *req)
 {
-    return;
+    static int called_cnt = 0;
+
+    MPIR_Assert(called_cnt <= REQUEST_CB_DEPTH);
+    called_cnt++;
+
+    /* trigger request_completed callback function */
+    if (req->dev.request_completed_cb != NULL && MPIR_Request_is_complete(req)) {
+        int mpi_errno = req->dev.request_completed_cb(req);
+        MPIR_Assert(mpi_errno == MPI_SUCCESS);
+
+        req->dev.request_completed_cb = NULL;
+    }
+
+    MPIDI_CH3_Progress_signal_completion();
+
+    called_cnt--;
 }
 
 void MPID_Request_destroy_hook(MPIR_Request *req)
@@ -605,15 +604,10 @@ void MPID_Request_destroy_hook(MPIR_Request *req)
         MPIR_Datatype_ptr_release(req->dev.datatype_ptr);
     }
 
-    if (req->dev.segment_ptr != NULL) {
-        MPIR_Segment_free(req->dev.segment_ptr);
-    }
-
     if (MPIDI_Request_get_srbuf_flag(req)) {
         MPIDI_CH3U_SRBuf_free(req);
     }
 
-    if (req->dev.ext_hdr_ptr != NULL) {
-        MPL_free(req->dev.ext_hdr_ptr);
-    }
+    MPL_free(req->dev.ext_hdr_ptr);
+    MPL_free(req->dev.flattened_type);
 }
