@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpidimpl.h"
@@ -21,10 +20,6 @@
 /*
  * MPID_Isend()
  */
-#undef FUNCNAME
-#define FUNCNAME MPID_Isend
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPID_Isend(const void * buf, MPI_Aint count, MPI_Datatype datatype, int rank,
 	       int tag, MPIR_Comm * comm, int context_offset,
                MPIR_Request ** request)
@@ -63,29 +58,20 @@ int MPID_Isend(const void * buf, MPI_Aint count, MPI_Datatype datatype, int rank
 	goto fn_exit;
     }
 
-    if (rank != MPI_PROC_NULL) {
-        MPIDI_Comm_get_vc_set_active(comm, rank, &vc);
+    MPIDI_Comm_get_vc_set_active(comm, rank, &vc);
 #ifdef ENABLE_COMM_OVERRIDES
-        /* this needs to come before the sreq is created, since the override
-         * function is responsible for creating its own request */
-        if (vc->comm_ops && vc->comm_ops->isend)
-        {
-            mpi_errno = vc->comm_ops->isend( vc, buf, count, datatype, rank, tag, comm, context_offset, &sreq);
-            goto fn_exit;
-        }
-#endif
+    /* this needs to come before the sreq is created, since the override
+     * function is responsible for creating its own request */
+    if (vc->comm_ops && vc->comm_ops->isend)
+    {
+        mpi_errno = vc->comm_ops->isend( vc, buf, count, datatype, rank, tag, comm, context_offset, &sreq);
+        goto fn_exit;
     }
+#endif
 
     MPIDI_Request_create_sreq(sreq, mpi_errno, goto fn_exit);
     MPIDI_Request_set_type(sreq, MPIDI_REQUEST_TYPE_SEND);
 
-    if (rank == MPI_PROC_NULL)
-    {
-	MPIR_Object_set_ref(sreq, 1);
-        MPIR_cc_set(&sreq->cc, 0);
-	goto fn_exit;
-    }
-
     MPIDI_Datatype_get_info(count, datatype, dt_contig, data_sz, dt_ptr, 
 			    dt_true_lb);
     
@@ -144,7 +130,7 @@ int MPID_Isend(const void * buf, MPI_Aint count, MPI_Datatype datatype, int rank
 	    mpi_errno = MPIDI_CH3_EagerNoncontigSend( &sreq, 
                                                       MPIDI_CH3_PKT_EAGER_SEND,
                                                       buf, count, datatype,
-                                                      data_sz, rank, tag, 
+                                                      rank, tag,
                                                       comm, context_offset );
 	    /* If we're not complete, then add a reference to the datatype */
 	    if (sreq) {
@@ -189,3 +175,29 @@ int MPID_Isend(const void * buf, MPI_Aint count, MPI_Datatype datatype, int rank
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_ISEND);
     return mpi_errno;
 }
+
+int MPID_Isend_coll(const void * buf, MPI_Aint count, MPI_Datatype datatype, int rank, int tag,
+                    MPIR_Comm * comm, int context_offset, MPIR_Request ** request,
+                    MPIR_Errflag_t * errflag)
+{
+    int mpi_errno = MPI_SUCCESS;
+
+    MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPID_ISEND_COLL);
+    MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPID_ISEND_COLL);
+
+    switch (*errflag) {
+    case MPIR_ERR_NONE:
+        break;
+    case MPIR_ERR_PROC_FAILED:
+        MPIR_TAG_SET_PROC_FAILURE_BIT(tag);
+        break;
+    default:
+        MPIR_TAG_SET_ERROR_BIT(tag);
+    }
+
+    mpi_errno = MPID_Isend(buf, count, datatype, rank, tag, comm, context_offset, request);
+
+    MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_ISEND_COLL);
+
+    return mpi_errno;
+}
