@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpidimpl.h"
@@ -9,10 +8,6 @@
 /*
  * MPID_Ssend()
  */
-#undef FUNCNAME
-#define FUNCNAME MPID_Ssend
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPID_Ssend(const void * buf, MPI_Aint count, MPI_Datatype datatype, int rank, int tag, MPIR_Comm * comm, int context_offset,
 	       MPIR_Request ** request)
 {
@@ -55,7 +50,7 @@ int MPID_Ssend(const void * buf, MPI_Aint count, MPI_Datatype datatype, int rank
 	    /* --BEGIN ERROR HANDLING-- */
 	    if (sreq != NULL && MPIR_cc_get(sreq->cc) != 0)
 	    {
-		mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_RECOVERABLE, FCNAME, __LINE__, MPI_ERR_OTHER,
+		mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_RECOVERABLE, __func__, __LINE__, MPI_ERR_OTHER,
 						 "**dev|selfsenddeadlock", 0);
 		goto fn_exit;
 	    }
@@ -64,11 +59,6 @@ int MPID_Ssend(const void * buf, MPI_Aint count, MPI_Datatype datatype, int rank
 #	endif
 	goto fn_exit;
     }
-    
-    if (rank == MPI_PROC_NULL)
-    {
-	goto fn_exit;
-    }
 
     MPIDI_Comm_get_vc_set_active(comm, rank, &vc);
 
