@@ -1,14 +1,9 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpidimpl.h"
-/* FIXME: This bsend header shouldn't be needed (the function prototype
-   should be in mpiimpl.h), to allow a devices MPID_Startall to use the
-   MPIR_Bsend_isend function */
-#include "../../../mpi/pt2pt/bsendutil.h"
 
 /* FIXME: Consider using function pointers for invoking persistent requests;
    if we made those part of the public request structure, the top-level routine
@@ -47,10 +42,6 @@
 /*
  * MPID_Startall()
  */
-#undef FUNCNAME
-#define FUNCNAME MPID_Startall
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPID_Startall(int count, MPIR_Request * requests[])
 {
     int i;
@@ -64,6 +55,10 @@ int MPID_Startall(int count, MPIR_Request * requests[])
     {
 	MPIR_Request * const preq = requests[i];
 
+        /* continue if the source/dest is MPI_PROC_NULL */
+        if (preq->dev.match.parts.rank == MPI_PROC_NULL)
+            continue;
+
 	/* FIXME: The odd 7th arg (match.context_id - comm->context_id) 
 	   is probably to get the context offset.  Do we really need the
 	   context offset? Is there any case where the offset isn't zero? */
@@ -103,15 +98,16 @@ int MPID_Startall(int count, MPIR_Request * requests[])
 
 	    case MPIDI_REQUEST_TYPE_BSEND:
 	    {
-		MPI_Request sreq_handle;
-
-                rc = MPIR_Ibsend_impl(preq->dev.user_buf, preq->dev.user_count,
+                rc = MPIR_Bsend_isend(preq->dev.user_buf, preq->dev.user_count,
                                       preq->dev.datatype, preq->dev.match.parts.rank,
                                       preq->dev.match.parts.tag, preq->comm,
-                                      &sreq_handle);
-                if (rc == MPI_SUCCESS)
-                {
-                    MPIR_Request_get_ptr(sreq_handle, preq->u.persist.real_request);
+                                      &preq->u.persist.real_request);
+                if (rc == MPI_SUCCESS) {
+                    preq->status.MPI_ERROR = MPI_SUCCESS;
+                    preq->cc_ptr = &preq->cc;
+                    /* bsend is local-complete */
+                    MPIR_cc_set(preq->cc_ptr, 0);
+                    goto fn_exit;
                 }
 		break;
 	    }
@@ -119,7 +115,7 @@ int MPID_Startall(int count, MPIR_Request * requests[])
 	    default:
 	    {
 		/* --BEGIN ERROR HANDLING-- */
-		rc = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL, FCNAME, __LINE__, MPI_ERR_INTERN, "**ch3|badreqtype",
+		rc = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL, __func__, __LINE__, MPI_ERR_INTERN, "**ch3|badreqtype",
 					  "**ch3|badreqtype %d", MPIDI_Request_get_type(preq));
 		/* --END ERROR HANDLING-- */
 	    }
@@ -146,6 +142,7 @@ int MPID_Startall(int count, MPIR_Request * requests[])
 	/* --END ERROR HANDLING-- */
     }
 
+  fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_STARTALL);
     return mpi_errno;
 }
@@ -157,10 +154,6 @@ int MPID_Startall(int count, MPIR_Request * requests[])
 /*
  * MPID_Send_init()
  */
-#undef FUNCNAME
-#define FUNCNAME MPID_Send_init
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPID_Send_init(const void * buf, int count, MPI_Datatype datatype, int rank, int tag, MPIR_Comm * comm, int context_offset,
 		   MPIR_Request ** request)
 {
@@ -172,7 +165,7 @@ int MPID_Send_init(const void * buf, int count, MPI_Datatype datatype, int rank,
 
     MPIDI_Request_create_psreq(sreq, mpi_errno, goto fn_exit);
     MPIDI_Request_set_type(sreq, MPIDI_REQUEST_TYPE_SEND);
-    if (HANDLE_GET_KIND(datatype) != HANDLE_KIND_BUILTIN)
+    if (!HANDLE_IS_BUILTIN(datatype))
     {
 	MPIR_Datatype_get_ptr(datatype, sreq->dev.datatype_ptr);
     MPIR_Datatype_ptr_add_ref(sreq->dev.datatype_ptr);
@@ -187,10 +180,6 @@ int MPID_Send_init(const void * buf, int count, MPI_Datatype datatype, int rank,
 /*
  * MPID_Ssend_init()
  */
-#undef FUNCNAME
-#define FUNCNAME MPID_Ssend_init
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPID_Ssend_init(const void * buf, int count, MPI_Datatype datatype, int rank, int tag, MPIR_Comm * comm, int context_offset,
 		    MPIR_Request ** request)
 {
@@ -202,7 +191,7 @@ int MPID_Ssend_init(const void * buf, int count, MPI_Datatype datatype, int rank
 
     MPIDI_Request_create_psreq(sreq, mpi_errno, goto fn_exit);
     MPIDI_Request_set_type(sreq, MPIDI_REQUEST_TYPE_SSEND);
-    if (HANDLE_GET_KIND(datatype) != HANDLE_KIND_BUILTIN)
+    if (!HANDLE_IS_BUILTIN(datatype))
     {
 	MPIR_Datatype_get_ptr(datatype, sreq->dev.datatype_ptr);
     MPIR_Datatype_ptr_add_ref(sreq->dev.datatype_ptr);
@@ -217,10 +206,6 @@ int MPID_Ssend_init(const void * buf, int count, MPI_Datatype datatype, int rank
 /*
  * MPID_Rsend_init()
  */
-#undef FUNCNAME
-#define FUNCNAME MPID_Rsend_init
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPID_Rsend_init(const void * buf, int count, MPI_Datatype datatype, int rank, int tag, MPIR_Comm * comm, int context_offset,
 		    MPIR_Request ** request)
 {
@@ -232,7 +217,7 @@ int MPID_Rsend_init(const void * buf, int count, MPI_Datatype datatype, int rank
 
     MPIDI_Request_create_psreq(sreq, mpi_errno, goto fn_exit);
     MPIDI_Request_set_type(sreq, MPIDI_REQUEST_TYPE_RSEND);
-    if (HANDLE_GET_KIND(datatype) != HANDLE_KIND_BUILTIN)
+    if (!HANDLE_IS_BUILTIN(datatype))
     {
 	MPIR_Datatype_get_ptr(datatype, sreq->dev.datatype_ptr);
     MPIR_Datatype_ptr_add_ref(sreq->dev.datatype_ptr);
@@ -247,10 +232,6 @@ int MPID_Rsend_init(const void * buf, int count, MPI_Datatype datatype, int rank
 /*
  * MPID_Bsend_init()
  */
-#undef FUNCNAME
-#define FUNCNAME MPID_Bsend_init
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPID_Bsend_init(const void * buf, int count, MPI_Datatype datatype, int rank, int tag, MPIR_Comm * comm, int context_offset,
 		    MPIR_Request ** request)
 {
@@ -262,7 +243,7 @@ int MPID_Bsend_init(const void * buf, int count, MPI_Datatype datatype, int rank
 
     MPIDI_Request_create_psreq(sreq, mpi_errno, goto fn_exit);
     MPIDI_Request_set_type(sreq, MPIDI_REQUEST_TYPE_BSEND);
-    if (HANDLE_GET_KIND(datatype) != HANDLE_KIND_BUILTIN)
+    if (!HANDLE_IS_BUILTIN(datatype))
     {
 	MPIR_Datatype_get_ptr(datatype, sreq->dev.datatype_ptr);
     MPIR_Datatype_ptr_add_ref(sreq->dev.datatype_ptr);
@@ -285,10 +266,6 @@ int MPID_Bsend_init(const void * buf, int count, MPI_Datatype datatype, int rank
 /*
  * MPID_Recv_init()
  */
-#undef FUNCNAME
-#define FUNCNAME MPID_Recv_init
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPID_Recv_init(void * buf, int count, MPI_Datatype datatype, int rank, int tag, MPIR_Comm * comm, int context_offset,
 		   MPIR_Request ** request)
 {
@@ -302,7 +279,7 @@ int MPID_Recv_init(void * buf, int count, MPI_Datatype datatype, int rank, int t
     if (rreq == NULL)
     {
 	/* --BEGIN ERROR HANDLING-- */
-	mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL, FCNAME, __LINE__, MPI_ERR_OTHER, "**nomemreq", 0);
+	mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL, __func__, __LINE__, MPI_ERR_OTHER, "**nomemreq", 0);
 	/* --END ERROR HANDLING-- */
 	goto fn_exit;
     }
@@ -319,7 +296,7 @@ int MPID_Recv_init(void * buf, int count, MPI_Datatype datatype, int rank, int t
     rreq->dev.datatype = datatype;
     rreq->u.persist.real_request = NULL;
     MPIDI_Request_set_type(rreq, MPIDI_REQUEST_TYPE_RECV);
-    if (HANDLE_GET_KIND(datatype) != HANDLE_KIND_BUILTIN)
+    if (!HANDLE_IS_BUILTIN(datatype))
     {
 	MPIR_Datatype_get_ptr(datatype, rreq->dev.datatype_ptr);
     MPIR_Datatype_ptr_add_ref(rreq->dev.datatype_ptr);
