@@ -1,10 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2016 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
- *
- *  Portions of this code were written by Mellanox Technologies Ltd.
- *  Copyright (C) Mellanox Technologies Ltd. 2016. ALL RIGHTS RESERVED
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpl.h"
@@ -16,29 +12,32 @@ MPL_SUPPRESS_OSX_HAS_NO_SYMBOLS_WARNING;
 #include <mpidimpl.h>
 #include "netmod_inline.h"
 MPIDI_NM_funcs_t MPIDI_NM_ucx_funcs = {
-    .mpi_init = MPIDI_NM_mpi_init_hook,
-    .mpi_finalize = MPIDI_NM_mpi_finalize_hook,
-    .progress = MPIDI_NM_progress,
-    .mpi_comm_connect = MPIDI_NM_mpi_comm_connect,
-    .mpi_comm_disconnect = MPIDI_NM_mpi_comm_disconnect,
-    .mpi_open_port = MPIDI_NM_mpi_open_port,
-    .mpi_close_port = MPIDI_NM_mpi_close_port,
-    .mpi_comm_accept = MPIDI_NM_mpi_comm_accept,
+    .mpi_init = MPIDI_UCX_mpi_init_hook,
+    .mpi_finalize = MPIDI_UCX_mpi_finalize_hook,
+    .post_init = MPIDI_UCX_post_init,
+    .progress = MPIDI_UCX_progress,
+    .mpi_comm_connect = MPIDI_UCX_mpi_comm_connect,
+    .mpi_comm_disconnect = MPIDI_UCX_mpi_comm_disconnect,
+    .mpi_open_port = MPIDI_UCX_mpi_open_port,
+    .mpi_close_port = MPIDI_UCX_mpi_close_port,
+    .mpi_comm_accept = MPIDI_UCX_mpi_comm_accept,
     /* Routines that handle addressing */
     .comm_get_lpid = MPIDI_NM_comm_get_lpid,
-    .get_local_upids = MPIDI_NM_get_local_upids,
-    .upids_to_lupids = MPIDI_NM_upids_to_lupids,
-    .create_intercomm_from_lpids = MPIDI_NM_create_intercomm_from_lpids,
-    .mpi_comm_create_hook = MPIDI_NM_mpi_comm_create_hook,
-    .mpi_comm_free_hook = MPIDI_NM_mpi_comm_free_hook,
+    .get_local_upids = MPIDI_UCX_get_local_upids,
+    .get_vci_attr = MPIDI_UCX_get_vci_attr,
+    .upids_to_lupids = MPIDI_UCX_upids_to_lupids,
+    .create_intercomm_from_lpids = MPIDI_UCX_create_intercomm_from_lpids,
+    .mpi_comm_commit_pre_hook = MPIDI_UCX_mpi_comm_commit_pre_hook,
+    .mpi_comm_commit_post_hook = MPIDI_UCX_mpi_comm_commit_post_hook,
+    .mpi_comm_free_hook = MPIDI_UCX_mpi_comm_free_hook,
     /* Window initialization/cleanup routines */
-    .mpi_win_create_hook = MPIDI_NM_mpi_win_create_hook,
-    .mpi_win_allocate_hook = MPIDI_NM_mpi_win_allocate_hook,
-    .mpi_win_allocate_shared_hook = MPIDI_NM_mpi_win_allocate_shared_hook,
-    .mpi_win_create_dynamic_hook = MPIDI_NM_mpi_win_create_dynamic_hook,
-    .mpi_win_attach_hook = MPIDI_NM_mpi_win_attach_hook,
-    .mpi_win_detach_hook = MPIDI_NM_mpi_win_detach_hook,
-    .mpi_win_free_hook = MPIDI_NM_mpi_win_free_hook,
+    .mpi_win_create_hook = MPIDI_UCX_mpi_win_create_hook,
+    .mpi_win_allocate_hook = MPIDI_UCX_mpi_win_allocate_hook,
+    .mpi_win_allocate_shared_hook = MPIDI_UCX_mpi_win_allocate_shared_hook,
+    .mpi_win_create_dynamic_hook = MPIDI_UCX_mpi_win_create_dynamic_hook,
+    .mpi_win_attach_hook = MPIDI_UCX_mpi_win_attach_hook,
+    .mpi_win_detach_hook = MPIDI_UCX_mpi_win_detach_hook,
+    .mpi_win_free_hook = MPIDI_UCX_mpi_win_free_hook,
     /* RMA synchronization routines */
     .rma_win_cmpl_hook = MPIDI_NM_rma_win_cmpl_hook,
     .rma_win_local_cmpl_hook = MPIDI_NM_rma_win_local_cmpl_hook,
@@ -54,30 +53,27 @@ MPIDI_NM_funcs_t MPIDI_NM_ucx_funcs = {
     .am_send_hdr_reply = MPIDI_NM_am_send_hdr_reply,
     .am_isend_reply = MPIDI_NM_am_isend_reply,
     .am_hdr_max_sz = MPIDI_NM_am_hdr_max_sz,
-    .am_recv = MPIDI_NM_am_recv
+    .am_eager_limit = MPIDI_NM_am_eager_limit,
+    .am_eager_buf_limit = MPIDI_NM_am_eager_buf_limit
 };
 
 MPIDI_NM_native_funcs_t MPIDI_NM_native_ucx_funcs = {
     .mpi_send = MPIDI_NM_mpi_send,
+    .send_coll = MPIDI_NM_send_coll,
     .mpi_ssend = MPIDI_NM_mpi_ssend,
-    .mpi_startall = MPIDI_NM_mpi_startall,
-    .mpi_send_init = MPIDI_NM_mpi_send_init,
-    .mpi_ssend_init = MPIDI_NM_mpi_ssend_init,
-    .mpi_rsend_init = MPIDI_NM_mpi_rsend_init,
-    .mpi_bsend_init = MPIDI_NM_mpi_bsend_init,
     .mpi_isend = MPIDI_NM_mpi_isend,
+    .isend_coll = MPIDI_NM_isend_coll,
     .mpi_issend = MPIDI_NM_mpi_issend,
     .mpi_cancel_send = MPIDI_NM_mpi_cancel_send,
-    .mpi_recv_init = MPIDI_NM_mpi_recv_init,
     .mpi_recv = MPIDI_NM_mpi_recv,
     .mpi_irecv = MPIDI_NM_mpi_irecv,
     .mpi_imrecv = MPIDI_NM_mpi_imrecv,
     .mpi_cancel_recv = MPIDI_NM_mpi_cancel_recv,
-    .mpi_alloc_mem = MPIDI_NM_mpi_alloc_mem,
-    .mpi_free_mem = MPIDI_NM_mpi_free_mem,
+    .mpi_alloc_mem = MPIDI_UCX_mpi_alloc_mem,
+    .mpi_free_mem = MPIDI_UCX_mpi_free_mem,
     .mpi_improbe = MPIDI_NM_mpi_improbe,
     .mpi_iprobe = MPIDI_NM_mpi_iprobe,
-    .mpi_win_set_info = MPIDI_NM_mpi_win_set_info,
+    .mpi_win_set_info = MPIDI_UCX_mpi_win_set_info,
     .mpi_win_shared_query = MPIDI_NM_mpi_win_shared_query,
     .mpi_put = MPIDI_NM_mpi_put,
     .mpi_win_start = MPIDI_NM_mpi_win_start,
@@ -87,26 +83,26 @@ MPIDI_NM_native_funcs_t MPIDI_NM_native_ucx_funcs = {
     .mpi_win_test = MPIDI_NM_mpi_win_test,
     .mpi_win_lock = MPIDI_NM_mpi_win_lock,
     .mpi_win_unlock = MPIDI_NM_mpi_win_unlock,
-    .mpi_win_get_info = MPIDI_NM_mpi_win_get_info,
+    .mpi_win_get_info = MPIDI_UCX_mpi_win_get_info,
     .mpi_get = MPIDI_NM_mpi_get,
-    .mpi_win_free = MPIDI_NM_mpi_win_free,
+    .mpi_win_free = MPIDI_UCX_mpi_win_free,
     .mpi_win_fence = MPIDI_NM_mpi_win_fence,
-    .mpi_win_create = MPIDI_NM_mpi_win_create,
+    .mpi_win_create = MPIDI_UCX_mpi_win_create,
     .mpi_accumulate = MPIDI_NM_mpi_accumulate,
-    .mpi_win_attach = MPIDI_NM_mpi_win_attach,
-    .mpi_win_allocate_shared = MPIDI_NM_mpi_win_allocate_shared,
+    .mpi_win_attach = MPIDI_UCX_mpi_win_attach,
+    .mpi_win_allocate_shared = MPIDI_UCX_mpi_win_allocate_shared,
     .mpi_rput = MPIDI_NM_mpi_rput,
     .mpi_win_flush_local = MPIDI_NM_mpi_win_flush_local,
-    .mpi_win_detach = MPIDI_NM_mpi_win_detach,
+    .mpi_win_detach = MPIDI_UCX_mpi_win_detach,
     .mpi_compare_and_swap = MPIDI_NM_mpi_compare_and_swap,
     .mpi_raccumulate = MPIDI_NM_mpi_raccumulate,
     .mpi_rget_accumulate = MPIDI_NM_mpi_rget_accumulate,
     .mpi_fetch_and_op = MPIDI_NM_mpi_fetch_and_op,
-    .mpi_win_allocate = MPIDI_NM_mpi_win_allocate,
+    .mpi_win_allocate = MPIDI_UCX_mpi_win_allocate,
     .mpi_win_flush = MPIDI_NM_mpi_win_flush,
     .mpi_win_flush_local_all = MPIDI_NM_mpi_win_flush_local_all,
     .mpi_win_unlock_all = MPIDI_NM_mpi_win_unlock_all,
-    .mpi_win_create_dynamic = MPIDI_NM_mpi_win_create_dynamic,
+    .mpi_win_create_dynamic = MPIDI_UCX_mpi_win_create_dynamic,
     .mpi_rget = MPIDI_NM_mpi_rget,
     .mpi_win_sync = MPIDI_NM_mpi_win_sync,
     .mpi_win_flush_all = MPIDI_NM_mpi_win_flush_all,
@@ -159,33 +155,11 @@ MPIDI_NM_native_funcs_t MPIDI_NM_native_ucx_funcs = {
     .mpi_iscan = MPIDI_NM_mpi_iscan,
     .mpi_iscatter = MPIDI_NM_mpi_iscatter,
     .mpi_iscatterv = MPIDI_NM_mpi_iscatterv,
-    .mpi_ibarrier_sched = MPIDI_NM_mpi_ibarrier_sched,
-    .mpi_ibcast_sched = MPIDI_NM_mpi_ibcast_sched,
-    .mpi_iallgather_sched = MPIDI_NM_mpi_iallgather_sched,
-    .mpi_iallgatherv_sched = MPIDI_NM_mpi_iallgatherv_sched,
-    .mpi_iallreduce_sched = MPIDI_NM_mpi_iallreduce_sched,
-    .mpi_ialltoall_sched = MPIDI_NM_mpi_ialltoall_sched,
-    .mpi_ialltoallv_sched = MPIDI_NM_mpi_ialltoallv_sched,
-    .mpi_ialltoallw_sched = MPIDI_NM_mpi_ialltoallw_sched,
-    .mpi_iexscan_sched = MPIDI_NM_mpi_iexscan_sched,
-    .mpi_igather_sched = MPIDI_NM_mpi_igather_sched,
-    .mpi_igatherv_sched = MPIDI_NM_mpi_igatherv_sched,
-    .mpi_ireduce_scatter_block_sched = MPIDI_NM_mpi_ireduce_scatter_block_sched,
-    .mpi_ireduce_scatter_sched = MPIDI_NM_mpi_ireduce_scatter_sched,
-    .mpi_ireduce_sched = MPIDI_NM_mpi_ireduce_sched,
-    .mpi_iscan_sched = MPIDI_NM_mpi_iscan_sched,
-    .mpi_iscatter_sched = MPIDI_NM_mpi_iscatter_sched,
-    .mpi_iscatterv_sched = MPIDI_NM_mpi_iscatterv_sched,
-    .mpi_ineighbor_allgather_sched = MPIDI_NM_mpi_ineighbor_allgather_sched,
-    .mpi_ineighbor_allgatherv_sched = MPIDI_NM_mpi_ineighbor_allgatherv_sched,
-    .mpi_ineighbor_alltoall_sched = MPIDI_NM_mpi_ineighbor_alltoall_sched,
-    .mpi_ineighbor_alltoallv_sched = MPIDI_NM_mpi_ineighbor_alltoallv_sched,
-    .mpi_ineighbor_alltoallw_sched = MPIDI_NM_mpi_ineighbor_alltoallw_sched,
     /* Datatype hooks */
-    .mpi_type_commit_hook = MPIDI_NM_mpi_type_commit_hook,
-    .mpi_type_free_hook = MPIDI_NM_mpi_type_free_hook,
+    .mpi_type_commit_hook = MPIDI_UCX_mpi_type_commit_hook,
+    .mpi_type_free_hook = MPIDI_UCX_mpi_type_free_hook,
     /* Op hooks */
-    .mpi_op_commit_hook = MPIDI_NM_mpi_op_commit_hook,
-    .mpi_op_free_hook = MPIDI_NM_mpi_op_free_hook,
+    .mpi_op_commit_hook = MPIDI_UCX_mpi_op_commit_hook,
+    .mpi_op_free_hook = MPIDI_UCX_mpi_op_free_hook,
 };
 #endif
