@@ -1,11 +1,8 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2016 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
- *
- *  Portions of this code were written by Mellanox Technologies Ltd.
- *  Copyright (C) Mellanox Technologies Ltd. 2016. ALL RIGHTS RESERVED
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
+
 #ifndef UCX_IMPL_H_INCLUDED
 #define UCX_IMPL_H_INCLUDED
 
@@ -19,14 +16,15 @@
 #define MPIDI_UCX_COMM(comm)     ((comm)->dev.ch4.netmod.ucx)
 #define MPIDI_UCX_REQ(req)       ((req)->dev.ch4.netmod.ucx)
 #define COMM_TO_INDEX(comm,rank) MPIDIU_comm_rank_to_pid(comm, rank, NULL, NULL)
-#define MPIDI_UCX_COMM_TO_EP(comm,rank) \
-    MPIDI_UCX_AV(MPIDIU_comm_rank_to_av(comm, rank)).dest
-#define MPIDI_UCX_AV_TO_EP(av) MPIDI_UCX_AV((av)).dest
+#define MPIDI_UCX_COMM_TO_EP(comm,rank,vni_src,vni_dst) \
+    MPIDI_UCX_AV(MPIDIU_comm_rank_to_av(comm, rank)).dest[vni_src][vni_dst]
+#define MPIDI_UCX_AV_TO_EP(av,vni_src,vni_dst) MPIDI_UCX_AV((av)).dest[vni_src][vni_dst]
 
 #define MPIDI_UCX_WIN(win) ((win)->dev.netmod.ucx)
 #define MPIDI_UCX_WIN_INFO(win, rank) MPIDI_UCX_WIN(win).info_table[rank]
 
-static inline uint64_t MPIDI_UCX_init_tag(MPIR_Context_id_t contextid, int source, uint64_t tag)
+MPL_STATIC_INLINE_PREFIX uint64_t MPIDI_UCX_init_tag(MPIR_Context_id_t contextid, int source,
+                                                     uint64_t tag)
 {
     uint64_t ucp_tag = 0;
     ucp_tag = contextid;
@@ -37,7 +35,7 @@ static inline uint64_t MPIDI_UCX_init_tag(MPIR_Context_id_t contextid, int sourc
     return ucp_tag;
 }
 
-static inline uint64_t MPIDI_UCX_tag_mask(int mpi_tag, int src)
+MPL_STATIC_INLINE_PREFIX uint64_t MPIDI_UCX_tag_mask(int mpi_tag, int src)
 {
     uint64_t tag_mask = 0xffffffffffffffff;
     MPIR_TAG_CLEAR_ERROR_BITS(tag_mask);
@@ -50,7 +48,8 @@ static inline uint64_t MPIDI_UCX_tag_mask(int mpi_tag, int src)
     return tag_mask;
 }
 
-static inline uint64_t MPIDI_UCX_recv_tag(int mpi_tag, int src, MPIR_Context_id_t contextid)
+MPL_STATIC_INLINE_PREFIX uint64_t MPIDI_UCX_recv_tag(int mpi_tag, int src,
+                                                     MPIR_Context_id_t contextid)
 {
     uint64_t ucp_tag = contextid;
 
@@ -63,12 +62,12 @@ static inline uint64_t MPIDI_UCX_recv_tag(int mpi_tag, int src, MPIR_Context_id_
     return ucp_tag;
 }
 
-static inline int MPIDI_UCX_get_tag(uint64_t match_bits)
+MPL_STATIC_INLINE_PREFIX int MPIDI_UCX_get_tag(uint64_t match_bits)
 {
     return ((int) (match_bits & MPIDI_UCX_TAG_MASK));
 }
 
-static inline int MPIDI_UCX_get_source(uint64_t match_bits)
+MPL_STATIC_INLINE_PREFIX int MPIDI_UCX_get_source(uint64_t match_bits)
 {
     return ((int) ((match_bits & MPIDI_UCX_SOURCE_MASK) >> MPIDI_UCX_TAG_SHIFT));
 }
@@ -82,47 +81,10 @@ static inline int MPIDI_UCX_get_source(uint64_t match_bits)
                              "**ucx_nm_status %s %d %s %s",             \
                              __SHORT_FILE__,                            \
                              __LINE__,                                  \
-                             FCNAME,                                    \
+                             __func__,                                    \
                              ucs_status_string(STATUS));                \
     } while (0)
 
-
-#define MPIDI_UCX_PMI_ERROR(_errno)                             \
-    do                                                          \
-    {                                                           \
-        MPIR_ERR_CHKANDJUMP4(_errno!=PMI_SUCCESS,               \
-                             mpi_errno,                         \
-                             MPI_ERR_OTHER,                     \
-                             "**ucx_nm_pmi_error",              \
-                             "**ucx_nm_pmi_error %s %d %s %s",  \
-                             __SHORT_FILE__,                    \
-                             __LINE__,                          \
-                             FCNAME,                            \
-                             "pmi_error");                      \
-    } while (0)
-
-#define MPIDI_UCX_MPI_ERROR(_errno)                                     \
-    do                                                                  \
-    {                                                                   \
-        if (unlikely(_errno!=MPI_SUCCESS)) MPIR_ERR_POP(mpi_errno);     \
-    } while (0)
-
-#define MPIDI_UCX_STR_ERRCHK(_errno)                            \
-    do                                                          \
-    {                                                           \
-        MPIR_ERR_CHKANDJUMP4(_errno!=MPL_STR_SUCCESS,           \
-                             mpi_errno,                         \
-                             MPI_ERR_OTHER,                     \
-                             "**ucx_nm_str_error",              \
-                             "**ucx_nm_str_error %s %d %s %s",  \
-                             __SHORT_FILE__,                    \
-                             __LINE__,                          \
-                             FCNAME,                            \
-                             "strng_error");                    \
-    } while (0)
-
-
-
 #define MPIDI_UCX_CHK_REQUEST(_req)                                     \
     do {                                                                \
         MPIR_ERR_CHKANDJUMP4(UCS_PTR_IS_ERR(_req),                      \
@@ -132,8 +94,33 @@ static inline int MPIDI_UCX_get_source(uint64_t match_bits)
                              "**ucx_nm_rq_error %s %d %s %s",           \
                              __SHORT_FILE__,                            \
                              __LINE__,                                  \
-                             FCNAME,                                    \
+                             __func__,                                    \
                              ucs_status_string(UCS_PTR_STATUS(_req)));  \
     } while (0)
 
+MPL_STATIC_INLINE_PREFIX bool MPIDI_UCX_is_reachable_target(int rank, MPIR_Win * win,
+                                                            MPIDI_winattr_t winattr)
+{
+    /* unmapped win target does not have rkey. */
+    return (winattr & MPIDI_WINATTR_NM_REACHABLE) || (MPIDI_UCX_WIN(win).info_table &&
+                                                      MPIDI_UCX_WIN_INFO(win, rank).rkey != NULL);
+}
+
+/* This function implements netmod vci to vni(context) mapping.
+ * It returns -1 if the vci does not have a mapping.
+ */
+MPL_STATIC_INLINE_PREFIX int MPIDI_UCX_vci_to_vni(int vci)
+{
+    return vci < MPIDI_UCX_global.num_vnis ? vci : -1;
+}
+
+/* vni mapping */
+/* NOTE: concerned by the modulo? If we restrict num_vnis to power of 2,
+ * we may get away with bit mask */
+MPL_STATIC_INLINE_PREFIX int MPIDI_UCX_get_vni(int flag, MPIR_Comm * comm_ptr,
+                                               int src_rank, int dst_rank, int tag)
+{
+    return MPIDI_get_vci(flag, comm_ptr, src_rank, dst_rank, tag) % MPIDI_UCX_global.num_vnis;
+}
+
 #endif /* UCX_IMPL_H_INCLUDED */
