@@ -1,22 +1,20 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2016 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
- *
- *  Portions of this code were written by Mellanox Technologies Ltd.
- *  Copyright (C) Mellanox Technologies Ltd. 2016. ALL RIGHTS RESERVED
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
+
 #ifndef UCX_PROBE_H_INCLUDED
 #define UCX_PROBE_H_INCLUDED
 
 #include "ucx_impl.h"
 
-static inline int MPIDI_NM_mpi_improbe(int source,
-                                       int tag,
-                                       MPIR_Comm * comm,
-                                       int context_offset,
-                                       MPIDI_av_entry_t * addr,
-                                       int *flag, MPIR_Request ** message, MPI_Status * status)
+MPL_STATIC_INLINE_PREFIX int MPIDI_NM_mpi_improbe(int source,
+                                                  int tag,
+                                                  MPIR_Comm * comm,
+                                                  int context_offset,
+                                                  MPIDI_av_entry_t * addr,
+                                                  int *flag, MPIR_Request ** message,
+                                                  MPI_Status * status)
 {
     int mpi_errno = MPI_SUCCESS;
     uint64_t ucp_tag, tag_mask;
@@ -25,17 +23,20 @@ static inline int MPIDI_NM_mpi_improbe(int source,
     ucp_tag_message_h message_h;
     MPIR_Request *req = NULL;
 
+    int vni_dst = MPIDI_UCX_get_vni(DST_VCI_FROM_RECVER, comm, source, comm->rank, tag);
+    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vni_dst).lock);
+
     tag_mask = MPIDI_UCX_tag_mask(tag, source);
     ucp_tag = MPIDI_UCX_recv_tag(tag, source, comm->recvcontext_id + context_offset);
 
-    message_h = ucp_tag_probe_nb(MPIDI_UCX_global.worker, ucp_tag, tag_mask, 1, &info);
+    message_h = ucp_tag_probe_nb(MPIDI_UCX_global.ctx[vni_dst].worker, ucp_tag, tag_mask, 1, &info);
 
     if (message_h) {
         *flag = 1;
-        req = (MPIR_Request *) MPIR_Request_create(MPIR_REQUEST_KIND__MPROBE);
+        req = (MPIR_Request *) MPIR_Request_create_from_pool(MPIR_REQUEST_KIND__MPROBE, vni_dst);
         MPIR_ERR_CHKANDSTMT((req) == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
         MPIR_Request_add_ref(req);
-        MPIDI_UCX_REQ(req).a.message_handler = message_h;
+        MPIDI_UCX_REQ(req).message_handler = message_h;
 
         if (status != MPI_STATUS_IGNORE) {
             status->MPI_SOURCE = MPIDI_UCX_get_source(info.sender_tag);
@@ -49,17 +50,19 @@ static inline int MPIDI_NM_mpi_improbe(int source,
     *message = req;
 
   fn_exit:
+    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vni_dst).lock);
     return mpi_errno;
   fn_fail:
     goto fn_exit;
 }
 
 
-static inline int MPIDI_NM_mpi_iprobe(int source,
-                                      int tag,
-                                      MPIR_Comm * comm,
-                                      int context_offset,
-                                      MPIDI_av_entry_t * addr, int *flag, MPI_Status * status)
+MPL_STATIC_INLINE_PREFIX int MPIDI_NM_mpi_iprobe(int source,
+                                                 int tag,
+                                                 MPIR_Comm * comm,
+                                                 int context_offset,
+                                                 MPIDI_av_entry_t * addr, int *flag,
+                                                 MPI_Status * status)
 {
     int mpi_errno = MPI_SUCCESS;
     uint64_t ucp_tag, tag_mask;
@@ -67,10 +70,13 @@ static inline int MPIDI_NM_mpi_iprobe(int source,
     ucp_tag_recv_info_t info;
     ucp_tag_message_h message_h;
 
+    int vni_dst = MPIDI_UCX_get_vni(DST_VCI_FROM_RECVER, comm, source, comm->rank, tag);
+    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vni_dst).lock);
+
     tag_mask = MPIDI_UCX_tag_mask(tag, source);
     ucp_tag = MPIDI_UCX_recv_tag(tag, source, comm->recvcontext_id + context_offset);
 
-    message_h = ucp_tag_probe_nb(MPIDI_UCX_global.worker, ucp_tag, tag_mask, 0, &info);
+    message_h = ucp_tag_probe_nb(MPIDI_UCX_global.ctx[vni_dst].worker, ucp_tag, tag_mask, 0, &info);
 
     if (message_h) {
         *flag = 1;
@@ -86,6 +92,8 @@ static inline int MPIDI_NM_mpi_iprobe(int source,
         *flag = 0;
     }
 
+    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vni_dst).lock);
+
     return mpi_errno;
 }
 
