@@ -1,22 +1,63 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2006 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
- *
- *  Portions of this code were written by Intel Corporation.
- *  Copyright (C) 2011-2016 Intel Corporation.  Intel provides this material
- *  to Argonne National Laboratory subject to Software Grant and Corporate
- *  Contributor License Agreement dated February 8, 2012.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
+
 #ifndef CH4_RECV_H_INCLUDED
 #define CH4_RECV_H_INCLUDED
 
 #include "ch4_impl.h"
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_recv_unsafe
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
+#ifndef MPIDI_CH4_DIRECT_NETMOD
+MPL_STATIC_INLINE_PREFIX int anysource_irecv(void *buf, MPI_Aint count, MPI_Datatype datatype,
+                                             int rank, int tag, MPIR_Comm * comm,
+                                             int context_offset, MPIDI_av_entry_t * av,
+                                             MPIR_Request ** request)
+{
+    int mpi_errno = MPI_SUCCESS;
+    int need_unlock = 0;
+
+    mpi_errno = MPIDI_SHM_mpi_irecv(buf, count, datatype, rank, tag, comm, context_offset, request);
+    MPIR_ERR_CHECK(mpi_errno);
+
+    MPIR_Assert(*request);
+    /* 1. hold shm vci lock to prevent shm progress while we establish netmod request.
+     * 2. MPIDI_NM_mpi_irecv need use recursive locking in case it share the shm vci lock
+     */
+#if MPICH_THREAD_GRANULARITY == MPICH_THREAD_GRANULARITY__VCI
+    int shm_vci = MPIDI_Request_get_vci(*request);
+#endif
+    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(shm_vci).lock);
+    need_unlock = 1;
+    if (!MPIR_Request_is_complete(*request) && !MPIDIG_REQUEST_IN_PROGRESS(*request)) {
+        MPIR_Request *nm_rreq = NULL;
+        mpi_errno = MPIDI_NM_mpi_irecv(buf, count, datatype, rank, tag, comm, context_offset,
+                                       av, &nm_rreq, *request);
+        MPIR_ERR_CHECK(mpi_errno);
+        MPIDI_REQUEST_ANYSOURCE_PARTNER(*request) = nm_rreq;
+
+        /* cancel the shm request if netmod/am handles the request from unexpected queue. */
+        if (MPIR_Request_is_complete(nm_rreq)) {
+            mpi_errno = MPIDI_SHM_mpi_cancel_recv(*request);
+            if (MPIR_STATUS_GET_CANCEL_BIT((*request)->status)) {
+                (*request)->status = nm_rreq->status;
+            }
+            /* nm_rreq will be freed here. User-layer will have a completed (cancelled)
+             * request with correct status. */
+            MPIR_Request_free_unsafe(nm_rreq);
+            goto fn_exit;
+        }
+    }
+  fn_exit:
+    if (need_unlock) {
+        MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(shm_vci).lock);
+    }
+    return mpi_errno;
+  fn_fail:
+    goto fn_exit;
+}
+#endif
+
 MPL_STATIC_INLINE_PREFIX int MPIDI_recv_unsafe(void *buf,
                                                MPI_Aint count,
                                                MPI_Datatype datatype,
@@ -36,36 +77,10 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_recv_unsafe(void *buf,
                           request);
 #else
     if (unlikely(rank == MPI_ANY_SOURCE)) {
-        mpi_errno =
-            MPIDI_SHM_mpi_irecv(buf, count, datatype, rank, tag, comm, context_offset, request);
-
-        if (mpi_errno != MPI_SUCCESS) {
-            MPIR_ERR_POP(mpi_errno);
-        }
-
-        mpi_errno = MPIDI_NM_mpi_irecv(buf, count, datatype, rank, tag, comm, context_offset, av,
-                                       &(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request)));
-
-        if (mpi_errno != MPI_SUCCESS) {
-            MPIR_ERR_POP(mpi_errno);
-        }
-        /* cancel the shm request if netmod/am handles the request from unexpected queue. */
-        if (MPIR_Request_is_complete(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request))) {
-            mpi_errno = MPIDI_SHM_mpi_cancel_recv(*request);
-            if (MPIR_STATUS_GET_CANCEL_BIT((*request)->status)) {
-                (*request)->status = MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request)->status;
-            }
-            MPIR_Request_free(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request));
-            goto fn_exit;
-        }
-        MPIDI_CH4I_REQUEST(*request, is_local) = 1;
-        MPIDI_CH4I_REQUEST(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request), is_local) = 0;
-
-        MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request)) =
-            *request;
+        mpi_errno = anysource_irecv(buf, count, datatype, rank, tag, comm, context_offset,
+                                    av, request);
     } else {
-        int r;
-        if ((r = MPIDI_av_is_local(av)))
+        if (MPIDI_av_is_local(av))
             mpi_errno =
                 MPIDI_SHM_mpi_recv(buf, count, datatype, rank, tag, comm, context_offset, status,
                                    request);
@@ -73,16 +88,10 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_recv_unsafe(void *buf,
             mpi_errno =
                 MPIDI_NM_mpi_recv(buf, count, datatype, rank, tag, comm, context_offset, av, status,
                                   request);
-        if (mpi_errno == MPI_SUCCESS && *request) {
-            MPIDI_CH4I_REQUEST(*request, is_local) = r;
-            MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request) = NULL;
-        }
     }
 #endif
 
-    if (mpi_errno != MPI_SUCCESS) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
   fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_RECV_UNSAFE);
     return mpi_errno;
@@ -90,10 +99,6 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_recv_unsafe(void *buf,
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPID_irecv_unsafe
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPIDI_irecv_unsafe(void *buf,
                                                 MPI_Aint count,
                                                 MPI_Datatype datatype,
@@ -108,48 +113,25 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_irecv_unsafe(void *buf,
 
 
 #ifdef MPIDI_CH4_DIRECT_NETMOD
-    mpi_errno =
-        MPIDI_NM_mpi_irecv(buf, count, datatype, rank, tag, comm, context_offset, av, request);
+    mpi_errno = MPIDI_NM_mpi_irecv(buf, count, datatype, rank, tag, comm, context_offset, av,
+                                   request, NULL);
 #else
     if (unlikely(rank == MPI_ANY_SOURCE)) {
-        mpi_errno =
-            MPIDI_SHM_mpi_irecv(buf, count, datatype, rank, tag, comm, context_offset, request);
+        mpi_errno = anysource_irecv(buf, count, datatype, rank, tag, comm, context_offset,
+                                    av, request);
 
-        if (mpi_errno != MPI_SUCCESS) {
-            MPIR_ERR_POP(mpi_errno);
-        }
-
-        mpi_errno = MPIDI_NM_mpi_irecv(buf, count, datatype, rank, tag, comm, context_offset, av,
-                                       &(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request)));
-
-        if (mpi_errno != MPI_SUCCESS) {
-            MPIR_ERR_POP(mpi_errno);
-        } else if (*request) {
-            MPIDI_CH4I_REQUEST(*request, is_local) = 1;
-            MPIDI_CH4I_REQUEST(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request), is_local) = 0;
-        }
-
-        MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request)) =
-            *request;
     } else {
-        int r;
-        if ((r = MPIDI_av_is_local(av)))
+        if (MPIDI_av_is_local(av))
             mpi_errno =
                 MPIDI_SHM_mpi_irecv(buf, count, datatype, rank, tag, comm, context_offset, request);
         else
             mpi_errno =
                 MPIDI_NM_mpi_irecv(buf, count, datatype, rank, tag, comm, context_offset, av,
-                                   request);
-        if (mpi_errno == MPI_SUCCESS && *request) {
-            MPIDI_CH4I_REQUEST(*request, is_local) = r;
-            MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request) = NULL;
-        }
+                                   request, NULL);
     }
 #endif
 
-    if (mpi_errno != MPI_SUCCESS) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
   fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_IRECV_UNSAFE);
     return mpi_errno;
@@ -157,13 +139,9 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_irecv_unsafe(void *buf,
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_imrecv_unsafe
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPIDI_imrecv_unsafe(void *buf,
                                                  MPI_Aint count, MPI_Datatype datatype,
-                                                 MPIR_Request * message, MPIR_Request ** rreqp)
+                                                 MPIR_Request * message)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_IMRECV_UNSAFE);
@@ -172,24 +150,16 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_imrecv_unsafe(void *buf,
 #ifdef MPIDI_CH4_DIRECT_NETMOD
     mpi_errno = MPIDI_NM_mpi_imrecv(buf, count, datatype, message);
 #else
-    if (MPIDI_CH4I_REQUEST(message, is_local))
-        mpi_errno = MPIDI_SHM_mpi_imrecv(buf, count, datatype, message, rreqp);
+    if (MPIDI_REQUEST(message, is_local))
+        mpi_errno = MPIDI_SHM_mpi_imrecv(buf, count, datatype, message);
     else
         mpi_errno = MPIDI_NM_mpi_imrecv(buf, count, datatype, message);
 #endif
 
-  fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_IMRECV_UNSAFE);
     return mpi_errno;
-
-  fn_fail:
-    goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_cancel_recv_unsafe
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPIDI_cancel_recv_unsafe(MPIR_Request * rreq)
 {
     int mpi_errno;
@@ -199,11 +169,20 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_cancel_recv_unsafe(MPIR_Request * rreq)
 #ifdef MPIDI_CH4_DIRECT_NETMOD
     mpi_errno = MPIDI_NM_mpi_cancel_recv(rreq);
 #else
-    if (MPIDI_CH4I_REQUEST(rreq, is_local))
+    if (MPIDI_REQUEST(rreq, is_local)) {
+        MPIR_Request *partner_rreq = MPIDI_REQUEST_ANYSOURCE_PARTNER(rreq);
+        if (unlikely(partner_rreq)) {
+            /* Canceling MPI_ANY_SOURCE receive -- first cancel NM recv, then SHM */
+            mpi_errno = MPIDI_NM_mpi_cancel_recv(partner_rreq);
+            MPIR_ERR_CHECK(mpi_errno);
+            MPIR_Request_free_unsafe(partner_rreq);
+        }
         mpi_errno = MPIDI_SHM_mpi_cancel_recv(rreq);
-    else
+    } else {
         mpi_errno = MPIDI_NM_mpi_cancel_recv(rreq);
+    }
 #endif
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_CANCEL_RECV_UNSAFE);
@@ -212,12 +191,8 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_cancel_recv_unsafe(MPIR_Request * rreq)
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_recv_safe
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPIDI_recv_safe(void *buf,
-                                             int count,
+                                             MPI_Aint count,
                                              MPI_Datatype datatype,
                                              int rank,
                                              int tag,
@@ -225,30 +200,27 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_recv_safe(void *buf,
                                              int context_offset, MPIDI_av_entry_t * av,
                                              MPI_Status * status, MPIR_Request ** req)
 {
-    int mpi_errno = MPI_SUCCESS, cs_acq = 0;
+    int mpi_errno = MPI_SUCCESS;
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_RECV_SAFE);
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_RECV_SAFE);
 
-    MPID_THREAD_SAFE_BEGIN(VNI, MPIDI_CH4_Global.vni_lock, cs_acq);
-
-    if (!cs_acq) {
-        *(req) = MPIR_Request_create(MPIR_REQUEST_KIND__RECV);
-        MPIR_ERR_CHKANDSTMT((*req) == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
-        MPIR_Datatype_add_ref_if_not_builtin(datatype);
-        MPIDI_workq_pt2pt_enqueue(RECV, NULL /*send_buf */ , buf, count, datatype,
-                                  rank, tag, comm, context_offset, av,
-                                  status, *req, NULL /*flag */ , NULL /*message */ ,
-                                  NULL /*processed */);
-    } else {
-        *(req) = NULL;
-        MPIDI_workq_vni_progress_unsafe();
-        mpi_errno =
-            MPIDI_recv_unsafe(buf, count, datatype, rank, tag, comm, context_offset, av, status,
-                              req);
-    }
+#ifdef MPIDI_CH4_USE_WORK_QUEUES
+    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(0).lock);
+    *(req) = MPIR_Request_create_from_pool(MPIR_REQUEST_KIND__RECV, 0);
+    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(0).lock);
+    MPIR_ERR_CHKANDSTMT((*req) == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
+    MPIR_Datatype_add_ref_if_not_builtin(datatype);
+    MPIDI_workq_pt2pt_enqueue(RECV, NULL /*send_buf */ , buf, count, datatype,
+                              rank, tag, comm, context_offset, av,
+                              status, *req, NULL /*flag */ , NULL /*message */ ,
+                              NULL /*processed */);
+#else
+    *(req) = NULL;
+    mpi_errno = MPIDI_recv_unsafe(buf, count, datatype, rank, tag, comm,
+                                  context_offset, av, status, req);
+#endif
 
   fn_exit:
-    MPID_THREAD_SAFE_END(VNI, MPIDI_CH4_Global.vni_lock, cs_acq);
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_RECV_SAFE);
     return mpi_errno;
 
@@ -256,12 +228,8 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_recv_safe(void *buf,
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_irecv_safe
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPIDI_irecv_safe(void *buf,
-                                              int count,
+                                              MPI_Aint count,
                                               MPI_Datatype datatype,
                                               int rank,
                                               int tag,
@@ -269,29 +237,26 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_irecv_safe(void *buf,
                                               int context_offset, MPIDI_av_entry_t * av,
                                               MPIR_Request ** req)
 {
-    int mpi_errno = MPI_SUCCESS, cs_acq = 0;
+    int mpi_errno = MPI_SUCCESS;
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_IRECV_SAFE);
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_IRECV_SAFE);
 
-    MPID_THREAD_SAFE_BEGIN(VNI, MPIDI_CH4_Global.vni_lock, cs_acq);
-
-    if (!cs_acq) {
-        *(req) = MPIR_Request_create(MPIR_REQUEST_KIND__RECV);
-        MPIR_ERR_CHKANDSTMT((*req) == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
-        MPIR_Datatype_add_ref_if_not_builtin(datatype);
-        MPIDI_workq_pt2pt_enqueue(IRECV, NULL /*send_buf */ , buf, count, datatype,
-                                  rank, tag, comm, context_offset, av,
-                                  NULL /*status */ , *req, NULL /*flag */ , NULL /*message */ ,
-                                  NULL /*processed */);
-    } else {
-        *(req) = NULL;
-        MPIDI_workq_vni_progress_unsafe();
-        mpi_errno =
-            MPIDI_irecv_unsafe(buf, count, datatype, rank, tag, comm, context_offset, av, req);
-    }
+#ifdef MPIDI_CH4_USE_WORK_QUEUES
+    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(0).lock);
+    *(req) = MPIR_Request_create_from_pool(MPIR_REQUEST_KIND__RECV, 0);
+    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(0).lock);
+    MPIR_ERR_CHKANDSTMT((*req) == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
+    MPIR_Datatype_add_ref_if_not_builtin(datatype);
+    MPIDI_workq_pt2pt_enqueue(IRECV, NULL /*send_buf */ , buf, count, datatype,
+                              rank, tag, comm, context_offset, av,
+                              NULL /*status */ , *req, NULL /*flag */ , NULL /*message */ ,
+                              NULL /*processed */);
+#else
+    *(req) = NULL;
+    mpi_errno = MPIDI_irecv_unsafe(buf, count, datatype, rank, tag, comm, context_offset, av, req);
+#endif
 
   fn_exit:
-    MPID_THREAD_SAFE_END(VNI, MPIDI_CH4_Global.vni_lock, cs_acq);
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_IRECV_SAFE);
     return mpi_errno;
 
@@ -299,43 +264,30 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_irecv_safe(void *buf,
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_imrecv_safe
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPIDI_imrecv_safe(void *buf,
                                                MPI_Aint count, MPI_Datatype datatype,
-                                               MPIR_Request * message, MPIR_Request ** rreqp)
+                                               MPIR_Request * message)
 {
-    int mpi_errno = MPI_SUCCESS, cs_acq = 0;
+    int mpi_errno = MPI_SUCCESS;
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_IMRECV_SAFE);
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_IMRECV_SAFE);
 
-    MPID_THREAD_SAFE_BEGIN(VNI, MPIDI_CH4_Global.vni_lock, cs_acq);
-
-    if (!cs_acq) {
-        MPIR_Request *request = MPIR_Request_create(MPIR_REQUEST_KIND__RECV);
-#ifndef MPIDI_CH4_DIRECT_NETMOD
-        *rreqp = request;
+#ifdef MPIDI_CH4_USE_WORK_QUEUES
+    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(0).lock);
+    MPIR_Request *request = MPIR_Request_create_from_pool(MPIR_REQUEST_KIND__RECV, 0);
+    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(0).lock);
+    MPIR_ERR_CHKANDSTMT(request == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
+    MPIR_Datatype_add_ref_if_not_builtin(datatype);
+    MPIDI_workq_pt2pt_enqueue(IMRECV, NULL /*send_buf */ , buf, count, datatype,
+                              0 /*rank */ , 0 /*tag */ , NULL /*comm */ ,
+                              0 /*context_offset */ , NULL /*av */ ,
+                              NULL /*status */ , request, NULL /*flag */ ,
+                              &message, NULL /*processed */);
+#else
+    mpi_errno = MPIDI_imrecv_unsafe(buf, count, datatype, message);
 #endif
-        MPIR_ERR_CHKANDSTMT(request == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
-        MPIR_Datatype_add_ref_if_not_builtin(datatype);
-        MPIDI_workq_pt2pt_enqueue(IMRECV, NULL /*send_buf */ , buf, count, datatype,
-                                  0 /*rank */ , 0 /*tag */ , NULL /*comm */ ,
-                                  0 /*context_offset */ , NULL /*av */ ,
-                                  NULL /*status */ , request, NULL /*flag */ ,
-                                  &message, NULL /*processed */);
-    } else {
-        MPIDI_workq_vni_progress_unsafe();
-#ifndef MPIDI_CH4_DIRECT_NETMOD
-        if (MPIDI_CH4I_REQUEST(message, is_local))
-            *rreqp = NULL;
-#endif
-        mpi_errno = MPIDI_imrecv_unsafe(buf, count, datatype, message, rreqp);
-    }
 
   fn_exit:
-    MPID_THREAD_SAFE_END(VNI, MPIDI_CH4_Global.vni_lock, cs_acq);
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_IMRECV_SAFE);
     return mpi_errno;
 
@@ -343,24 +295,26 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_imrecv_safe(void *buf,
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_cancel_recv_safe
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPIDI_cancel_recv_safe(MPIR_Request * rreq)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_IRECV_SAFE);
-    MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_IRECV_SAFE);
-
-    MPID_THREAD_CS_ENTER(VNI, MPIDI_CH4_Global.vni_lock);
+    MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPIDI_CANCEL_RECV_SAFE);
+    MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPIDI_CANCEL_RECV_SAFE);
 
+#if MPICH_THREAD_GRANULARITY == MPICH_THREAD_GRANULARITY__VCI
+    int vci = MPIDI_Request_get_vci(rreq);
+#endif
+    /* MPIDI_NM_mpi_cancel_recv is used both externally and internally. For internal
+     * usage it's often used inside a critical section (e.g. progress and anysource
+     * receive). Therefore, we allow recursive lock usage here.
+     */
+    MPID_THREAD_CS_ENTER_REC_VCI(MPIDI_VCI(vci).lock);
     mpi_errno = MPIDI_cancel_recv_unsafe(rreq);
-
-    MPID_THREAD_CS_EXIT(VNI, MPIDI_CH4_Global.vni_lock);
+    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vci).lock);
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
-    MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_IRECV_SAFE);
+    MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPIDI_CANCEL_RECV_SAFE);
     return mpi_errno;
 
   fn_fail:
@@ -368,10 +322,6 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_cancel_recv_safe(MPIR_Request * rreq)
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPID_Recv
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPID_Recv(void *buf,
                                        MPI_Aint count,
                                        MPI_Datatype datatype,
@@ -382,29 +332,14 @@ MPL_STATIC_INLINE_PREFIX int MPID_Recv(void *buf,
                                        MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIDI_av_entry_t *av = NULL;
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPID_RECV);
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPID_RECV);
 
-    if (unlikely(rank == MPI_PROC_NULL)) {
-        MPIR_Request *rreq = MPIR_Request_create(MPIR_REQUEST_KIND__RECV);
-        MPIR_ERR_CHKANDSTMT((rreq) == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
-        *request = rreq;
-        MPIR_Request_add_ref(rreq);
-        rreq->status.MPI_SOURCE = rank;
-        rreq->status.MPI_TAG = tag;
-        MPIDI_CH4U_request_complete(rreq);
-        mpi_errno = MPI_SUCCESS;
-        goto fn_exit;
-    }
-
-    av = MPIDIU_comm_rank_to_av(comm, rank);
+    MPIDI_av_entry_t *av = (rank == MPI_ANY_SOURCE ? NULL : MPIDIU_comm_rank_to_av(comm, rank));
     mpi_errno =
         MPIDI_recv_safe(buf, count, datatype, rank, tag, comm, context_offset, av, status, request);
 
-    if (mpi_errno != MPI_SUCCESS) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
   fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_RECV);
     return mpi_errno;
@@ -412,79 +347,49 @@ MPL_STATIC_INLINE_PREFIX int MPID_Recv(void *buf,
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPID_Recv_init
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPID_Recv_init(void *buf,
-                                            int count,
+                                            MPI_Aint count,
                                             MPI_Datatype datatype,
                                             int rank,
                                             int tag,
                                             MPIR_Comm * comm, int context_offset,
                                             MPIR_Request ** request)
 {
-    int mpi_errno;
-    MPIDI_av_entry_t *av = NULL;
+    int mpi_errno = MPI_SUCCESS;
+    MPIR_Request *rreq;
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPID_RECV_INIT);
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPID_RECV_INIT);
 
-    av = MPIDIU_comm_rank_to_av(comm, rank);
-#ifdef MPIDI_CH4_DIRECT_NETMOD
-    mpi_errno =
-        MPIDI_NM_mpi_recv_init(buf, count, datatype, rank, tag, comm, context_offset, av, request);
-#else
-    if (unlikely(rank == MPI_ANY_SOURCE)) {
-        mpi_errno =
-            MPIDI_SHM_mpi_recv_init(buf, count, datatype, rank, tag, comm, context_offset, request);
+    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(0).lock);
+    rreq = MPIR_Request_create_from_pool(MPIR_REQUEST_KIND__PREQUEST_RECV, 0);
+    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(0).lock);
+    MPIR_ERR_CHKANDSTMT(rreq == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
 
-        if (mpi_errno != MPI_SUCCESS) {
-            MPIR_ERR_POP(mpi_errno);
-        }
+    *request = rreq;
+    rreq->comm = comm;
+    MPIR_Comm_add_ref(comm);
 
-        mpi_errno =
-            MPIDI_NM_mpi_recv_init(buf, count, datatype, rank, tag, comm, context_offset, av,
-                                   &(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request)));
+    MPIDI_PREQUEST(rreq, buffer) = (void *) buf;
+    MPIDI_PREQUEST(rreq, count) = count;
+    MPIDI_PREQUEST(rreq, datatype) = datatype;
+    MPIDI_PREQUEST(rreq, rank) = rank;
+    MPIDI_PREQUEST(rreq, tag) = tag;
+    MPIDI_PREQUEST(rreq, context_id) = comm->context_id + context_offset;
+    rreq->u.persist.real_request = NULL;
+    MPIR_cc_set(rreq->cc_ptr, 0);
 
-        if (mpi_errno != MPI_SUCCESS) {
-            MPIR_ERR_POP(mpi_errno);
-        }
+    MPIDI_PREQUEST(rreq, p_type) = MPIDI_PTYPE_RECV;
+    MPIR_Datatype_add_ref_if_not_builtin(datatype);
 
-        MPIDI_CH4I_REQUEST(*request, is_local) = 1;
-        MPIDI_CH4I_REQUEST(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request), is_local) = 0;
-
-        MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request)) =
-            *request;
-    } else {
-        int r;
-        if ((r = MPIDI_av_is_local(av)))
-            mpi_errno = MPIDI_SHM_mpi_recv_init(buf, count, datatype, rank, tag,
-                                                comm, context_offset, request);
-        else
-            mpi_errno = MPIDI_NM_mpi_recv_init(buf, count, datatype, rank, tag,
-                                               comm, context_offset, av, request);
-        if (mpi_errno == MPI_SUCCESS) {
-            MPIDI_CH4I_REQUEST(*request, is_local) = r;
-            MPIDI_CH4I_REQUEST_ANYSOURCE_PARTNER(*request) = NULL;
-        }
-    }
   fn_exit:
-#endif
-
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_RECV_INIT);
     return mpi_errno;
 
-#ifndef MPIDI_CH4_DIRECT_NETMOD
   fn_fail:
     goto fn_exit;
-#endif /* MPIDI_CH4_DIRECT_NETMOD */
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPID_Mrecv
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPID_Mrecv(void *buf,
                                         MPI_Aint count,
                                         MPI_Datatype datatype, MPIR_Request * message,
@@ -496,7 +401,7 @@ MPL_STATIC_INLINE_PREFIX int MPID_Mrecv(void *buf,
 
     *rreq = NULL;
 
-    if (message == NULL) {
+    if (message == NULL || message->handle == MPIR_REQUEST_NULL_RECV) {
         /* treat as though MPI_MESSAGE_NO_PROC was passed */
         MPIR_Status_set_procnull(status);
         mpi_errno = MPI_SUCCESS;
@@ -505,12 +410,10 @@ MPL_STATIC_INLINE_PREFIX int MPID_Mrecv(void *buf,
 
     MPIR_Assert(message->kind == MPIR_REQUEST_KIND__MPROBE);
     message->kind = MPIR_REQUEST_KIND__RECV;
-    *rreq = message;    /* SHM will override this pointer */
+    *rreq = message;
 
-    mpi_errno = MPIDI_imrecv_safe(buf, count, datatype, message, rreq);
-    if (mpi_errno != MPI_SUCCESS) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    mpi_errno = MPIDI_imrecv_safe(buf, count, datatype, message);
+    MPIR_ERR_CHECK(mpi_errno);
 
   fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_MRECV);
@@ -519,10 +422,6 @@ MPL_STATIC_INLINE_PREFIX int MPID_Mrecv(void *buf,
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPID_Imrecv
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPID_Imrecv(void *buf, MPI_Aint count, MPI_Datatype datatype,
                                          MPIR_Request * message, MPIR_Request ** rreqp)
 {
@@ -532,18 +431,16 @@ MPL_STATIC_INLINE_PREFIX int MPID_Imrecv(void *buf, MPI_Aint count, MPI_Datatype
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPID_IMRECV);
 
     if (message == NULL) {
-        MPIDI_Request_create_null_rreq(*rreqp, mpi_errno, goto fn_fail);
+        *rreqp = MPIR_Request_create_null_recv();
         goto fn_exit;
     }
 
     MPIR_Assert(message->kind == MPIR_REQUEST_KIND__MPROBE);
     message->kind = MPIR_REQUEST_KIND__RECV;
-    *rreqp = message;   /* SHM will override this pointer */
+    *rreqp = message;
 
-    mpi_errno = MPIDI_imrecv_safe(buf, count, datatype, message, rreqp);
-    if (mpi_errno != MPI_SUCCESS) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    mpi_errno = MPIDI_imrecv_safe(buf, count, datatype, message);
+    MPIR_ERR_CHECK(mpi_errno);
   fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_IMRECV);
     return mpi_errno;
@@ -551,10 +448,6 @@ MPL_STATIC_INLINE_PREFIX int MPID_Imrecv(void *buf, MPI_Aint count, MPI_Datatype
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPID_Irecv
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPID_Irecv(void *buf,
                                         MPI_Aint count,
                                         MPI_Datatype datatype,
@@ -564,29 +457,14 @@ MPL_STATIC_INLINE_PREFIX int MPID_Irecv(void *buf,
                                         MPIR_Request ** request)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIDI_av_entry_t *av = NULL;
     MPIR_FUNC_VERBOSE_STATE_DECL(MPID_STATE_MPID_IRECV);
     MPIR_FUNC_VERBOSE_ENTER(MPID_STATE_MPID_IRECV);
 
-    if (unlikely(rank == MPI_PROC_NULL)) {
-        MPIR_Request *rreq = MPIR_Request_create(MPIR_REQUEST_KIND__RECV);
-        MPIR_ERR_CHKANDSTMT((rreq) == NULL, mpi_errno, MPIX_ERR_NOREQ, goto fn_fail, "**nomemreq");
-        *request = rreq;
-        MPIR_Request_add_ref(rreq);
-        rreq->status.MPI_SOURCE = rank;
-        rreq->status.MPI_TAG = tag;
-        MPIDI_CH4U_request_complete(rreq);
-        mpi_errno = MPI_SUCCESS;
-        goto fn_exit;
-    }
-
-    av = MPIDIU_comm_rank_to_av(comm, rank);
+    MPIDI_av_entry_t *av = (rank == MPI_ANY_SOURCE ? NULL : MPIDIU_comm_rank_to_av(comm, rank));
     mpi_errno =
         MPIDI_irecv_safe(buf, count, datatype, rank, tag, comm, context_offset, av, request);
 
-    if (mpi_errno != MPI_SUCCESS) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
   fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_IRECV);
     return mpi_errno;
@@ -594,10 +472,6 @@ MPL_STATIC_INLINE_PREFIX int MPID_Irecv(void *buf,
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_Cancel_Recv
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 MPL_STATIC_INLINE_PREFIX int MPID_Cancel_recv(MPIR_Request * rreq)
 {
     int mpi_errno;
@@ -606,9 +480,7 @@ MPL_STATIC_INLINE_PREFIX int MPID_Cancel_recv(MPIR_Request * rreq)
 
     mpi_errno = MPIDI_cancel_recv_safe(rreq);
 
-    if (mpi_errno != MPI_SUCCESS) {
-        MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
   fn_exit:
     MPIR_FUNC_VERBOSE_EXIT(MPID_STATE_MPID_CANCEL_RECV);
     return mpi_errno;
