@@ -1,16 +1,16 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2010 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "hydra.h"
 #include "pmip.h"
 #include "pmip_pmi.h"
-#include "ckpoint.h"
 #include "demux.h"
 #include "topo.h"
 
+#define MAX_GPU_STR_LEN   (128)
+
 struct HYD_pmcd_pmip_pmi_handle *HYD_pmcd_pmip_pmi_handle = { 0 };
 
 static int pmi_storage_len = 0;
@@ -93,6 +93,76 @@ static HYD_status stdoe_cb(int fd, HYD_event_t events, void *userp)
     goto fn_exit;
 }
 
+static HYD_status handle_pmi_cmd(int fd, char *buf, struct HYD_pmcd_hdr hdr)
+{
+    struct HYD_pmcd_pmip_pmi_handle *h;
+    char *pmi_cmd = NULL, **args = NULL;
+    int sent, closed;
+    HYD_status status = HYD_SUCCESS;
+
+    HYDU_FUNC_ENTER();
+
+    if (hdr.pmi_version == 1)
+        HYD_pmcd_pmip_pmi_handle = HYD_pmcd_pmip_pmi_v1;
+    else
+        HYD_pmcd_pmip_pmi_handle = HYD_pmcd_pmip_pmi_v2;
+
+    HYDU_MALLOC_OR_JUMP(args, char **, MAX_PMI_ARGS * sizeof(char *), status);
+    for (int i = 0; i < MAX_PMI_ARGS; i++)
+        args[i] = NULL;
+
+    status = HYD_pmcd_pmi_parse_pmi_cmd(buf, hdr.pmi_version, &pmi_cmd, args);
+    HYDU_ERR_POP(status, "unable to parse PMI command\n");
+
+    if (HYD_pmcd_pmip.user_global.debug) {
+        HYDU_dump(stdout, "got pmi command (from %d): %s\n", fd, pmi_cmd);
+        HYDU_print_strlist(args);
+    }
+
+    h = HYD_pmcd_pmip_pmi_handle;
+    while (h->handler) {
+        if (!strcmp(pmi_cmd, h->cmd)) {
+            status = h->handler(fd, args);
+            HYDU_ERR_POP(status, "PMI handler returned error\n");
+            goto fn_exit;
+        }
+        h++;
+    }
+
+    if (HYD_pmcd_pmip.user_global.debug) {
+        HYDU_dump(stdout, "we don't understand this command %s; forwarding upstream\n", pmi_cmd);
+    }
+
+    /* We don't understand the command; forward it upstream */
+    hdr.cmd = PMI_CMD;
+    hdr.pid = fd;
+    hdr.buflen = strlen(buf);
+    status =
+        HYDU_sock_write(HYD_pmcd_pmip.upstream.control, &hdr, sizeof(hdr), &sent, &closed,
+                        HYDU_SOCK_COMM_MSGWAIT);
+    HYDU_ERR_POP(status, "unable to send PMI header upstream\n");
+    HYDU_ASSERT(!closed, status);
+
+    status =
+        HYDU_sock_write(HYD_pmcd_pmip.upstream.control, buf, hdr.buflen, &sent, &closed,
+                        HYDU_SOCK_COMM_MSGWAIT);
+    HYDU_ERR_POP(status, "unable to send PMI command upstream\n");
+    HYDU_ASSERT(!closed, status);
+
+  fn_exit:
+    MPL_free(pmi_cmd);
+    if (args) {
+        HYDU_free_strlist(args);
+        MPL_free(args);
+    }
+
+    HYDU_FUNC_EXIT();
+    return status;
+
+  fn_fail:
+    goto fn_exit;
+}
+
 static HYD_status check_pmi_cmd(char **buf, int *pmi_version, int *repeat)
 {
     int full_command, buflen, cmdlen;
@@ -195,10 +265,9 @@ static HYD_status check_pmi_cmd(char **buf, int *pmi_version, int *repeat)
 
 static HYD_status pmi_cb(int fd, HYD_event_t events, void *userp)
 {
-    char *buf = NULL, *pmi_cmd = NULL, **args = NULL;
+    char *buf = NULL;
     int closed, repeat, sent, i = -1, linelen, pid = -1;
     struct HYD_pmcd_hdr hdr;
-    struct HYD_pmcd_pmip_pmi_handle *h;
     HYD_status status = HYD_SUCCESS;
 
     HYDU_FUNC_ENTER();
@@ -288,66 +357,15 @@ static HYD_status pmi_cb(int fd, HYD_event_t events, void *userp)
     if (pid != -1 && !HYD_pmcd_pmip.downstream.pmi_fd_active[pid])
         HYD_pmcd_pmip.downstream.pmi_fd_active[pid] = 1;
 
-    if (hdr.pmi_version == 1)
-        HYD_pmcd_pmip_pmi_handle = HYD_pmcd_pmip_pmi_v1;
-    else
-        HYD_pmcd_pmip_pmi_handle = HYD_pmcd_pmip_pmi_v2;
-
-    HYDU_MALLOC_OR_JUMP(args, char **, MAX_PMI_ARGS * sizeof(char *), status);
-    for (i = 0; i < MAX_PMI_ARGS; i++)
-        args[i] = NULL;
-
-    status = HYD_pmcd_pmi_parse_pmi_cmd(buf, hdr.pmi_version, &pmi_cmd, args);
-    HYDU_ERR_POP(status, "unable to parse PMI command\n");
-
-    if (HYD_pmcd_pmip.user_global.debug) {
-        HYDU_dump(stdout, "got pmi command (from %d): %s\n", fd, pmi_cmd);
-        HYDU_print_strlist(args);
-    }
-
-    h = HYD_pmcd_pmip_pmi_handle;
-    while (h->handler) {
-        if (!strcmp(pmi_cmd, h->cmd)) {
-            status = h->handler(fd, args);
-            HYDU_ERR_POP(status, "PMI handler returned error\n");
-            goto fn_exit;
-        }
-        h++;
-    }
-
-    if (HYD_pmcd_pmip.user_global.debug) {
-        HYDU_dump(stdout, "we don't understand this command %s; forwarding upstream\n", pmi_cmd);
-    }
-
-    /* We don't understand the command; forward it upstream */
-    hdr.cmd = PMI_CMD;
-    hdr.pid = fd;
-    hdr.buflen = strlen(buf);
-    status =
-        HYDU_sock_write(HYD_pmcd_pmip.upstream.control, &hdr, sizeof(hdr), &sent, &closed,
-                        HYDU_SOCK_COMM_MSGWAIT);
-    HYDU_ERR_POP(status, "unable to send PMI header upstream\n");
-    HYDU_ASSERT(!closed, status);
-
-    status =
-        HYDU_sock_write(HYD_pmcd_pmip.upstream.control, buf, hdr.buflen, &sent, &closed,
-                        HYDU_SOCK_COMM_MSGWAIT);
-    HYDU_ERR_POP(status, "unable to send PMI command upstream\n");
-    HYDU_ASSERT(!closed, status);
+    status = handle_pmi_cmd(fd, buf, hdr);
+    MPL_free(buf);
+    HYDU_ERR_POP(status, "unable to handle PMI command\n");
 
     if (repeat)
         /* there are more commands to process. */
         goto check_cmd;
 
   fn_exit:
-    if (pmi_cmd)
-        MPL_free(pmi_cmd);
-    if (args) {
-        HYDU_free_strlist(args);
-        MPL_free(args);
-    }
-    if (buf)
-        MPL_free(buf);
     HYDU_FUNC_EXIT();
     return status;
 
@@ -403,14 +421,12 @@ static HYD_status handle_pmi_response(int fd, struct HYD_pmcd_hdr hdr)
     }
 
   fn_exit:
-    if (pmi_cmd)
-        MPL_free(pmi_cmd);
+    MPL_free(pmi_cmd);
     if (args) {
         HYDU_free_strlist(args);
         MPL_free(args);
     }
-    if (buf)
-        MPL_free(buf);
+    MPL_free(buf);
     HYDU_FUNC_EXIT();
     return status;
 
@@ -462,7 +478,6 @@ static int local_to_global_id(int local_id)
 static HYD_status launch_procs(void)
 {
     int i, j, process_id, dummy;
-    int using_pmi_port = 0;
     char *str, *envstr, *list, *pmi_port = NULL;
     struct HYD_string_stash stash;
     struct HYD_env *env, *force_env = NULL;
@@ -513,34 +528,13 @@ static HYD_status launch_procs(void)
                             HYD_pmcd_pmip.user_global.mapping, HYD_pmcd_pmip.user_global.membind);
     HYDU_ERR_POP(status, "unable to initialize process topology\n");
 
-    status = HYDT_ckpoint_init(HYD_pmcd_pmip.user_global.ckpointlib,
-                               HYD_pmcd_pmip.user_global.ckpoint_num);
-    HYDU_ERR_POP(status, "unable to initialize checkpointing\n");
-
-    if (HYD_pmcd_pmip.user_global.ckpoint_prefix) {
-        using_pmi_port = 1;
+    if (HYD_pmcd_pmip.user_global.pmi_port) {
         status = HYDU_sock_create_and_listen_portstr(HYD_pmcd_pmip.user_global.iface,
                                                      NULL, NULL, &pmi_port, pmi_listen_cb, NULL);
         HYDU_ERR_POP(status, "unable to create PMI port\n");
-    }
 
-    if (HYD_pmcd_pmip.exec_list->exec[0] == NULL) {     /* Checkpoint restart case */
         status = HYDU_env_create(&env, "PMI_PORT", pmi_port);
         HYDU_ERR_POP(status, "unable to create env\n");
-
-        /* Restart the proxy -- we use the first prefix in the list */
-        status = HYDT_ckpoint_restart(HYD_pmcd_pmip.local.pgid, HYD_pmcd_pmip.local.id,
-                                      env, HYD_pmcd_pmip.local.proxy_process_count,
-                                      HYD_pmcd_pmip.downstream.pmi_rank,
-                                      HYD_pmcd_pmip.downstream.pmi_rank[0] ? NULL :
-                                      &HYD_pmcd_pmip.downstream.in,
-                                      HYD_pmcd_pmip.downstream.out,
-                                      HYD_pmcd_pmip.downstream.err,
-                                      HYD_pmcd_pmip.downstream.pid,
-                                      HYD_pmcd_pmip.local.ckpoint_prefix_list[0]);
-        HYDU_ERR_POP(status, "unable to restart from checkpoint\n");
-
-        goto fn_spawn_complete;
     }
 
     /* Spawn the processes */
@@ -633,12 +627,46 @@ static HYD_status launch_procs(void)
             str = HYDU_int_to_str(HYD_pmcd_pmip.local.proxy_process_count);
             status = HYDU_append_env_to_list("MPI_LOCALNRANKS", str, &force_env);
             HYDU_ERR_POP(status, "unable to add env to list\n");
+            MPL_free(str);
 
             str = HYDU_int_to_str(process_id);
             status = HYDU_append_env_to_list("MPI_LOCALRANKID", str, &force_env);
             HYDU_ERR_POP(status, "unable to add env to list\n");
+            MPL_free(str);
 
-            if (using_pmi_port) {
+            if (HYD_pmcd_pmip.user_global.gpus_per_proc == HYD_GPUS_PER_PROC_AUTO) {
+                /* nothing to do */
+            } else if (HYD_pmcd_pmip.user_global.gpus_per_proc == 0) {
+                str = HYDU_int_to_str(-1);
+
+                status = HYDU_append_env_to_list("CUDA_VISIBLE_DEVICES", str, &force_env);
+                HYDU_ERR_POP(status, "unable to add env to list\n");
+
+                MPL_free(str);
+            } else {
+                char cuda_str[MAX_GPU_STR_LEN] = { 0 };
+                int cuda_str_offset = 0;
+
+                for (int k = 0; k < HYD_pmcd_pmip.user_global.gpus_per_proc; k++) {
+                    int p = process_id * HYD_pmcd_pmip.user_global.gpus_per_proc + k;
+                    str = HYDU_int_to_str(p);
+
+                    if (k) {
+                        MPL_strncpy(cuda_str + cuda_str_offset, ",",
+                                    MAX_GPU_STR_LEN - cuda_str_offset);
+                        cuda_str_offset++;
+                    }
+                    MPL_strncpy(cuda_str + cuda_str_offset, str, MAX_GPU_STR_LEN - cuda_str_offset);
+                    cuda_str_offset += strlen(str);
+
+                    MPL_free(str);
+                }
+
+                status = HYDU_append_env_to_list("CUDA_VISIBLE_DEVICES", cuda_str, &force_env);
+                HYDU_ERR_POP(status, "unable to add env to list\n");
+            }
+
+            if (HYD_pmcd_pmip.user_global.pmi_port) {
                 /* If we are using the PMI_PORT format */
 
                 /* PMI_PORT */
@@ -793,19 +821,10 @@ static HYD_status parse_exec_params(char **t_argv)
     if (HYD_pmcd_pmip.local.proxy_core_count == -1)
         HYDU_ERR_SETANDJUMP(status, HYD_INTERNAL_ERROR, "proxy core count not available\n");
 
-    if (HYD_pmcd_pmip.exec_list == NULL && HYD_pmcd_pmip.user_global.ckpoint_prefix == NULL)
-        HYDU_ERR_SETANDJUMP(status, HYD_INTERNAL_ERROR,
-                            "no executable given and doesn't look like a restart either\n");
-
     /* Set default values */
     if (HYD_pmcd_pmip.user_global.topolib == NULL && HYDRA_DEFAULT_TOPOLIB != NULL)
         HYD_pmcd_pmip.user_global.topolib = MPL_strdup(HYDRA_DEFAULT_TOPOLIB);
 
-#ifdef HYDRA_DEFAULT_CKPOINTLIB
-    if (HYD_pmcd_pmip.user_global.ckpointlib == NULL)
-        HYD_pmcd_pmip.user_global.ckpointlib = MPL_strdup(HYDRA_DEFAULT_CKPOINTLIB);
-#endif
-
   fn_exit:
     HYDU_FUNC_EXIT();
     return status;
@@ -883,14 +902,6 @@ HYD_status HYD_pmcd_pmip_control_cmd_cb(int fd, HYD_event_t events, void *userp)
 
         status = launch_procs();
         HYDU_ERR_POP(status, "launch_procs returned error\n");
-    } else if (hdr.cmd == CKPOINT) {
-        HYDU_dump(stdout, "requesting checkpoint\n");
-
-        status = HYDT_ckpoint_checkpoint(HYD_pmcd_pmip.local.pgid, HYD_pmcd_pmip.local.id,
-                                         HYD_pmcd_pmip.local.ckpoint_prefix_list[0]);
-
-        HYDU_ERR_POP(status, "checkpoint suspend failed\n");
-        HYDU_dump(stdout, "checkpoint completed\n");
     } else if (hdr.cmd == PMI_RESPONSE) {
         status = handle_pmi_response(fd, hdr);
         HYDU_ERR_POP(status, "unable to handle PMI response\n");
