@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2009 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "hydra_server.h"
@@ -45,8 +44,7 @@ HYD_status HYD_pmcd_pmi_free_publish(struct HYD_pmcd_pmi_publish * publish)
         MPL_free(publish->info_keys[i].key);
         MPL_free(publish->info_keys[i].val);
     }
-    if (publish->info_keys)
-        MPL_free(publish->info_keys);
+    MPL_free(publish->info_keys);
 
     HYDU_FUNC_EXIT();
     return status;
