@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2010 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "hydra.h"
@@ -45,8 +44,7 @@ HYD_status HYDT_bscd_lsf_query_node_list(struct HYD_node **node_list)
             hostname = strtok(NULL, " ");
         }
 
-        if (thosts)
-            MPL_free(thosts);
+        MPL_free(thosts);
     }
 
   fn_exit:
