@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2008 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "hydra_server.h"
@@ -13,7 +12,7 @@
 #include "ui.h"
 #include "uiu.h"
 
-struct HYD_server_info_s HYD_server_info = { {0} };
+struct HYD_server_info_s HYD_server_info;
 
 struct HYD_exec *HYD_uii_mpx_exec_list = NULL;
 struct HYD_ui_info_s HYD_ui_info;
@@ -27,25 +26,6 @@ static void signal_cb(int signum)
 
     HYDU_FUNC_ENTER();
 
-    /* SIGALRM is a special signal that indicates that a checkpoint
-     * needs to be initiated */
-    if (signum == SIGALRM) {
-        if (HYD_server_info.user_global.ckpoint_prefix == NULL) {
-            HYDU_dump(stderr, "No checkpoint prefix provided\n");
-            return;
-        }
-#if HAVE_ALARM
-        if (HYD_ui_mpich_info.ckpoint_int != -1)
-            alarm(HYD_ui_mpich_info.ckpoint_int);
-#endif /* HAVE_ALARM */
-
-        cmd.type = HYD_CKPOINT;
-        HYDU_sock_write(HYD_server_info.cmd_pipe[1], &cmd, sizeof(cmd), &sent, &closed,
-                        HYDU_SOCK_COMM_MSGWAIT);
-
-        goto fn_exit;
-    }
-
     cmd.type = HYD_SIGNAL;
     cmd.signum = signum;
 
@@ -147,13 +127,6 @@ int main(int argc, char **argv)
     status = HYD_uii_mpx_get_parameters(argv);
     HYDU_ERR_POP(status, "error parsing parameters\n");
 
-    /* Now we initialize engines that require us to know user
-     * preferences */
-#if HAVE_ALARM
-    if (HYD_ui_mpich_info.ckpoint_int != -1)
-        alarm(HYD_ui_mpich_info.ckpoint_int);
-#endif /* HAVE_ALARM */
-
     /* The demux engine should be initialized before any sockets are
      * created, since it checks for STDIN's validity.  If STDIN was
      * closed and we opened a socket that got the same fd as STDIN,
@@ -193,19 +166,36 @@ int main(int argc, char **argv)
         }
     }
 
-    /*
-     * If this is a checkpoint-restart, if the user specified the
-     * number of processes, we already have a dummy executable. If the
-     * number of processes came from the RMK, our executable list is
-     * still NULL; a dummy executable needs to be created.
-     */
-    if (HYD_uii_mpx_exec_list == NULL) {
-        HYDU_ASSERT(HYD_server_info.user_global.ckpoint_prefix, status);
+    if (HYD_server_info.user_global.skip_launch_node) {
+        struct HYD_node *newlist = NULL, *tail = NULL;
+        struct HYD_node *next;
+        int node_id = 0;
+        for (node = HYD_server_info.node_list; node; node = next) {
+            next = node->next;
+            if (MPL_host_is_local(node->hostname)) {
+                MPL_free(node->hostname);
+                MPL_free(node->user);
+                MPL_free(node->local_binding);
+                MPL_free(node);
+            } else {
+                node->next = NULL;
+                if (newlist == NULL) {
+                    assert(tail == NULL);
+                    newlist = node;
+                } else {
+                    tail->next = node;
+                }
+                tail = node;
+                node->node_id = node_id++;
+            }
+        }
 
-        /* create a dummy executable */
-        status = HYDU_alloc_exec(&HYD_uii_mpx_exec_list);
-        HYDU_ERR_POP(status, "unable to allocate exec\n");
-        HYD_uii_mpx_exec_list->appnum = 0;
+        if (newlist == NULL) {
+            status = HYD_INVALID_PARAM;
+            HYDU_ERR_POP(status, "no nodes available to launch processes\n");
+        } else {
+            HYD_server_info.node_list = newlist;
+        }
     }
 
     if (HYD_server_info.user_global.debug)
