@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2017 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "hydra.h"
@@ -425,7 +424,7 @@ static HYD_status launch_processes(void)
 #define MAPPING_EXPECT_C(_c, _e) do { if (*(_c) != _e) MAPPING_PARSE_ERROR(); } while (0)
 #define MAPPING_EXPECT_AND_SKIP_C(_c, _e) do { MAPPING_EXPECT_C(_c, _e); ++c; } while (0)
 /* give an error iff the first |_m| characters of the string _s are equal to _e */
-#define MAPPING_EXPECT_S(_s, _e) (MPL_strncmp(_s, _e, strlen(_e)) == 0 && !MAPPING_ISIDENT((_s)[strlen(_e)]))
+#define MAPPING_EXPECT_S(_s, _e) (strncmp(_s, _e, strlen(_e)) == 0 && !MAPPING_ISIDENT((_s)[strlen(_e)]))
 
 typedef enum {
     UNKNOWN_MAPPING = -1,
@@ -806,11 +805,8 @@ int main(int argc, char **argv)
         MPL_free(hash);
     }
 
-    if (proxy_params.cwd)
-        MPL_free(proxy_params.cwd);
-
-    if (proxy_params.root.hostname)
-        MPL_free(proxy_params.root.hostname);
+    MPL_free(proxy_params.cwd);
+    MPL_free(proxy_params.root.hostname);
 
     HYD_ASSERT(proxy_params.immediate.proxy.fd_control_hash == NULL, status);
     HYD_ASSERT(proxy_params.immediate.proxy.fd_stdin_hash == NULL, status);
@@ -818,15 +814,12 @@ int main(int argc, char **argv)
     HYD_ASSERT(proxy_params.immediate.proxy.fd_stderr_hash == NULL, status);
     HYD_ASSERT(proxy_params.immediate.proxy.pid_hash == NULL, status);
 
-    if (proxy_params.immediate.proxy.block_start)
-        MPL_free(proxy_params.immediate.proxy.block_start);
-    if (proxy_params.immediate.proxy.block_size)
-        MPL_free(proxy_params.immediate.proxy.block_size);
+    MPL_free(proxy_params.immediate.proxy.block_start);
+    MPL_free(proxy_params.immediate.proxy.block_size);
 
     if (proxy_params.immediate.proxy.num_children) {
         for (i = 0; i < proxy_params.immediate.proxy.num_children; i++)
-            if (proxy_params.immediate.proxy.kvcache[i])
-                MPL_free(proxy_params.immediate.proxy.kvcache[i]);
+            MPL_free(proxy_params.immediate.proxy.kvcache[i]);
         MPL_free(proxy_params.immediate.proxy.kvcache);
         MPL_free(proxy_params.immediate.proxy.kvcache_size);
         MPL_free(proxy_params.immediate.proxy.kvcache_num_blocks);
@@ -837,32 +830,27 @@ int main(int argc, char **argv)
     HYD_ASSERT(proxy_params.immediate.process.fd_pmi_hash == NULL, status);
     HYD_ASSERT(proxy_params.immediate.process.pid_hash == NULL, status);
 
-    if (proxy_params.immediate.process.pmi_id)
-        MPL_free(proxy_params.immediate.process.pmi_id);
+    MPL_free(proxy_params.immediate.process.pmi_id);
 
-    if (proxy_params.all.kvsname)
-        MPL_free(proxy_params.all.kvsname);
+    MPL_free(proxy_params.all.kvsname);
     if (proxy_params.all.complete_exec_list)
         HYD_exec_free_list(proxy_params.all.complete_exec_list);
 
-    if (proxy_params.all.primary_env.serial_buf)
-        MPL_free(proxy_params.all.primary_env.serial_buf);
+    MPL_free(proxy_params.all.primary_env.serial_buf);
     if (proxy_params.all.primary_env.argc) {
         for (i = 0; i < proxy_params.all.primary_env.argc; i++)
             MPL_free(proxy_params.all.primary_env.argv[i]);
         MPL_free(proxy_params.all.primary_env.argv);
     }
 
-    if (proxy_params.all.secondary_env.serial_buf)
-        MPL_free(proxy_params.all.secondary_env.serial_buf);
+    MPL_free(proxy_params.all.secondary_env.serial_buf);
     if (proxy_params.all.secondary_env.argc) {
         for (i = 0; i < proxy_params.all.secondary_env.argc; i++)
             MPL_free(proxy_params.all.secondary_env.argv[i]);
         MPL_free(proxy_params.all.secondary_env.argv);
     }
 
-    if (proxy_params.all.pmi_process_mapping)
-        MPL_free(proxy_params.all.pmi_process_mapping);
+    MPL_free(proxy_params.all.pmi_process_mapping);
 
   fn_exit:
     return status;
